<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"
    xmlns:as="http://www.oasis-open.org/committees/cam"
    xmlns:camed="http://jcam.org.uk/editor">
    <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
        Authors: David RR Webber, May, 2008 -->
    <!--
        Purpose  : Parse CAM cxf file and create NIEM cross-reference spreadsheet.  Lookup dictionary of NIEM properties definitions.
        Original : 07/07/2008
        Initials : drrw
        Description : Parse NIEM library into XML lookup format for use in building wantlist cross reference spreadsheets.
       
        Updated  - Inits - Change
        07/07/08   drrw   Initial release 
        07/29/08   drrw   Refine for EDXL handling and add annotations
    -->
    <xsl:param name="NIEMrepository">NIEM-repository.xml</xsl:param>
    <xsl:param name="annotations">true</xsl:param>
    <xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>
    
    <xsl:variable name="version">1.01</xsl:variable>
    <xsl:variable name="NIEMproperties" select="document($NIEMrepository)"/>
    
    <xsl:template match="/">
        <NIEMxref>
            
            <xsl:comment>
                Cross reference generator, version <xsl:value-of select="$version"/>
                Using repository - <xsl:value-of select="$NIEMrepository"/>
                Generated on - <xsl:value-of select="current-dateTime()"></xsl:value-of>
                <xsl:value-of select="'&#xD; &#xD;' "/>
            </xsl:comment>
            
            <xsl:apply-templates select="as:CAM/as:AssemblyStructure/as:Structure/child::*"/>
        </NIEMxref>
    </xsl:template>
    
    <xsl:template match="as:Element">
        <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="this" select="@name"/>
        <xsl:variable name="itemCount" select="count(//as:Element[@name=$this])"/>
        <xsl:variable name="itemPath" select="//as:Element[@name=$this][1]/@path"/>
        <xsl:variable name="niem" select="$NIEMproperties//Property[upper-case(@NIEM)= upper-case($itemName)][1]"/>
        <xsl:variable name="anno">
            <xsl:choose>
                <xsl:when test="camed:annotation/camed:documentation[@type='Definition']"><xsl:value-of select="camed:annotation/camed:documentation[@type='Definition']/text()"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="replace(as:Rule/as:constraint/as:annotation/as:documentation[@type='Definition']/text(),'\|','; ')"></xsl:value-of></xsl:otherwise>
            </xsl:choose> 
        </xsl:variable> 
        
        <xsl:if test="$itemCount=1 or($itemPath[1]=@path)">
        <xsl:choose>
            <xsl:when test="$annotations='true'"><crossref source="{@name}" NIEM="{$niem//@NIEM}" type="{$niem//@type}" definition="{$niem//@definition}" annotation="{$anno}"/></xsl:when>
            <xsl:otherwise><crossref source="{@name}" NIEM="{$niem//@NIEM}" type="{$niem//@type}" definition="{$niem//@definition}"/></xsl:otherwise>
        </xsl:choose>
        </xsl:if>
        
        <xsl:apply-templates select="as:Attribute"/>
        <xsl:apply-templates select="as:Element"/>
    </xsl:template>
    
    <xsl:template match="as:Attribute">
        <xsl:variable name="itemName" select="if(contains(@name,':')) then substring-after(@name,':') else @name"/>
        <xsl:variable name="this" select="@name"/>
        <xsl:variable name="itemCount" select="count(//as:Attribute[@name=$this])"/>
        <xsl:variable name="itemPath" select="//as:Attribute[@name=$this][1]/@path"/>
        
        <xsl:variable name="niem" select="$NIEMproperties//Property[upper-case(@NIEM)= upper-case($itemName)][1]"/>
        <xsl:variable name="anno">
            <xsl:choose>
                <xsl:when test="camed:annotation/camed:documentation[@type='Definition']"><xsl:value-of select="camed:annotation/camed:documentation[@type='Definition']/text()"/></xsl:when>
                <xsl:otherwise><xsl:value-of select="replace(as:Rule/as:constraint/as:annotation/as:documentation[@type='Definition']/text(),'\|','; ')"></xsl:value-of></xsl:otherwise>
            </xsl:choose> 
        </xsl:variable>
        <xsl:if test="$itemCount=1 or($itemPath[1]=@path)">
        <xsl:choose>
            <xsl:when test="$annotations='true'"><crossref source="{@name}" NIEM="{$niem//@NIEM}" type="{$niem//@type}" definition="{$niem//@definition}" annotation="{$anno}"/></xsl:when>
            <xsl:otherwise><crossref source="{@name}" NIEM="{$niem//@NIEM}" type="{$niem//@type}" definition="{$niem//@definition}"/></xsl:otherwise>
       </xsl:choose>
       </xsl:if>
    </xsl:template>
    
    <xsl:template match="node()"></xsl:template>
    
</xsl:stylesheet>
