<?xml version="1.0"?>
<!DOCTYPE stylesheet [
<!ENTITY copy  "&#169;" >
<!ENTITY nbsp "&#160;"> 
<!ENTITY gte "&gt;=">
<!ENTITY lte "&lt;="> 
]>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
	xmlns:w2e="http://thisSheet" xmlns:xsd="http://www.w3.org/2001/XMLSchema"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:w_pick="http://jcam.org.uk/editor/picklist/1.0"
	exclude-result-prefixes="xsd xs w2e w_pick">
	<!-- CAM kit utility program -->
	<!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
		Authors: Martin ME Roberts, March, 2008 -->
	<!--
		Purpose  : Parse XSD schema into CAM template XML structure components.
		Creates XML instance to be used as Structure in CAM template editor.
		Original : 03/21/2008
		Initials : mmer
		CAM ver    : 1.1 
		Description : Part of a set of tools to ease alignment of XSD based validations and documentation
		and CAM templates. Generates an XML instance in CAM structure format
		Updated  - Inits - Change
		03/21/08   mmer   Initial release
		03/22/08   drrw    Added simple type handling - enumeration value
		03/23/08   drrw    Correct handling of ref and base lookups by nameoutp without namespace
		03/23/08   drrw    Resolve enclosing type 
		03/24/08   drrw    Corrected attributes with @ref
		03/24/08   drrw    Added attributeGroup handling
		03/24/08   drrw    Prevented type outputting twice when duplicate type definitions
		03/24/08   drrw    Resolve xsd:token value referencing
		03/24/08   mmer  Added option for CAM output or not
		03/25/08   drrw    Updated namespace handling - resolve to use xsd namespaces
		03/25/08   drrw    Added new parameter for rules, inline and anno generation
		03/26/08   drrw    Add check for people who have redefined base types to prevent looping
		03/26/08   drrw    Prevent failure if enumerated type occurs more than once with same name
		03/27/08   drrw    Tweak the check on enumerated type names to process element sequences
		03/27/08   drrw    Clean up use of // referencing using match[1] technique
		03/28/08   drrw    Added include and logic for CAM rule generation
		03/31/08   mmer  Create inline output processing
		03/31/08   drrw    Added counter of attribute items to key off.
		01/04/08   mmer   Refactored to create a complete template with full rules for elements and attributes
		01/04/08   drrw    Tweak CAM description and ID value assigning.
		01/04/08   drrw    Reconcile changes in xsd2example with same logic in here.
		02/04/08   drrw    Reconcile fixes in xsd2example with same logic in here.
		03/04/08   drrw    Add logic to handle enumerations and restrictions.
		04/04/08   mk/mmer Fix for namespace outputting on root element (thanks to Michael Kay for the technique).
		04/04/08   drrw    Complete logic on enumeration handling for attributes.
		04/04/08   drrw    Add inlining of attributes within regular attributes.
		04/06/08   drrw    Element with simpleType and enumerations.
		04/06/08   drrw    Handle extension sequences of elements with types
		04/06/08   drrw    Carry forward attributes on referenced element types
		04/07/08   drrw    Handle substitutionGroups
		04/08/08   drrw    Resolve attributes on substitution items
		04/08/08   drrw    Resolve enumeration on elements with referenced base types
		04/09/08   drrw    Handle schema with no namespace declarations
		04/12/08   drrw    Remove duplicate rules
		04/14/08   drrw    Element with simpletype restriction example content
		04/14/08   drrw    Check enumeration values exist before outputting rule
		04/16/08   mmer  Handle attributes on complexType 
		04/16/08   drrw    Resolve sequence within sequence 
		04/16/08   drrw    Add simpleType for element
		04/16/08   drrw    Output inline rules for attributes with @ref
		04/16/08   drrw    Extend attribute rule inline to all areas where attributes output
		04/17/08   drrw    Comment out non-essential debugging messages
		04/21/08   drrw    Refine substring handling for [1] instance only
		04/21/08   drrw    Ambiguous attribute match
		04/30/08   drrw    Match type references by namespace
		05/02/08   drrw    Support type referencing a la UBL style
		05/04/08   drrw    Add xsd:all and xsd:group support
		05/04/08   drrw    Fix attributeGroup handling
		05/04/08   drrw    xsd:list type handling
		05/05/08   drrw    Consolidate changes from xsd2example
		05/06/08   drrw    Fix rule outputting for resolved element type referencing
		05/08/08   drrw    Default namespace handling within content structure
		05/09/08   drrw    Correct restriction enumeration handling
		05/09/08   drrw    simpleType restriction referencing via chkType
		05/12/08   drrw    handle sequence within choice
		05/13/08   drrw    resolvedType handling refinement
		05/14/08   drrw    add time stamp to trace messages 
		05/17/08   drrw    add logic for annotation outputting
		05/19/08   drrw    refine annotation outputting
		05/19/08   drrw    add exclude anno prefixing
		05/21/08   drrw    anno generation inline and embedded XML content 
		05/21/08   drrw    add enumeration anno generation inline
		05/22/08   drrw    handle xsd:any and xsd:anyAttribute
		05/23/08   drrw    fix @ref facets extraction
		05/26/08   drrw    anno for simpleType findType lookup and ns aware matching
		05/26/08   drrw    use tokenized exclusion list for ns
		06/16/08   drrw    minor change to message level on exclusion anno type
		06/20/08   drrw    Add handling of xsd:minLength and xsd:maxLength
		06/28/08   drrw    Add recursive @ref handling with annotation flag
		07/10/08   drrw    Support wantlist in initial structure generation
		07/10/08   drrw    Minimize rules for attributeGroup
		07/14/08   drrw    Flag if XSD default namespace declared
		07/17/08   mmer  LogLevel as removed
		07/18/08   mmer  wantlist check for input test put in.
		07/19/08   drrw    fix loading of element exclude rules.
		07/19/08   drrw    wantlist file path handling
		07/21/08   drrw    wantlist rule matching on Group bug fix
		07/23/08   mmer  outputElementWithType was not selecting correct complexType with ancestor::
		07/24/08   drrw    w3e namespace fix
		07/27/08   drrw    Handle the sequence xsd:choice double definition kludge that is in XSD syntax
		07/27/08   drrw    Handle multiple sequence in xsd:choice kludge
		08/02/08   drrw    Handle DoD complexType with immediate xsd:choice without xsd:sequence first
		08/02/08   drrw    Handle DoD base type use and restriction
		08/02/08   drrw    Accept xs:int or xsd:int as integer shorthand
		08/02/08   drrw    Add handling for  xsd:totalDigits xsd:fractionDigits
		08/06/08   drrw    Escape ' in enumeration value strings
		08/06/08   drrw    Replace non-UTF8 chars with equivalents in anno
		08/13/08   drrw    Adjust matched logic on simpleTypes for no namespace schema
		08/15/08   drrw    Enumeration and UTF8
		09/14/08   drrw    UTF8 check of inline annotations
		09/22/08   drrw    Additional UTF character check - 7
		10/17/08   drrw    maxInclusive to setNumberRange mapping
		11/06/08   drrw    xsd:pattern regexp to setStringMask mapping
		11/07/08   drrw    xsd:length on xsd:string type restriction mapping
		11/09/08   drrw    align date mask with xsd:pattern
		12/11/08   drrw    align fixed attribute with fixed structure value
		12/13/08   drrw    Additional UTF8 character checks for & character definitions
		12/29/08   drrw    Change conditional paths to relative XPath target
		01/02/09   drrw    Support for xs:all
		01/02/09   drrw    Enhanced choice() XPath for non-namespaced items (contains)
		01/02/09   drrw    Support for xs:sequence within xs:sequence
		01/02/09   drrw    SimpleType and restriction handling
		01/04/09   drrw    Clean up extra namespace declarations
		01/11/09   drrw    xsd:pattern to mask handling improved
		02/03/09   drrw    ignore attributes with as: prefix (as:ns-prefix)
		02/09/09   drrw    fix attribute leakage issue
		02/14/09   drrw    extend non-UTF8 scanning in chkUTF8 
		02/19/09   drrw    choice() handling in complexType
		02/21/09   drrw     Support appinfo for recursive nodes
		03/18/09   drrw    Add Pattern A-H,J-N,P-Z for alpha strings support
		03/29/09   drrw    Improve error handling for invalid input file
		03/29/09   drrw    Resolve results prefixes issue
		04/26/09   drrw    Add xs:union support for multiple type definitions
		04/28/09   drrw    More UTF8 character issues Ã© = e,  and Ã… = Aue
		05/07/09   drrw    Correct support for xsd:group construct
		05/14/09   drrw    More UTF8 character issues Â· for quotes
		05/14/09   drrw    Add handling of union type for xsd:integer xsd:decimal xsd:float
		05/15/09   drrw    Fix data type matching for complextype @name same as element @name
		05/27/09   drrw    Support xsd version attribute on schema root element
		06/04/09   drrw    Support root element as '*' for ALL (where there is no root element)
		06/05/09   drrw    Add annotations of extension type references to permit tracking
		06/06/09   drrw    Add documentation details for substitutionGroup, base and type reference path linkages
		06/25/09   drrw    Fix annotation group details outputting to avoid issue with substitutionGroup name duplicates 
		06/25/09   drrw    Fix string-length for chkElementMatch[1] to avoid duplicates
		06/25/09   drrw    Remove duplicate reprocessing of annotations into extension after main template generation
		06/25/09   drrw    For annotations with only one node path - find where used in structure rules
		06/30/09   drrw    Regex xsd:pattern handling improvements and UK post code mask 
		07/01/09   drrw    Fix type xsd:pattern scanning for type= 
		07/01/09   drrw    Fix simpleType matching on type name 
		07/01/09   drrw    Add xsd:any handling on xsd:sequence on complexContent 
		07/02/09   drrw    Numeric mask repeat count tens fix
		08/18/09   drrw    setChoice for complexTypes returns /// in XPath - so replace with //
		08/31/09   drrw    Change namespace on picklist to http://jcam.org.uk/editor/picklist/1.0
		09/16/09   drrw    Support NIEM augmentation type structure constructs
		09/16/09   drrw    Corrected ref= namespace matching to type
		10/05/09   drrw    Allow types name range selection pairs A-Z, not just asterix
		12/24/09   drrw    UTF-8 handling for ç - Ã§
		02/26/09   drrw    all to All
		03/01/10   drrw    Progress diagnostic message output log level 1
		03/09/10   drrw    Refine handling of annotation details output
		03/09/10   drrw    Report abstract types
		03/09/10   drrw    For @ref in a complexType sequence - generate a marker annotation 
		03/12/10   drrw    Improve Extension type handling on attributes
		03/15/10   drrw    Improve Extension restriction rule handling
		03/16/10   drrw    Decimal handling for vaguries of fraction digits in models
		04/04/10   drrw    Progress messages outputting
		06/24/10   drrw    Handle xsd:union of date types
		06/24/10   drrw    Handle simpleType with restriction and base
		06/24/10   drrw    Decimal elements with zero fractions for non-numeric content
		07/12/10   drrw    Element key / keyref properties to annotation Key / KeyRef mapping
		07/14/10   drrw    Retain "recursive" annotation into template from XSD
		07/14/10   drrw    When ingesting XSD from library such as NIEM - then retain the original type definition in annotation
		07/14/10   drrw    Improve performance of duplicate rules removal processing step by reducing look ahead on action count
		07/27/10   drrw    Duplicate type definition in annotation suppressed; base xsd: type skipping string, integer, et al.
		07/26/10   drrw    Fix handling of complexTypes and elements in "*" mode
		07/30/10   drrw    Discover how to deduce the default target namespace prefix
		07/30/10   drrw    Add target namespace prefix to "*" mode element lookups
		08/05/10   drrw    Add annotation for attributeGroup
		08/10/10   drrw   Suppress base type attributeGroup extensions
		08/12/10   drrw   Insert recursive annotation note on recursive nodes
		08/30/10   drrw   Make output of restrictValues and lookup rules a parameter option 
		08/31/10   drrw   Optional output of lookup files
		09/04/10   drrw   Correct concat bug with name() of ancestors of attributeGroup
		09/05/10   drrw   Replace @ with attr- in lookup file name
		12/01/10   drrw   Handle union of base xsd types
		12/02/10   drrw   Change codelists extension from .lists to .cll
		01/05/11   drrw   Handling of lookup includes updated
		01/05/11   drrw   Output path for lookups location
		01/06/11   drrw   For any type put % any % in example content
		01/06/11   drrw   Add tempFilePath to dump file location
		01/23/11   drrw   Change .cll to .ccl and cllr to cclr - CAM code list
		01/24/11   drrw   Fix for simpleType name reference without namespace
		01/24/11   drrw   Non-UTF8 char for coulomb Âº
		02/19/11   drrw   Non-UTF8 char for following â€ sequence of chars
		02/19/11   drrw   Resolve xsd:element and xsd:complexType with same name issue but element has no @type
		02/19/11   drrw   Put imports / includes as annotations on structure root element
		05/06/11   drrw   Add multiple element selection feature for service components
		05/12/11   drrw   Enhance multiple element selection for complexTypes
		05/12/11   drrw   Handling of elements with empty xsd:sequence
		05/18/11   drrw   Multiple selection of complexType rootlevel components processing
		05/18/11   drrw   Added namespace handling to multiple selection also
		05/18/11   drrw   Fix conflicting setNumberRange() rules when item is integer and has xsd: restrictions as well
		05/18/11   drrw   Fix numeric restriction of attributes handling in inlined-to-cam-rules process
		05/25/11   drrw   Missing annotations on elements - duplicate name in XPath
		06/02/11   drrw   Handle choice with minOccurs and maxOccurs
		06/04/11   drrw   Set attributes optional by default
		06/09/11   drrw   Add SubstitutionGroup annotation note type
		06/15/11   drrw   Output %any% with no whitespace following
		07/11/11  drrw    Improve lookup lists filepath handling
		10/29/11  drrw    Fix handling of union with one type that is recursive
		01/04/12  drrw    Fix handling of type definition references with same name as element
		01/05/12  drrw    Fix handling of attributes with base types with namespaced type
		01/05/12  drrw    Improve setChoice logic for NIEM substitutionGroups
		03/14/12  drrw    Add handling for xsd:Group constructs
		03/14/12  drrw    Suppress error message about base XSD types in simpleType check
		03/14/12  drrw    Add annotation of type Group for xsd:group items
		03/15/12  drrw    Fix missing namespace prefix on child element component outputting
		04/05/12  drrw    Refactor xsd:group handling and add annoations of xsd:group details
		04/05/12  drrw    Add nsURI to xsd import details as annotations on root element
		04/11/12  drrw    Change xsd:group to include choice on elements but not group items
		04/12/12  drrw    Modify RemoveDups to inspect inside choice for elements
		06/01/12  drrw    Update xsd:pattern handling for extended UK postcode and BFPO patterns
		06/12/12  drrw    Adjust regexp for [0-9A-Za-z'\.\-_] patterns and optional
		06/23/12  drrw    Improve simpleType with restriction handling
		06/23/12  drrw    UTF-8 handling for quotes
		06/23/12  drrw    Non-UTF-8 in setStringMask() characters for restricted string type GDSC
		06/23/12  drrw    Default mask for number value type with setNumberRange
		06/24/12  drrw    Fix duplicate elements within substitutionGroup
		06/29/12  drrw    Fix complexType with xsd:choice as first component processing
		10/03/12  drrw    Fix for STAR BOD where they clone a complexType at parent level from inside common components
		10/05/12  drrw    Catch attributes as well with new complexType handing logic and xsd:choice
		11/15/12  drrw    Support base enumeration usage - xsd:complexType/xsd:simpleContent/xsd:extension
		11/16/12  drrw    Use of pattern on string and totalDigits on short items
		11/16/12  drrw    Use of nonNegativeInteger and totalDigits
		11/27/12  drrw    Fix simpleType with union and list for enumeration values 
		01/27/13  drrw    AppInfo annotation handling enhancement
		02/08/13  drrw    Enhance concat() processing for schema syntax errors
		03/21/13  drrw    Update AppInfo annotation handling for URN 
		03/25/13  drrw    Change date rules to check for period instead of length    
	-->

	<xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>
	<xsl:output name="annoContent" method="xml" indent="yes" omit-xml-declaration="yes"/>
	<xsl:output name="lookupContent" method="xml" indent="yes" omit-xml-declaration="yes"/>

	<xsl:param name="elementName"/>
	<xsl:param name="complexTypeName"/>
	<xsl:param name="logLevel">1</xsl:param>
	<xsl:param name="CAM">true</xsl:param>
	<xsl:param name="CAMmode">diagnostic, rules, anno</xsl:param>
	<xsl:param name="annoFile">default.anno</xsl:param>
	<xsl:param name="annoExclude">xsd:|xlink:|xpil:|axal:|nxnl:|xnl:|xal:</xsl:param>
	<xsl:param name="annoMode">inline</xsl:param>
	<xsl:param name="annoContent">text</xsl:param>
	<!-- values: text or all -->
	<xsl:param name="wantlist"/>
	<xsl:param name="useLookups"/>
	<!-- values: no, yes, all -->
	<xsl:param name="tempFilePath"/>
	<!-- xsl:param name="anonymousNS"></xsl:param -->

	<!-- values are: structure, rules, inline, anno, reference, diagnostics -->

	<xsl:variable name="allRootLevelTypeDefinitions">AllRootLevelTypeDefinitions</xsl:variable>
	<xsl:variable name="serviceComponentDefinitions">ServiceComponentDefinitions</xsl:variable>
	<xsl:variable name="elementNameValue"
		select="if(ends-with($elementName,',')) then substring-before($elementName,',') else $elementName"/>
	<!-- determine root element name passed as parameter -->
	<xsl:variable name="elementNameList" select="count(tokenize($elementName,',')) - 1"/>
	<!-- determine if multiple element names passed as parameter -->
<xsl:variable name="newline">
<xsl:text>
</xsl:text>
</xsl:variable>

	<xsl:variable name="tempFilePathStr" select="replace($tempFilePath,'\\','/')"/>
	<xsl:variable name="lookupsFilePath">
		<xsl:if test="string-length($tempFilePathStr) &gt; 0">
			<xsl:if test="not(starts-with($tempFilePathStr,'file:/'))">file:/</xsl:if>
			<xsl:value-of select="$tempFilePathStr"/>
			<xsl:if test="not(ends-with($tempFilePathStr,'/'))">/</xsl:if>
		</xsl:if>
	</xsl:variable>

	<xsl:variable name="wantlistFile">
		<xsl:choose>
			<xsl:when test="contains($wantlist,'/')"><xsl:value-of select="if( starts-with($wantlist,'file:')) then translate($wantlist,'\','/') else concat('file:///',translate($wantlist,'\','/'))"/></xsl:when>
			<xsl:when test="contains($wantlist,'\')"><xsl:value-of select="if( starts-with($wantlist,'file:')) then translate($wantlist,'\','/') else concat('file:///',translate($wantlist,'\','/'))"/></xsl:when>
			<xsl:otherwise><xsl:value-of select="$wantlist"/></xsl:otherwise>
		</xsl:choose>
	</xsl:variable>

	<xsl:variable name="annoExclTokens" select="tokenize($annoExclude,'\|')"/>
	<xsl:variable name="defaultNS"
		select="if(count(/xsd:schema) &gt; 0) then namespace-uri-for-prefix('',/xsd:schema) else '*ERROR*'"/>

	<xsl:variable name="utf8-1">’</xsl:variable>
	<xsl:variable name="utf8-2">“</xsl:variable>
	<xsl:variable name="utf8-3">”</xsl:variable>
	<xsl:variable name="utf8-4">`</xsl:variable>
	<xsl:variable name="utf8-5">¦</xsl:variable>
	<xsl:variable name="utf8-6">â</xsl:variable>
	<xsl:variable name="utf8-7">ƒ</xsl:variable>
	<xsl:variable name="utf8-8">€</xsl:variable>
	<xsl:variable name="utf8-9">¡</xsl:variable>
	<xsl:variable name="utf8-10">¾</xsl:variable>
	<xsl:variable name="utf8-11">¿</xsl:variable>
	<xsl:variable name="utf8-12">Ã</xsl:variable>
	<xsl:variable name="utf8-13">Â</xsl:variable>
	<xsl:variable name="utf8-14">™</xsl:variable>
	<xsl:variable name="utf8-15">œ</xsl:variable>
	<xsl:variable name="utf8-16"></xsl:variable>
	<xsl:variable name="utf8-17">¢</xsl:variable>
	<xsl:variable name="utf8-18">•</xsl:variable>
	<xsl:variable name="utf8-19">–</xsl:variable>
	<xsl:variable name="utf8-30">é</xsl:variable>
	<xsl:variable name="utf8-31">Å</xsl:variable>
	<xsl:variable name="utf8-32">©</xsl:variable>
	<xsl:variable name="utf8-33" select="'…'"/>
	<xsl:variable name="utf8-34">·</xsl:variable>
	<xsl:variable name="utf8-35" select="'Ã©'"/>
	<xsl:variable name="utf8-36">Ã§</xsl:variable>
	<xsl:variable name="utf8-37">ç</xsl:variable>
	<xsl:variable name="utf8-38" select="'º'"/>
	<xsl:variable name="utf8-r0" select="'â€'"/>
	<xsl:variable name="utf8-r1" select="'â€™'"/>
	<xsl:variable name="utf8-r2" select="'â€“'"/>
	<xsl:variable name="utf8-r3" select="'â€˜'"/>
	<xsl:variable name="utf8-r4" select="'â€œ'"/>
	<xsl:variable name="utf8-r5" select="'â€'"/>
	<xsl:variable name="utf8-r6" select="'â€¦'"/>
	<xsl:variable name="utf8-r7" select="'â€”'"/>
	<xsl:variable name="utf8-r8" select="'â€¢'"/>
	<xsl:variable name="utf8-r9">§</xsl:variable>
	<xsl:variable name="utf8-r10" select="' '"/>
	<xsl:variable name="utf8-r11" select="'·'"/>
	<xsl:variable name="utf8-r12">‘</xsl:variable>
	<!-- â€™ ’ &rsquo;
	â€“ – &ndash;
	â€˜ ‘ &lsquo;
	â€œ “ &ldquo;
	â€ ” &rdquo;
	â€¦ . period – wierdly always at the end of “etc.”
	â€” — &mdash; 1 occurrence.
	â€¢ · &middot; -->
	<xsl:variable name="period">.</xsl:variable>
	<xsl:variable name="dash">-</xsl:variable>
	<xsl:variable name="quote">'</xsl:variable>
	<xsl:variable name="quotes">"</xsl:variable>
	<xsl:variable name="space"> </xsl:variable>
	<xsl:variable name="twiddle">{!}</xsl:variable>
	<xsl:variable name="dot">'</xsl:variable>

	<xsl:variable name="GDSC-restricts">A-Za-z0-9\s~!&quot;@#$%&amp;'\(\)\*\+,\-\./:;&lt;=&gt;\?\[\\\]_\{\}\^&#xa3;&#x20ac;</xsl:variable>
	<xsl:variable name="GDSC-CAM">X'_''^£€]*'</xsl:variable>
	<xsl:variable name="GDSC-Mask">X?*</xsl:variable>
	<xsl:variable name="UK-postcode">[A-Z]{1,2}[0-9R][0-9A-Z]? [0-9][A-Z-[CIKMOV]]{2}</xsl:variable>
	<xsl:variable name="UK-postcode-1">[A-Z]{1,2}[0-9R][0-9A-Z]? [0-9][A-Z]{2}</xsl:variable>
	<xsl:variable name="UK-postcode-BFPO">[B][F][P][O] [1-9][0-9]{0,2}</xsl:variable>
	<xsl:variable name="TelephoneNumber">\+?[0-9\(\)\-\s]{1,35}</xsl:variable>
	<xsl:variable name="PAON-postcode">[0-9 ]{4}[A-Z ]([0-9 ]{4}[A-Z ][a-zA-Z0-9:;&amp;lt;=&amp;gt;\?@%&amp;'\(\)\*\+,\-\. ]{0,90})?</xsl:variable>
	<xsl:variable name="SAON-postcode">{4}[A-Z ]([0-9 ]{4}[A-Z ][a-zA-Z0-9:;</xsl:variable>
	<!-- regex for UK postal code -->
	<xsl:variable name="UK-postcode-CAM">U^##^' '0UU</xsl:variable>
	<xsl:variable name="UK-postcode-BFPO-CAM">'B''F''P''O'^####</xsl:variable>
	<xsl:variable name="ANY-postcode-CAM">#4U#4UX90</xsl:variable>
	<xsl:variable name="TelephoneCAM">X35</xsl:variable>
	<!-- CAM string mask for UK postal code -->

	<xsl:variable name="baseTypes">string|integer|decimal|float|dateTime|date|time|boolean|URI|token|base64Binary|nonNegativeInteger|</xsl:variable>
	<!-- used to check annotation type outputting -->
	<xsl:variable name="baseDataTypes">string|boolean|datetime|date|time|dateTime|decimal|float|double|integer|nonNegativeInteger|unsignedInt|unsignedLong|token|NMTOKEN|NMTOKENS|language|list|anyURI|base64Binary|any|anyAttribute|idRef|</xsl:variable>

	<xsl:variable name="xsdTypeAttributes">xsd:minLength xsd:maxLength xs:minLength xs:maxLength xs:totalDigits xsd:totalDigits xs:fractionDigits xsd:fractionDigits xsd:maxInclusive xs:maxInclusive xs:minInclusive xsd:minInclusive xsd:pattern xs:pattern xsd:length xs:length</xsl:variable>

	<xsl:variable name="ns-prefix">as:ns-prefix</xsl:variable>

	<xsl:variable name="version">2.11</xsl:variable>
	<xsl:variable name="excludeAttrRules" select="if (string-length($wantlistFile) &gt; 0) then document($wantlistFile)//Attribute[@w_pick:isXclude='1'] else ''"/>
	<xsl:variable name="excludeElemRules" select="if (string-length($wantlistFile) &gt; 0) then document($wantlistFile)//Element[@w_pick:isXclude='1'] else ''"/>
	<xsl:variable name="excludeGrpRules" select="if (string-length($wantlistFile) &gt; 0) then document($wantlistFile)//Group[@w_pick:isXclude='1']/@w_pick:n else ''"/>
	<xsl:variable name="emptyList"><attribute n="#any"/></xsl:variable>
	<!-- remove namespaces from exclude paths using replace and regex on '\\?*:' -->
	<xsl:variable name="excludeAttrPaths">
		<xsl:if test="string-length($wantlist) > 0">
			<xsl:for-each select="$excludeAttrRules" exclude-result-prefixes="#all">
				<xsl:variable name="temp" select="replace(@w_pick:n,'\\[a-z,A-Z,0-9,_]*[:]','\\')"/>
				<xsl:variable name="chk" select="concat(substring-before($temp,':'),':')"/>
				<Attribute n="{if(starts-with($temp,$chk) and not(contains($chk,'@'))) then substring-after($temp,':') else $temp}"/>
			</xsl:for-each>
		</xsl:if>
	</xsl:variable>
	<xsl:variable name="excludeElemPaths">
		<xsl:if test="string-length($wantlist) > 0">
			<xsl:for-each select="$excludeElemRules" exclude-result-prefixes="#all">
				<xsl:variable name="temp" select="replace(@w_pick:n,'\\[a-z,A-Z,0-9,_]*[:]','\\')"/>
				<Element n="{if(contains($temp,':')) then substring-after($temp,':') else $temp}"/>
			</xsl:for-each>
		</xsl:if>
	</xsl:variable>
	<xsl:variable name="excludeGrpPaths">
		<xsl:if test="string-length($wantlist) > 0">
			<xsl:for-each select="$excludeGrpRules" exclude-result-prefixes="#all">
				<xsl:variable name="temp" select="replace(.,'\\[a-z,A-Z,0-9,_]*[:]','\\')"/>
				<Group n="{if(contains($temp,':')) then substring-after($temp,':') else $temp}"/>
			</xsl:for-each>
		</xsl:if>
	</xsl:variable>

	<xsl:variable name="rootVersion" select="/xs:schema/@version"/>

	<xsl:include href="cam-utils.xsl"/>
	<xsl:include href="inlined-to-cam-rules.xsl"/>

	<xsl:variable name="namespaces" select="/xs:schema/namespace::*"/>

	<xsl:variable name="targetNsPrefix">
		<xsl:variable name="nsPrefixURI" select="/xs:schema/@targetNamespace"/>
		<xsl:for-each select="$namespaces">
			<!-- xsl:message> - <xsl:value-of select=" namespace-uri-for-prefix(name(.),/xs:schema)"/> | <xsl:value-of select="name(.)"/></xsl:message -->
			<xsl:if test="namespace-uri-for-prefix(name(.),/xs:schema)=$nsPrefixURI">	<xsl:value-of select="name(.)"/></xsl:if>
		</xsl:for-each>
	</xsl:variable>

	<xsl:variable name="entireXSDcollection" select="/"/>

	<xsl:variable name="abstractElements" select="//xsd:element[@abstract]"/>

	<xsl:template match="/">
		<xsl:if test="$defaultNS='*ERROR*'"><xsl:message terminate="yes">ERROR: Input file is not a valid XSD schema!</xsl:message></xsl:if>
		<xsl:variable name="element-to-process">
			<xsl:choose>
				<xsl:when test="string-length($elementNameValue) = 0"><xsl:value-of select="xsd:schema/xsd:element[1]/@name"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="$elementNameValue"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('beginning processing - ',format-dateTime(current-dateTime() ,'[h01]:[m01]:[s01].[f01]')),2)"/ -->
		<xsl:message>xsd2cam - <xsl:value-of select="$version"/> - <xsl:value-of select="format-dateTime(current-dateTime() ,'[h01]:[m01]:[s01].[f01]')"/></xsl:message>
		<xsl:message>components <xsl:value-of select="if($elementNameList &gt; 1) then tokenize($elementName,',') else $elementNameValue"/> selection</xsl:message>
		<xsl:message>namespaces: <xsl:for-each select="$namespaces"><xsl:value-of select="name(.)"/> - <xsl:value-of select="namespace-uri-for-prefix(name(.),/xs:schema)"/> | </xsl:for-each></xsl:message>
		<xsl:if test="not($targetNsPrefix='')"><xsl:message>default ns prefix - <xsl:value-of select="$targetNsPrefix"/></xsl:message></xsl:if>
		<xsl:message>parameters - <xsl:value-of select="$CAMmode"/></xsl:message>
		<xsl:message>anno exclusions - <xsl:value-of select="$annoExclude"/></xsl:message>
		<xsl:message>anno content - <xsl:value-of select="$annoContent"/></xsl:message>
		<xsl:message>anno mode - <xsl:value-of select="$annoMode"/></xsl:message>
		<xsl:message>annoFile - <xsl:value-of select="$annoFile"/></xsl:message>
		<xsl:message>lookups - <xsl:value-of select="$useLookups"/></xsl:message>
		<xsl:message>tempFilePath - <xsl:value-of select="$tempFilePath"/></xsl:message>

		<xsl:if test="contains($CAMmode,'dump')">
			<xsl:variable name="dumpFilePath">
				<xsl:choose>
					<xsl:when test="string-length($lookupsFilePath) &gt; 3">
						<xsl:variable name="pathParts" select="if(contains($lookupsFilePath,'\')) then tokenize($lookupsFilePath,'\\') else tokenize($lookupsFilePath,'/')"/>
						<!-- xsl:message>Path parts [<xsl:value-of select="$pathParts"/>] <xsl:value-of select="count($pathParts)"/></xsl:message -->
						<xsl:for-each select="$pathParts"><xsl:if test="position() &lt; count($pathParts)"><xsl:value-of select="concat(.,'/')"/></xsl:if></xsl:for-each>
						<xsl:value-of select="'dump-input.xml'"/>
					</xsl:when>
					<xsl:otherwise>dump-input.xml</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:message>dumping raw schema input to <xsl:value-of select="$dumpFilePath"/></xsl:message>
			<xsl:result-document exclude-result-prefixes="w2e as" format="annoContent" indent="yes"	href="{$dumpFilePath}"><xsl:copy-of select="$entireXSDcollection"/></xsl:result-document>
		</xsl:if>

		<xsl:message>building structure...</xsl:message>
		<xsl:for-each select="$annoExclTokens"><xsl:value-of select="w2e:message(concat('token =',.,' | '),4)"/></xsl:for-each>

		<xsl:variable name="doc">
			<xsl:choose>
				<xsl:when test="$elementNameValue='*'">
					<!-- no root element, so process all unique root level complexTypes -->
					<xsl:apply-templates select="/xsd:schema/xsd:complexType" mode="start"/>
				</xsl:when>
				<xsl:when test="$elementNameList &gt; 1">
					<xsl:message><xsl:value-of select="concat('selecting list of ',$elementNameList,' items')"/></xsl:message>
					<!-- range specified, so process all unique root level complexTypes in range -->
					<xsl:element name="{$serviceComponentDefinitions}">
						<xsl:for-each select="tokenize($elementName,',')">
							<xsl:variable name="element-to-process" select="."/>
							<xsl:if test="string-length($element-to-process) &gt; 0">
								<xsl:message><xsl:value-of select="concat('- processing ',$element-to-process)"/></xsl:message>
								<xsl:choose>
									<xsl:when test="count($entireXSDcollection/xsd:schema/xsd:element[@name=$element-to-process]) &gt; 0">
										<xsl:apply-templates select="$entireXSDcollection/xsd:schema/xsd:element[@name=$element-to-process]"/>
									</xsl:when>
									<xsl:otherwise>
										<xsl:choose>
											<xsl:when test="contains($element-to-process,':')">
												<xsl:variable name="ns-prefix" select="substring-before($element-to-process,':')"/>
												<xsl:variable name="ns-uri"><xsl:for-each select="$namespaces"><xsl:if test="name(.) = $ns-prefix"><xsl:value-of select="namespace-uri-for-prefix(name(.),/xs:schema)"/></xsl:if></xsl:for-each></xsl:variable>
												<xsl:element name="{$element-to-process}" namespace="{$ns-uri}"><xsl:apply-templates select="$entireXSDcollection/xsd:schema/xsd:complexType[@name=$element-to-process]"/></xsl:element>
											</xsl:when>
											<xsl:otherwise>
												<xsl:element name="{$element-to-process}"><xsl:apply-templates select="$entireXSDcollection/xsd:schema/xsd:complexType[@name=$element-to-process]"/></xsl:element>
											</xsl:otherwise>
										</xsl:choose>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:if>
						</xsl:for-each>
					</xsl:element>
				</xsl:when>
				<xsl:when test="contains($elementNameValue,'---')">
					<xsl:value-of select="w2e:message(concat('Applying range from: [',substring-before($elementNameValue,'---'),'][',substring-after($elementNameValue,'---'),']'),2)"/>
					<!-- range specified, so process all unique root level complexTypes in range -->
					<xsl:apply-templates select="/xsd:schema/xsd:complexType" mode="range">
						<xsl:with-param name="start" select="substring-before($elementNameValue,'---')"/>
						<xsl:with-param name="end" select="substring-after($elementNameValue,'---')"/>
					</xsl:apply-templates>
				</xsl:when>
				<xsl:when test="string-length($elementNameValue) &gt; 0">
					<xsl:apply-templates select="/xsd:schema/xsd:element[@name=$element-to-process]"/>
				</xsl:when>
				<xsl:when test="string-length($complexTypeName) &gt; 0">
					<xsl:apply-templates select="//xsd:complexType[@name=$complexTypeName][1]" mode="start"/>
				</xsl:when>
				<xsl:otherwise><xsl:apply-templates select="//xsd:element[@name=$element-to-process][1]"/></xsl:otherwise>
			</xsl:choose>
			<!-- Now find orphan element declarations -->
			<xsl:if test="$elementNameValue='*'">
				<xsl:message>resolving more root elements...</xsl:message>
				<xsl:variable name="rootElements" select="$entireXSDcollection/xsd:schema/xsd:element"/>
				<xsl:variable name="ElementReferences">
					<xsl:for-each select="$entireXSDcollection//xsd:sequence">
						<xsl:for-each select="xsd:element"><xsl:if test="@ref"><xsl:element name="ref"><xsl:value-of select="@ref"/></xsl:element></xsl:if>	</xsl:for-each>
					</xsl:for-each>
				</xsl:variable>
				<xsl:for-each select="$rootElements">
					<!-- check for complextypes that reference this and other conditions -->
					<xsl:variable name="typeName" select="@type"/>
					<xsl:variable name="typeNameNS" select="concat($targetNsPrefix,':',$typeName)"/>
					<xsl:variable name="thisName" select="@name"/>
					<xsl:variable name="thisNameNS" select="concat($targetNsPrefix,':',$thisName)"/>
					<xsl:choose>
						<xsl:when test="@substitutionGroup"/>
						<!-- has children -->
						<xsl:when test="xsd:key | xsd:keyref"/>
						<!-- has key elements -->
						<xsl:when test="@name=$elementNameValue"/>
						<!-- root element -->
						<xsl:when test="@name=$ElementReferences/ref"/>
						<xsl:when test="$thisNameNS=$ElementReferences/ref"/>
						<xsl:when test="@ref=$ElementReferences/ref"/>
						<xsl:when test="concat($targetNsPrefix,':',@ref)=$ElementReferences/ref"/>
						<xsl:when test="@type and($entireXSDcollection//xsd:complexType[@name=$typeName])"/>
						<!-- element has type of existing complexType -->
						<xsl:when test="not($targetNsPrefix='') and(@type and($entireXSDcollection//xsd:complexType[@name=$typeNameNS]))"/>
						<!-- element has type of existing complexType -->
						<xsl:when test="$entireXSDcollection//xsd:element[@substitutionGroup=$thisName]"/>
						<!-- referenced as substitutionGroup  -->
						<xsl:when test="not($targetNsPrefix='') and($entireXSDcollection//xsd:element[@substitutionGroup=$thisNameNS])"/>
						<!-- referenced as substitutionGroup  -->
						<xsl:when test="$entireXSDcollection//xsd:complexType/xsd:complexContent/xsd:extension[@base=$thisName]"/>
						<!-- referenced as extension -->
						<xsl:when test="not($targetNsPrefix='') and($entireXSDcollection//xsd:complexType/xsd:complexContent/xsd:extension[@base=$thisNameNS])"/>
						<!-- referenced as extension -->
						<xsl:otherwise>
							<xsl:message> - <xsl:value-of select="@name | @ref"/></xsl:message>
							<xsl:choose>
								<xsl:when test="@ref">
									<xsl:call-template name="outputElementWithRef">
										<xsl:with-param name="parent" select="/"/>
										<xsl:with-param name="ancestors" select="/"/>
									</xsl:call-template>
								</xsl:when>
								<xsl:when test="@type">
									<xsl:call-template name="outputElementWithType">
										<xsl:with-param name="ancestors" select="/"/>
										<xsl:with-param name="ref-name"/>
										<xsl:with-param name="ref-attributes" select="/"/>
										<xsl:with-param name="parent" select="/"/>
										<xsl:with-param name="parentType" select="@type"/>
									</xsl:call-template>
								</xsl:when>
								<xsl:otherwise>
									<xsl:call-template name="outputElementWithNoType">
										<xsl:with-param name="parent" select="/"/>
										<xsl:with-param name="ref-attributes" select="/"/>
										<xsl:with-param name="ancestors" select="/"/>
									</xsl:call-template>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
			</xsl:if>
		</xsl:variable>
		<xsl:variable name="nss">
			<xsl:call-template name="getNs">
				<xsl:with-param name="context" select="$doc"/>
			</xsl:call-template>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('resolving namespaces - ',format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].[f01]')),2)"/ -->
		<xsl:message>handling namespaces...</xsl:message>
		<xsl:variable name="cleanExample">
			<xsl:choose>
				<xsl:when test="$elementNameValue='*'">
					<xsl:element name="{$allRootLevelTypeDefinitions}">
						<xsl:apply-templates select="$doc" mode="ns">
							<xsl:with-param name="nss" select="$nss"/>
						</xsl:apply-templates>
					</xsl:element>
				</xsl:when>
				<xsl:when test="contains($elementNameValue,'---')">
					<xsl:element name="{$allRootLevelTypeDefinitions}">
						<xsl:apply-templates select="$doc" mode="ns">
							<xsl:with-param name="nss" select="$nss"/>
						</xsl:apply-templates>
					</xsl:element>
				</xsl:when>
				<xsl:otherwise>
					<xsl:apply-templates select="$doc" mode="ns">
						<xsl:with-param name="nss" select="$nss"/>
					</xsl:apply-templates>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('building rules - ',format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].[f1]')),2)"/ -->
		<xsl:message>building rules...</xsl:message>
		<xsl:variable name="rules">
			<xsl:call-template name="rules">
				<xsl:with-param name="root" select="$doc"/>
			</xsl:call-template>
		</xsl:variable>
		<!-- xsl:variable name="namespaces">
			<xsl:for-each select="//xsd:schema"><xsl:value-of select="concat(' xmlns:',generate-id(),'=&quot;',./@targetNamespace,'&quot;')" disable-output-escaping="yes"/></xsl:for-each>
			</xsl:variable -->
		<!-- xsl:value-of select="w2e:message(concat('removing duplicate rules - ',format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].[f01]')),2)"/ -->
		<xsl:message>removing duplicate rules...</xsl:message>
		<xsl:variable name="rules-unique">
			<xsl:call-template name="dedupRule">
				<xsl:with-param name="baseRules" select="$rules"/>
			</xsl:call-template>
		</xsl:variable>
		<!-- xsl:param name="annoMode">inline</xsl:param>
		<xsl:param name="annoContent">all</xsl:param -->
		<!-- xsl:value-of select="w2e:message(concat('building annotations - ',format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].[f1]')),2)"/ -->
		<xsl:message>building annotations...</xsl:message>

		<xsl:variable name="annotationAttrLines">
			<xsl:apply-templates select="$doc/*" mode="annoAttrLines"/>
		</xsl:variable>

		<xsl:variable name="extensions">
			<xsl:if test="$useLookups='yes' or($useLookups='all')">
				<!-- Check if lookup codelists are required -->
				<xsl:variable name="outputPath"/>
				<xsl:variable name="lookups">
					<xsl:for-each select="$rules//as:constraint">
						<xsl:if test="starts-with(@action,'restrictValues')">
							<xsl:variable name="actionStub"
								select="replace(replace(replace(substring-before(@action,','),'restrictValues','lookup'),'xsd:choice/',''),'xs:choice/','')"/>
							<xsl:variable name="actionXpath"
								select="substring-after($actionStub,'(')"/>
							<xsl:variable name="actionTarget" select="tokenize($actionXpath,'/')"/>
							<xsl:variable name="actionTargets" select="count($actionTarget)"/>
							<lookup
								name="{replace(replace(concat($actionTarget[position()=$actionTargets],'-',$actionTarget[position()=$actionTargets - 1]),'@','attr-'),':','-')}"
								values="{substring-after(@action,',')}"/>
						</xsl:if>
					</xsl:for-each>
				</xsl:variable>
				<xsl:variable name="uniqueLookups" select="distinct-values($lookups/lookup/@name)"/>
				<xsl:message>adding codelist includes...</xsl:message>
				<!-- Bug in editor prevents this working - allocateExtension not supporting .LookupLists type -->
				<!-- as:include>lookups-include.xml</as:include -->
				<!-- include is not supported either -->
				<!-- as:Extension xmlns:as="http://www.oasis-open.org/committees/cam" 
					name="uk.org.jcam.camed.extensions.LookupLists"  -->
				<xsl:for-each select="$uniqueLookups">
					<xsl:variable name="this" select="."/>
					<xsl:for-each select="$lookups/lookup[@name=$this][1]">
						<xsl:variable name="lookupFile"><xsl:if test="string-length($outputPath) &gt; 0"><xsl:if test="not(starts-with($outputPath,'file:/'))">file:/</xsl:if><xsl:value-of select="$outputPath"/><xsl:if test="not(ends-with($outputPath,'/'))">/</xsl:if></xsl:if>lookup-<xsl:value-of select="@name"/>.ccl</xsl:variable>
						<xsl:variable name="ruleEnumerations" select="tokenize(./@values,'\|')"/>
						<xsl:variable name="totalCodes" select="count($ruleEnumerations)"/>
						<xsl:if test="$useLookups='all' or($totalCodes &gt; 3)">
							<!-- xsl:message> - <xsl:value-of select="$lookupFile"/> (<xsl:value-of select="$totalCodes"/>)</xsl:message -->
							<!-- as:include><xsl:value-of select="$lookupFile"/></as:include -->
						</xsl:if>
					</xsl:for-each>
				</xsl:for-each>
				<!-- /as:Extension -->
			</xsl:if>
			<xsl:if test="contains($CAMmode,'anno')">
				<xsl:choose>
					<xsl:when test="contains($annoMode,'inline') or($annoMode='')">
						<as:Extension xmlns:as="http://www.oasis-open.org/committees/cam"
							xmlns:camed="http://jcam.org.uk/editor"
							name="uk.org.jcam.camed.extensions.StructureAnnotations">
							<xsl:apply-templates select="$cleanExample/*" mode="anno">
								<xsl:with-param name="schemaContent" select="/"/>
								<xsl:with-param name="rules" select="$rules-unique"/>
								<xsl:with-param name="annoAttrs" select="$annotationAttrLines"/>
							</xsl:apply-templates>
						</as:Extension>
					</xsl:when>
					<xsl:otherwise>
						<!-- Bug in editor prevents this working - same as codelist lookups -->
						<!-- as:include><xsl:value-of select="$annoFile"/></as:include -->
					</xsl:otherwise>
				</xsl:choose>
			</xsl:if>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('writing template - ',format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].[f01]')),2)"/ -->
		<xsl:message>writing template...</xsl:message>
		<xsl:variable name="results">
			<as:CAM CAMlevel="1" version="1.0">
				<!-- xsl:if test="$anonymousNS &gt; ' '">
		<xsl:namespace name="{$anonymousNS}"><xsl:value-of select="$defaultNS"/></xsl:namespace>
	</xsl:if -->
				<!-- xsl:comment>[<xsl:value-of select="$defaultNS"/>]</xsl:comment -->
				<as:Header>
					<as:Description>Generated for <xsl:value-of select="concat(if ($elementNameValue='*') then $allRootLevelTypeDefinitions else if($complexTypeName) then $complexTypeName else $element-to-process,' ')"/> by XSD 2 CAM generator v<xsl:value-of select="$version"/></as:Description>
					<as:Owner>To be Completed</as:Owner>
					<as:Version><xsl:value-of select="if(string-length($rootVersion) &gt; 0) then $rootVersion else '0.1'"/></as:Version>
					<as:DateTime><xsl:value-of select="current-dateTime()"/></as:DateTime>
				</as:Header>
				<xsl:if test="contains($CAMmode,'diagnostics')">
					<xsl:copy-of select="$doc"/>
				</xsl:if>
				<as:AssemblyStructure>
					<as:Structure taxonomy="XML"
						ID="{concat(if($elementNameValue='*') then $allRootLevelTypeDefinitions else if ($elementNameList &gt; 1) then $serviceComponentDefinitions else if($complexTypeName) then $complexTypeName else $element-to-process,'')}">
						<xsl:apply-templates select="$cleanExample/*" mode="structure"/>
					</as:Structure>
				</as:AssemblyStructure>
				<xsl:copy-of select="$rules-unique"/>
				<!-- Now write out lookup files if used in rules -->
				<xsl:if test="$useLookups='yes' or($useLookups='all')">
					<xsl:message>add lookup includes</xsl:message>
					<xsl:message>temporary work area: [<xsl:value-of select="$tempFilePath"/>]</xsl:message>
					<xsl:call-template name="writeLookupsIncludes">
						<xsl:with-param name="baseRules" select="$rules/as:BusinessUseContext/as:Rules/as:default/as:context"/>
					</xsl:call-template>
				</xsl:if>
				<xsl:copy-of select="$extensions"/>
			</as:CAM>
		</xsl:variable>
		<xsl:for-each select="$results/*">
			<xsl:copy>
				<xsl:for-each select=".//namespace::*"><xsl:if test="not(name()='w2e')"><xsl:namespace name="{name()}" select="."/></xsl:if></xsl:for-each>
				<!-- xsl:if test="$anonymousNS &gt; ' '">
			<xsl:namespace name="{$anonymousNS}"><xsl:value-of select="$defaultNS"/></xsl:namespace>
		</xsl:if -->
				<xsl:copy-of select="@*|node()"/>
			</xsl:copy>
		</xsl:for-each>
		<!-- xsl:value-of select="w2e:message(concat('completed template - ',format-dateTime(current-dateTime(),'[h01]:[m01]:[s01].[f01]')),2)"/ -->
		<xsl:message>build template done</xsl:message>

		<!-- Now write out lookup files if used in rules -->
		<xsl:if test="$useLookups='yes' or($useLookups='all')">
			<xsl:message>write lookup include files</xsl:message>
			<xsl:call-template name="writeLookups">
				<xsl:with-param name="baseRules"
					select="$rules/as:BusinessUseContext/as:Rules/as:default/as:context"/>
			</xsl:call-template>
		</xsl:if>

		<!-- xsl:result-document exclude-result-prefixes="w2e as w_pick" format="annoContent" indent="yes" href="{$annoFile}">
			<xsl:if test="contains($CAMmode,'anno') and(not(contains($annoMode,'inline')))">
				<xsl:message>write external annotations</xsl:message>
				<xsl:choose>
					<xsl:when test="$extensions//as:include">
						<as:Extension xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:camed="http://jcam.org.uk/editor"
							name="uk.org.jcam.camed.extensions.StructureAnnotations">
							<xsl:apply-templates select="$cleanExample/*" mode="anno"><xsl:with-param name="schemaContent" select="/"/><xsl:with-param name="rules" select="$rules-unique"/><xsl:with-param name="annoAttrs" select="$annotationAttrLines"/></xsl:apply-templates>
						</as:Extension>
					</xsl:when>
					<xsl:otherwise><xsl:copy-of select="$extensions"/></xsl:otherwise>
				</xsl:choose>
			</xsl:if>
		</xsl:result-document -->

		<xsl:message>template processing completed.</xsl:message>
	</xsl:template>

	<xsl:template match="xsd:element">
		<xsl:param name="ancestors"/>
		<xsl:param name="ref-name"/>
		<xsl:param name="ref-attributes"/>
		<xsl:param name="parent"/>
		<xsl:if test="count(@type) = 1"><xsl:value-of select="w2e:message(concat('element=',@name|@ref,' - ',@type),3)"/></xsl:if>
		<xsl:variable name="elementType" select="@type|@name"/>
		<xsl:variable name="chkExclude">
			<xsl:call-template name="chkExcludeElement">
				<xsl:with-param name="name" select="@name|@ref"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:value-of select="w2e:message(concat('exclude =',$chkExclude,' - ',$parent),4)"/>
		<xsl:choose>
			<xsl:when test="$chkExclude='true'"/>
			<xsl:when test="not(@ref) and @type">
				<xsl:call-template name="outputElementWithType">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="ref-name" select="$ref-name"/>
					<xsl:with-param name="ref-attributes" select="$ref-attributes"/>
					<xsl:with-param name="parent" select="$parent"/>
					<xsl:with-param name="parentType" select="@type"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="@ref">
				<!-- xsl:value-of select="w2e:message(concat('OutputElementWithRef for ',@ref),3)"/ -->
				<!-- xsl:value-of select="w2e:message($ref-attributes,4)"/ -->
				<xsl:variable name="ref-chk" select="@ref"/>
				<xsl:variable name="ref-attrs" select="@*"/>
				<!-- xsl:if test="count(@*) &gt; 1">
					<xsl:value-of select="w2e:message(concat('attrs:',count(@*),' ',name($ref-attrs[2])),4)"/></xsl:if -->
				<xsl:variable name="elementName" select="@name"/>
				<xsl:variable name="documentation">
					<xsl:choose>
						<xsl:when test=".//xsd:annotation/xsd:appinfo[@source='CAM']"><xsl:copy-of select=".//xsd:annotation/xsd:appinfo[@source='CAM']/text()"/></xsl:when>
						<xsl:when test="./xsd:annotation/xsd:documentation"><xsl:copy-of select="./xsd:annotation/xsd:documentation"/></xsl:when>
						<xsl:otherwise/>
					</xsl:choose>
				</xsl:variable>

				<xsl:choose>
					<xsl:when
						test="$documentation//child::*[name()='recursive'] or($documentation='recursive')">
						<xsl:variable name="ref-ns">
							<xsl:call-template name="namespace-uri-of-qname">
								<xsl:with-param name="qname" select="@ref"/>
							</xsl:call-template>
						</xsl:variable>
						<!-- xsl:value-of select="w2e:message(concat('recursion ignored=',@ref),2)"/ -->
						<xsl:message> - recursion ignored=<xsl:value-of select="@ref"/></xsl:message>
						<xsl:element name="{@ref}" namespace="{$ref-ns}">
							<xsl:for-each select="@*">
								<xsl:variable name="attrName" select="concat('xsd:',name())"/>
								<xsl:attribute name="{$attrName}" select="."/>
							</xsl:for-each>
							<xsl:attribute name="xsd:attr-annotation">recursive</xsl:attribute>
							<xsl:comment>recursive</xsl:comment>
							<xsd:any>xsd:any - ns[<xsl:value-of select="$ref-ns"/>]+[]+minOccurs[]</xsd:any>%anyContent%</xsl:element>
					</xsl:when>
					<xsl:when test="count(//xsd:element[@substitutionGroup=$ref-chk]) &gt; 0">
						<xsl:variable name="subCount" select="count(distinct-values(//xsd:element[@substitutionGroup=$ref-chk]/@name))"/>
						<xsl:value-of select="w2e:message(concat('substitutionGroup=',$ref-chk,' items=',$subCount ),3)"/>
						<xsl:variable name="thisType" select="@type"/>
						<xsl:variable name="ns">
							<xsl:choose>
								<xsl:when test="contains(@type,'xsd:')">
									<xsl:call-template name="namespace-uri-of-qname">
										<xsl:with-param name="qname" select="@name"/>
									</xsl:call-template>
								</xsl:when>
								<xsl:otherwise>
									<xsl:call-template name="namespace-uri-of-qname">
										<xsl:with-param name="qname" select="@type"/>
									</xsl:call-template>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<!-- xsl:value-of select="w2e:message(concat('substitutionNS=',$ns),4)"/ -->
						<xsl:variable name="items" select="subsequence(//xsd:element[@substitutionGroup=$ref-chk]/@name,1,$subCount)"/>
						<xsl:variable name="uniqueItems" select="distinct-values($items)"/>
						<xsl:choose>
							<xsl:when test="$subCount = 1">
								<xsl:variable name="thisRefName" 	select="//xsd:element[@substitutionGroup=$ref-chk][1]/@name[1]"/>
								<xsl:variable name="matched" select="//xsd:element[@substitutionGroup=$ref-chk]"/>
								<xsl:apply-templates select="$matched[1]">
									<xsl:with-param name="ancestors" select="$ancestors"/>
									<xsl:with-param name="ref-name" select="$thisRefName[1]"/>
									<xsl:with-param name="ref-attributes" select="$ref-attrs"/>
									<xsl:with-param name="parent" select="$parent"/>
								</xsl:apply-templates>
							</xsl:when>
							<xsl:when test="count($uniqueItems) = 1">
								<xsl:message> Substitutiongroup (1) - <xsl:value-of select="$ref-chk"/> - <xsl:value-of select="$items"/> - <xsl:value-of select="count($uniqueItems)"/></xsl:message>
								<xsl:for-each select="$items">
									<xsl:variable name="thisName" select="."/>
									<xsl:variable name="itemsListOccurances"><xsl:for-each  select="$items"><xsl:if test=". = $thisName"><xsl:value-of select="position()"/>|</xsl:if></xsl:for-each></xsl:variable>
									<xsl:variable name="duplicateChk" select="tokenize($itemsListOccurances,'\|')"/>
									<xsl:value-of select="w2e:message(concat('substitution position=',$thisName,' ',$itemsListOccurances,' - ',position()),4)"/>
									<xsl:variable name="thisItem" select="position()"/>
									<xsl:if test="$thisItem = xsd:integer($duplicateChk[1])">
										<xsl:value-of select="w2e:message(concat('substitutionName=',$thisName),4)"/>
										<xsl:variable name="matched" select="//xsd:element[@substitutionGroup=$ref-chk and @name=$thisName]"/>
										<xsl:apply-templates select="$matched[1]">
											<xsl:with-param name="ancestors" select="$ancestors"/>
											<xsl:with-param name="ref-name" select="$thisName"/>
											<xsl:with-param name="ref-attributes" select="$ref-attrs"/>
											<xsl:with-param name="parent" select="$parent"/>
										</xsl:apply-templates>
									</xsl:if>
								</xsl:for-each>
							</xsl:when>
							<xsl:otherwise>
								<xsl:message> Substitutiongroup (*) - <xsl:value-of select="$ref-chk"/> - <xsl:value-of select="$items"/> - <xsl:value-of select="count($uniqueItems)"/></xsl:message>
								<xsl:element name="xsd:choice">
									<!-- ??? this works here, but where else does it need to be? -->
									<xsl:for-each select="$items">
										<xsl:variable name="thisName" select="."/>
										<xsl:variable name="itemsListOccurances"><xsl:for-each  select="$items"><xsl:if test=". = $thisName"><xsl:value-of select="position()"/>|</xsl:if></xsl:for-each></xsl:variable>
										<xsl:variable name="duplicateChk" select="tokenize($itemsListOccurances,'\|')"/>
										<xsl:value-of select="w2e:message(concat('substitution position=',$thisName,' ',$itemsListOccurances,' - ',position()),4)"/>
										<xsl:variable name="thisItem" select="position()"/>
										<xsl:if test="$thisItem = xsd:integer($duplicateChk[1])">
											<xsl:value-of select="w2e:message(concat('substitutionName=',$thisName),4)"/>
											<xsl:variable name="matched" select="//xsd:element[@substitutionGroup=$ref-chk and @name=$thisName]"/>
											<xsl:apply-templates select="$matched[1]">
												<xsl:with-param name="ancestors" select="$ancestors"/>
												<xsl:with-param name="ref-name" select="$thisName"/>
												<xsl:with-param name="ref-attributes" select="$ref-attrs"/>
												<xsl:with-param name="parent" select="$parent"/>
											</xsl:apply-templates>
										</xsl:if>
									</xsl:for-each>
								</xsl:element>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="outputElementWithRef">
							<xsl:with-param name="ancestors" select="$ancestors"/>
							<xsl:with-param name="parent" select="$parent"/>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="count(xsd:simpleType/xsd:restriction/xsd:enumeration) &gt; 0">
				<xsl:value-of select="w2e:message('OutputElementSimpleType',4)"/>
				<xsl:variable name="value" select="xsd:simpleType/xsd:restriction/xsd:enumeration[1]/@value"/>
				<xsl:element name="{@name}">
					<xsl:for-each select="@*">
						<xsl:variable name="attrName" select="concat('xsd:',name())"/>
						<xsl:attribute name="{$attrName}" select="."/>
					</xsl:for-each>
					<xsl:for-each select="xsd:simpleType/xsd:restriction/@*">
						<xsl:variable name="attrName" select="concat('xsd:',name())"/>
						<xsl:attribute name="{$attrName}" select="."/>
					</xsl:for-each>
					<xsl:for-each select="xsd:simpleType/xsd:restriction/*">
						<xsl:if test="contains('xsd:minLength xsd:maxLength xs:minLength xs:maxLength xs:totalDigits xsd:totalDigits xs:fractionDigits xsd:fractionDigits',name())">
							<xsl:if test="not(starts-with(name(),$ns-prefix))">
								<xsl:variable name="attrName" select="concat('xsd:',local-name())"/>
								<xsl:attribute name="{$attrName}" select="@value"/>
							</xsl:if>
						</xsl:if>
					</xsl:for-each>
					<!-- xsl:value-of select="w2e:message(concat('outputEnum (1): ',@name),4)"/ -->
					<xsl:call-template name="outputEnumeration">
						<xsl:with-param name="typeName" select="@name"/>
						<xsl:with-param name="ns" select="@targetNamespace"/>
						<xsl:with-param name="enumValues" select="xsd:simpleType/xsd:restriction/xsd:enumeration"/>
						<xsl:with-param name="mode" select="'-E'"/>
					</xsl:call-template>
					<xsl:value-of select="w2e:type-to-example(concat('&quot;',$value[1],'&quot;'),'type')"/>
					<!-- xsl:value-of select="w2e:message(concat('closing fullname=',@name),3)"/ -->
				</xsl:element>
			</xsl:when>
			<xsl:when test="count(xsd:simpleType/xsd:restriction) &gt; 0">
				<!-- xsl:message> - ref name <xsl:value-of select="@name"/></xsl:message -->
				<xsl:variable name="elementName" select="if(string-length($ref-name) &gt; 0) then $ref-name else @name"/>
				<xsl:variable name="ns-ref">
					<xsl:call-template name="namespace-uri-of-qname">
						<xsl:with-param name="qname" select="$elementName"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:element name="{$elementName}" namespace="{$ns-ref}" >
					<xsl:for-each select="@*">
						<xsl:variable name="attrName" select="concat('xsd:',name())"/>
						<xsl:attribute name="{$attrName}" select="."/>
					</xsl:for-each>
					<xsl:for-each select="xsd:simpleType/xsd:restriction/@*">
						<xsl:variable name="attrName" select="concat('xsd:',name())"/>
						<xsl:attribute name="{$attrName}" select="."/>
					</xsl:for-each>
					<xsl:for-each select="xsd:simpleType/xsd:restriction/*">
						<xsl:if test="contains($xsdTypeAttributes,name())">
							<xsl:variable name="attrName" select="concat('xsd:',local-name())"/>
							<xsl:attribute name="{$attrName}" select="@value"/>
						</xsl:if>
					</xsl:for-each>
					<xsl:if test="xsd:simpleType/xsd:restriction/@base">
						<xsl:variable name="chkType" select="xsd:simpleType/xsd:restriction/@base"/>
						<xsl:variable name="simpleType" select="$entireXSDcollection//xsd:simpleType[@name=$chkType]"/>
						<xsl:variable name="chkBaseType" select="substring-before($simpleType[1]/xsd:restriction/@base,':')"/>
						<xsl:choose>
							<xsl:when test="count($simpleType[1]) &gt; 0"><!-- If this is a simpleType that references another type definition set then expand that -->
								<xsl:value-of select="w2e:message(concat('Outputting base type of type: ',$chkType),4)"/>
								<xsl:choose>
									<xsl:when test="$simpleType[1]/xsd:restriction/xsd:enumeration">
											<xsl:call-template name="outputEnumeration">
												<xsl:with-param name="typeName" select="$elementName"/>
												<xsl:with-param name="ns" select="$ns-ref"/>
												<xsl:with-param name="enumValues" select="$simpleType[1]/xsd:restriction/xsd:enumeration"/>
												<xsl:with-param name="mode" select="'-E'"/>
											</xsl:call-template>%"<xsl:value-of select="$simpleType[1]/xsd:restriction/xsd:enumeration[1]/@value"/>"%
									</xsl:when>
									<xsl:when test="$chkBaseType = 'xsd' or($chkBaseType = 'xs')">
										<xsl:attribute name="xsd:base" select="$simpleType[1]/xsd:restriction/@base"/>
										<xsl:value-of select="w2e:type-to-example($simpleType[1]/xsd:restriction/@base,'type')"/>
									</xsl:when>
									<xsl:otherwise>
										<xsl:apply-templates select="$simpleType[1]">
											<xsl:with-param name="ns" select="substring-before($chkType[1],':')"/>
											<xsl:with-param name="ancestors" select="$ancestors"/>
											<xsl:with-param name="parent" select="$parent"/>
										</xsl:apply-templates><xsl:value-of select="w2e:type-to-example(xsd:simpleType/xsd:restriction/@base,'type')"/>
									</xsl:otherwise>
								</xsl:choose>
								
							</xsl:when>
							<xsl:otherwise><xsl:value-of select="w2e:type-to-example($chkType[1],'type')"/></xsl:otherwise>
						</xsl:choose>
					</xsl:if>
				</xsl:element>
			</xsl:when>
			<xsl:when test="count(//xsd:complexType[@name=$elementType]/xsd:sequence) &gt; 0">
				<xsl:value-of select="w2e:message(concat('warning: matched element =',$elementType,' = complexType name'),3)"/>
				<xsl:variable name="thisType" select="//xsd:complexType[@name=$elementType]"/>
				<xsl:element name="{$elementType}">
					<!-- this element has no type declaration, but inferred because complexType has same name as element name -->
					<xsl:apply-templates select="$thisType[1]">
						<xsl:with-param name="ns" select="substring-before($elementNameValue,':')"/>
						<xsl:with-param name="ancestors" select="$ancestors"/>
						<xsl:with-param name="parent" select="$parent"/>
					</xsl:apply-templates>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="w2e:message(concat('Otherwise no type: ',@type|@ref|@name),3)"/>
				<xsl:call-template name="outputElementWithNoType">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="ref-attributes" select="$ref-attributes"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xsd:complexType" mode="start">
		<!-- xsl:value-of select="w2e:message(concat('ComplexType(start)=',@name),4)"/ -->
		<xsl:element name="{@name}">
			<xsl:apply-templates select="."/>
		</xsl:element>
	</xsl:template>

	<xsl:template match="xsd:complexType" mode="range">
		<xsl:param name="start"/>
		<xsl:param name="end"/>
		<xsl:variable name="firstChar" select="substring(@name,1,1)"/>
		<xsl:value-of select="w2e:message(concat('ComplexType(range)=',@name,' 1st:',$firstChar,' from: ',$start,' to ',$end),4)"/>
		<!-- xsl:message>Type: <xsl:value-of select="@name"/>[<xsl:value-of select="$firstChar"/>]</xsl:message -->
		<xsl:choose>
			<xsl:when test="$firstChar &gte; $start and($firstChar &lte; $end)">
				<xsl:value-of select="w2e:message(concat('ComplexType(range)=',@name,' selected'),3)"/>
				<xsl:element name="{@name}">
					<xsl:apply-templates select="."/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise/>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xsd:complexType">
		<xsl:param name="ns"/>
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<xsl:value-of select="w2e:message(concat('ComplexType(match any)=',@name,' [',position(),']'),2)"/>
		<xsl:variable name="thisName" select="@name"/>
		<xsl:variable name="useAttr">
			<xsl:call-template name="chkExcludeAttr">
				<xsl:with-param name="names"
					select="if (xsd:attribute/@ref) then xsd:attribute/@ref else xsd:attribute/@name"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
		</xsl:variable>	
		<!-- xsl:message> - attr -<xsl:copy-of select="$useAttr"/> - <xsl:value-of select="if (xsd:attribute/@ref) then xsd:attribute/@ref else xsd:attribute/@name"/> - <xsl:value-of select="$parent"/></xsl:message -->
		<xsl:apply-templates select="xsd:attribute">
			<xsl:with-param name="useList" select="$useAttr"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xsd:attributeGroup">
			<xsl:with-param name="parent" select="$parent"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xsd:anyAttribute"/>
		<xsl:call-template name="inlineAttributeRules">
			<xsl:with-param name="use" select="$useAttr"/>
		</xsl:call-template>
		<xsl:variable name="chkExclude">
			<xsl:call-template name="chkExcludeElement">
				<xsl:with-param name="name" select="@name"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:choose>
			<xsl:when test="$chkExclude='true'"/>
			<xsl:when test="count(//xsd:complexType[@name=$thisName]) =1">
				<!-- xsl:value-of select="w2e:message('ApplyingUniqueComplexType',4)"/ -->
				<!-- xsl:apply-templates select="xsd:attribute"/ -->
				<xsl:apply-templates select="xsd:choice|xsd:sequence">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="count(xsd:sequence|xsd:choice) =1">
				<!-- xsl:value-of select="w2e:message('ComplexTypeSimpleSequence',4)"/ -->
				<!-- xsl:apply-templates select="xsd:attribute"/ -->
				<xsl:apply-templates select="xsd:choice|xsd:sequence">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when
				test="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace[contains(.,$ns)][1]">
				<!-- xsl:value-of select="w2e:message('ApplyingComplexType',4)"/ -->
				<!-- xsl:apply-templates select="xsd:attribute"/ -->
				<xsl:apply-templates select="xsd:choice|xsd:sequence">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
			</xsl:when>
			<!-- xsl:when test="xsd:simpleContent">
				<xsl:apply-templates select="xsd:simpleContent">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
			</xsl:when -->
			<xsl:otherwise>
				<xsl:message> - warning: skipped complexType <xsl:value-of select="@name"/> <xsl:for-each select="child::*"> | <xsl:value-of select="name(.)"/></xsl:for-each></xsl:message>
				<xsl:value-of select="w2e:message(concat('namespace=',$ns,'[',ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace,']'),4)"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xsd:complexType" mode="enclosed">
		<xsl:param name="ns"/>
		<!-- xsl:value-of select="w2e:message(concat('ComplexType(enclosed)=',@name,'[',position(),']'),4)"/ -->
		<!-- xsl:if test="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace[contains(.,$ns)][1]">
			<xsl:value-of select="w2e:message('ApplyingComplexType',4)"/>
		</xsl:if -->
		<xsl:call-template name="outputEnclosedType">
			<xsl:with-param name="typeName" select="@name"/>
			<xsl:with-param name="typeNS" select="$ns"/>
		</xsl:call-template>
	</xsl:template>
	
	<xsl:template match="xsd:complexType/xsd:choice">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<xsl:variable name="names" select=".//xsd:element/@name|@ref"/>
		<xsl:variable name="namesUnique" select="distinct-values($names)"/>
		<xsl:variable name="namesList"><xsl:for-each select="$names"><xsl:value-of select="concat(.,' ')"/></xsl:for-each></xsl:variable>
		<xsl:variable name="parentName" select="reverse(tokenize($parent,'\\'))"/>
		<xsl:value-of select="w2e:message(concat($parentName[1],' is choice of: ',$namesList),2)"/>
		<!-- xsl:message> - choice of <xsl:value-of select="$names"/></xsl:message -->
		<!-- xsl:apply-templates select="xsd:attribute">   note: attributes of choice not handled currently ?????
			<xsl:with-param name="ancestors" select="$ancestors"/>
			<xsl:with-param name="parent" select="$parent"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xsd:attributeGroup">
			<xsl:with-param name="ancestors" select="$ancestors"/>
			<xsl:with-param name="parent" select="$parent"/>
		</xsl:apply-templates -->
		<xsl:element name="xsd:choice">
			<xsl:for-each select="./*">
				<!-- xsl:message> - <xsl:value-of select="name()"/> - <xsl:value-of select="@name|@ref"/> - <xsl:value-of select="position()"/> - <xsl:value-of select="$names[. = @name|@ref]/position()"/></xsl:message -->
				<xsl:apply-templates select=".">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
			</xsl:for-each>
		</xsl:element>
	</xsl:template>

	<xsl:template match="xsd:complexType[xsd:simpleContent/xsd:extension]">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<xsl:variable name="loopChk" select="if (string-length(substring-after(xsd:simpleContent/xsd:extension/@base,':')) = 0) then xsd:simpleContent/xsd:extension/@base else substring-after(xsd:simpleContent/xsd:extension/@base,':')"/>
		<xsl:variable name="baseTypeName" select="xsd:simpleContent/xsd:extension/@base"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:complexType[simpleContent][',@name,'+',$loopChk,']'),1)"/ -->
		<!-- xsl:value-of select="w2e:message(concat('xsd:simpleContent/xsd:extension/@base[',xsd:simpleContent/xsd:extension/@base,'+',$baseTypeName,']'),1)"/ -->
		<xsl:choose>
			<!-- enhanced loop namespace aware check -->
			<xsl:when test="not($loopChk=@name)">
				<xsl:value-of select="w2e:message(concat(' - simpleContent extension: ',$baseTypeName),4)"/>
				<xsl:apply-templates select="xsd:simpleContent/xsd:extension" mode="simple">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="$loopChk=@name and(not(@base=$loopChk))">
				<xsl:variable name="nsPath">
					<xsl:call-template name="namespace-uri-of-qname">
						<xsl:with-param name="qname" select="@base"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:variable name="matchedType"
					select="if(starts-with($baseTypeName,'xsd:') or(starts-with($baseTypeName,'xs:'))) then $baseTypeName else //xsd:import[@namespace=$nsPath]/xsd:schema/xsd:complexType[@name=$loopChk]/xs:simpleContent/xs:extension"/>
				<!-- xsl:value-of select="w2e:message(concat('matchedType[',$matchedType[1]/@base,'+',$baseTypeName,']'),1)"/ -->
				<xsl:apply-templates select="$matchedType[1]/xsd:attribute">
					<xsl:with-param name="useList" select="$emptyList"/>
				</xsl:apply-templates>
				<xsl:apply-templates select="$matchedType[1]/xsd:attributeGroup">
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
				<xsl:value-of select="w2e:type-to-example(if(exists($matchedType[1]/@base)) then $matchedType[1]/@base else $baseTypeName,'type')"/>
				<xsl:call-template name="inlineAttributeRulesType">
					<xsl:with-param name="attrs" select="$matchedType[1]/xsd:attribute"/>
					<xsl:with-param name="attrGrp" select="$matchedType[1]/xsd:attributeGroup"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise><xsl:value-of select="w2e:type-to-example(@name,'type')"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xsd:complexType[xsd:simpleContent/xsd:restriction]">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<xsl:variable name="loopChk"
			select="if (string-length(substring-after(xsd:simpleContent/xsd:restriction/@base,':')) = 0) then xsd:simpleContent/xsd:restriction/@base else substring-after(xsd:simpleContent/xsd:restriction/@base,':')"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:complexType[simpleContent][',@base,'+',$loopChk,']'),4)"/ -->
		<xsl:choose>
			<!-- enhanced loop namespace aware check -->
			<xsl:when test="not($loopChk=@name)">
				<xsl:apply-templates select="xsd:simpleContent/xsd:restriction" mode="simple">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="$loopChk=@name and(not(@base=$loopChk))">
				<xsl:variable name="nsPath">
					<xsl:call-template name="namespace-uri-of-qname">
						<xsl:with-param name="qname" select="@base"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:variable name="matchedType"
					select="//xsd:import[@namespace=$nsPath]/xsd:schema/xsd:complexType[@name=$loopChk]/xs:simpleContent/xs:restriction"/>
				<xsl:apply-templates select="$matchedType[1]/xsd:attribute">
					<xsl:with-param name="useList" select="$emptyList"/>
				</xsl:apply-templates>
				<xsl:apply-templates select="$matchedType[1]/xsd:attributeGroup">
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
				<xsl:value-of select="w2e:type-to-example($matchedType[1]/@base,'type')"/>
				<xsl:call-template name="inlineAttributeRulesType">
					<xsl:with-param name="attrs" select="$matchedType[1]/xsd:attribute"/>
					<xsl:with-param name="attrGrp" select="$matchedType[1]/xsd:attributeGroup"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="w2e:type-to-example(@name,'type')"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xsd:complexType[xsd:complexContent]">
		<xsl:param name="ancestors"/>
		<xsl:param name="ns"/>
		<xsl:param name="parent"/>
		<xsl:value-of select="w2e:message(concat('xsd:complexType[xsd:complexContent]:',@name,';','[',$ns,']'),3)"/>
		<xsl:variable name="recursionChk" select="xsd:annotation/appinfo[@source='CAM'][1]/text()"/>
		<xsl:if test="contains($recursionChk,'recursive')">
			<!-- xsl:value-of select="w2e:message(concat('recursive complexType:',@name,';','[',$ns,']'),2)"/ -->
			<xsl:message> - recursive complexType: <xsl:value-of select="@name"/> - ns: <xsl:value-of select="$ns"/></xsl:message>
			<!-- xsl:comment>recursive</xsl:comment -->
		</xsl:if>
		<xsl:variable name="baseNS" select="xsd:complexContent/xsd:extension/@base"/>
		<xsl:if
			test="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace[contains(.,$ns)][1] or($ns='' and(not(contains($baseNS,':'))))">
			<xsl:value-of select="w2e:message(concat('Applying xsd:complexContent:',@name,';','[',$ns,']'),3)"/>
			<xsl:apply-templates select="xsd:complexContent/xsd:extension" mode="complex">
				<xsl:with-param name="ancestors" select="$ancestors"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:apply-templates>
			<xsl:apply-templates select="xsd:simpleContent/xsd:extension" mode="simple">
				<xsl:with-param name="ancestors" select="$ancestors"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:apply-templates>
		</xsl:if>
	</xsl:template>

	<xsl:template match="xsd:simpleContent" mode="simple">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:simpleContent: ',xsd:extension/@base),4)"/ -->
		<xsl:apply-templates select="xsd:extension" mode="simple">
			<xsl:with-param name="ancestors" select="$ancestors"/>
			<xsl:with-param name="parent" select="$parent"/>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="xsd:simpleContent">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<xsl:value-of select="w2e:message(concat('xsd:simpleContent (any): ',xsd:extension/@base),4)"/>
		<xsl:apply-templates select="xsd:extension" mode="simple">
			<xsl:with-param name="ancestors" select="$ancestors"/>
			<xsl:with-param name="parent" select="$parent"/>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="xsd:sequence">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:sequence: ',xsd:element[1]/@name,' ',xsd:element[1]/@type),1)"/ -->
		<xsl:for-each select=".">
			<xsl:choose>
				<xsl:when test="empty(child::*)"><xsl:value-of select="w2e:message(concat('EMPTY xsd:sequence: ',$parent),4)"/>%string%</xsl:when>
				<!-- assume string content for empty xsd:sequence -->
				<xsl:when test="xsd:element|xsd:choice|xsd:sequence|xsd:group">
					<xsl:for-each select="xsd:element|xsd:choice|xsd:sequence|xsd:group">
						<xsl:if test="count(./@type) = 1"><xsl:value-of select="w2e:message(concat(name(),' ',./@name[1]|./@ref[1],' ',./@type[1]),3)"/></xsl:if>
						<xsl:apply-templates select=".">
							<xsl:with-param name="ancestors" select="$ancestors"/>
							<xsl:with-param name="parent" select="$parent"/>
						</xsl:apply-templates>
					</xsl:for-each>
				</xsl:when>
				<xsl:when test="xsd:any">
					<xsl:apply-templates select="xsd:any"/>
				</xsl:when>
				<xsl:otherwise/>
			</xsl:choose>
		</xsl:for-each>
		<xsl:if test="xsd:any and count(.) &lt;1">
			<xsl:value-of select="w2e:message(concat('xsd:sequence/xsd:any - ',$parent),4)"/>
			<xsl:apply-templates select="xsd:any"/>
		</xsl:if>
	</xsl:template>

	<xsl:template match="xsd:choice">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<xsl:value-of select="w2e:message('xsd:choice',4)"/>
		<xsl:comment>elements are a choice</xsl:comment>
		<xsl:variable name="maxOccurs" select="@maxOccurs"/>
		<!-- xsl:if test="$maxOccurs &gt; ' '"><xsl:message> - choice - maxOccurs - <xsl:value-of select="$maxOccurs"/> - <xsl:value-of select="//xsd:sequence/xsd:element/@name"/></xsl:message></xsl:if -->
		<xsl:choose>
			<xsl:when test="xsd:sequence">
				<xsl:element name="xsd:choice">
					<xsl:if test="$maxOccurs &gt; ' '">
						<xsl:element name="xsd:maxOccurs">
							<xsl:value-of select="$maxOccurs"/>
						</xsl:element>
					</xsl:if>
					<!-- make sure sequences contain different elements - avoid xsd kludge -->
					<xsl:variable name="sequenceChk">
						<xsl:for-each select="xsd:sequence">
							<xsl:variable name="seqn" select="position()"/>
							<xsl:for-each select="xsd:element" exclude-result-prefixes="#all">
								<seqn grp="{$seqn}" name="{@name|@ref}"/>
							</xsl:for-each>
						</xsl:for-each>
					</xsl:variable>
					<!-- xsl:if test="count(xsd:sequence) &gt; 1">	<xsl:message>[<xsl:copy-of select="$sequenceChk"></xsl:copy-of>]</xsl:message></xsl:if -->
					<xsl:for-each select="xsd:sequence">
						<xsl:comment> = choice sequence = </xsl:comment>

						<xsl:variable name="includeSeqn">
							<xsl:choose>
								<xsl:when test="position() &lt; 2">true</xsl:when>
								<xsl:otherwise>
									<xsl:variable name="this" select="position()"/>
									<xsl:for-each select="$sequenceChk/seqn[@grp=$this]">
										<xsl:variable name="chkName" select="@name"/>
										<xsl:if test="not($sequenceChk/seqn[@grp='1'][@name=$chkName])">true</xsl:if>
									</xsl:for-each>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>

						<xsl:if test="$includeSeqn='true'">
							<xsl:element name="xsd:choice">
								<xsl:if test="$maxOccurs &gt; ' '">
									<xsl:element name="xsd:maxOccurs">
										<xsl:value-of select="$maxOccurs"/>
									</xsl:element>
								</xsl:if>
								<xsl:apply-templates select="xsd:element">
									<xsl:with-param name="ancestors" select="$ancestors"/>
									<xsl:with-param name="parent" select="$parent"/>
								</xsl:apply-templates>
								<xsl:apply-templates select="xsd:any"/>
							</xsl:element>
						</xsl:if>
					</xsl:for-each>
					<xsl:variable name="choiceElements"><xsl:for-each select="xsd:sequence/xsd:element">|<xsl:value-of select="@name|@ref"/>|</xsl:for-each></xsl:variable>
					<xsl:for-each select="xsd:element">
						<xsl:variable name="chkName" select="concat('|',@name|@ref,'|')"/>
						<!-- xsl:message>[<xsl:value-of select="$chkName"></xsl:value-of>][<xsl:value-of select="$choiceElements"></xsl:value-of>][<xsl:value-of select="$parent"></xsl:value-of>]</xsl:message -->
						<!-- make sure the following element is not a duplicate of those in the choice (xsd kludge)-->
						<xsl:if test="not(contains($choiceElements,$chkName))">
							<xsl:apply-templates select=".">
								<xsl:with-param name="ancestors" select="$ancestors"/>
								<xsl:with-param name="parent" select="$parent"/>
							</xsl:apply-templates>
						</xsl:if>
					</xsl:for-each>
					<xsl:apply-templates select="xsd:any"/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:element name="xsd:choice">
					<xsl:if test="$maxOccurs &gt; ' '">
						<xsl:element name="xsd:maxOccurs">
							<xsl:value-of select="$maxOccurs"/>
						</xsl:element>
					</xsl:if>
					<xsl:apply-templates select="xsd:element|xsd:choice/xsd:element|xsd:group">
						<xsl:with-param name="ancestors" select="$ancestors"/>
						<xsl:with-param name="parent" select="$parent"/>
					</xsl:apply-templates>
					<xsl:apply-templates select="xsd:any"/>
				</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xs:all">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<xsl:value-of select="w2e:message('xsd:all(element)',4)"/>
		<xsl:comment> elements are an all group </xsl:comment>
		<xsl:apply-templates select="xsd:element">
			<xsl:with-param name="ancestors" select="$ancestors"/>
			<xsl:with-param name="parent" select="$parent"/>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="xsd:restriction" mode="simple">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:simpleContent/xsd:extension=&gt;',@base),4)"/ -->
		<xsl:variable name="type" select="if (string-length(substring-after(@base,':')) = 0) then @base else substring-after(@base,':')"/>
		<xsl:variable name="useAttr">
			<xsl:call-template name="chkExcludeAttr">
				<xsl:with-param name="names"
					select="if (xsd:attribute/@ref) then xsd:attribute/@ref else xsd:attribute/@name"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:apply-templates select="xsd:attribute">
			<xsl:with-param name="useList" select="$useAttr"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xsd:attributeGroup">
			<xsl:with-param name="parent" select="$parent"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xsd:anyAttribute"/>
		<xsl:variable name="matchedExtn"
			select="//xsd:complexType[@name=$type]/xsd:simpleContent/xsd:extension"/>
		<xsl:variable name="matchedRtcn"
			select="//xsd:complexType[@name=$type]/xsd:simpleContent/xsd:restriction"/>
		<xsl:if test="string-length($matchedRtcn[1]/@base) &gt; 0">
			<xsl:variable name="parentPath" select="tokenize($parent,'\\')"/>
			<xsl:variable name="parentName" select="$parentPath[count($parentPath)]"/>
			<xsl:for-each select="$matchedRtcn[1]/*">
				<xsl:if test="contains($xsdTypeAttributes,name())">
					<xsl:variable name="attrName" select="concat('xsd:',local-name())"/>
					<xsl:attribute name="{$attrName}" select="@value"/>
				</xsl:if>
			</xsl:for-each>
			<!-- Enumerations -->
			<xsl:call-template name="outputEnumeration">
				<xsl:with-param name="typeName" select="$parentName"/>
				<xsl:with-param name="ns" select="@targetNamespace"/>
				<xsl:with-param name="enumValues" select="$matchedRtcn[1]/xsd:enumeration"/>
				<xsl:with-param name="mode">-E</xsl:with-param>
			</xsl:call-template>
		</xsl:if>
		<xsl:call-template name="inlineAttributeRules">
			<xsl:with-param name="use" select="$useAttr"/>
		</xsl:call-template>
		<xsl:value-of select="w2e:message(concat('xsd:restriction (simple): ',$type),4)"/>

		<xsl:choose>
			<xsl:when test="string-length($matchedExtn[1]/@base) &gt; 0">
				<!-- Find base type type -->
				<xsl:variable name="findBaseType" select="$matchedExtn[1]/@base"/>
				<xsl:variable name="baseType" select="//xsd:complexType[@name=$findBaseType]/xsd:simpleContent/xsd:extension/@base"/>
				<xsl:value-of select="w2e:type-to-example(if($baseType[1]) then $baseType[1] else $findBaseType,'base')"/>
			</xsl:when>
			<xsl:when test="string-length($matchedRtcn[1]/@base) &gt; 0">
				<!-- Find base type type -->
				<xsl:variable name="findBaseType" select="$matchedRtcn[1]/@base"/>
				<xsl:variable name="baseType" select="//xsd:complexType[@name=$findBaseType]/xsd:simpleContent/xsd:extension/@base"/>
				<xsl:value-of select="w2e:type-to-example(if($baseType[1]) then $baseType[1] else $findBaseType,'base')"/>
			</xsl:when>
			<xsl:when test="string-length($type) &gt; 0"><xsl:value-of select="w2e:type-to-example($type,'base')"/></xsl:when>
			<xsl:otherwise><xsl:value-of select="concat('base=',@base)"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xsd:extension" mode="simple">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<xsl:value-of select="w2e:message(concat('[xsd:simpleContent/xsd:extension]=&gt;',@base,' Parent: ',name(../../parent::node()),' ',../../parent::node()/@name),4)"/>
		<xsl:variable name="type" select="if (string-length(substring-after(@base,':')) = 0) then @base else substring-after(@base,':')"/>
		<xsl:variable name="useAttr">
			<xsl:call-template name="chkExcludeAttr">
				<xsl:with-param name="names" select="if (xsd:attribute/@ref) then xsd:attribute/@ref else xsd:attribute/@name"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:apply-templates select="xsd:attribute">
			<xsl:with-param name="useList" select="$useAttr"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xsd:attributeGroup">
			<xsl:with-param name="parent" select="$parent"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xsd:anyAttribute"/>
		<xsl:value-of select="w2e:message(concat('xsd:extension (simple): ',$type),4)"/>
		<xsl:call-template name="inlineAttributeRules">
			<xsl:with-param name="use" select="$useAttr"/>
		</xsl:call-template>
		
		<!-- xsl:variable name="chkParentType" select="name(../../parent::node())"/>
							<xsl:variable name="chkParentTypeName" select="if(contains($chkParentType,':')) then substring-after($chkParentType,':') else $chkParentType"/>
							<xsl:if test="$chkParentTypeName = 'element'" -->
		<!-- xsl:value-of select="w2e:message(concat('xsd:enumeration: ',$type),4)"/ -->
		
		<xsl:variable name="baseTypeDetails" select="//xsd:simpleType[@name = $type]"/>
		
		<xsl:if test="$baseTypeDetails/xsd:restriction">
			<xsl:value-of select="w2e:message(concat(' - element enumerations ',name(../../parent::node()),' type ',$type),4)"/>
			<xsl:variable name="matched" select="$baseTypeDetails[1]/xsd:restriction/xsd:enumeration"/>
			<xsl:call-template name="outputEnumeration">
					<xsl:with-param name="typeName" select="$type"/>
					<xsl:with-param name="ns" select="@targetNamespace"/>
					<xsl:with-param name="enumValues" select="$matched"/>
					<xsl:with-param name="mode" select="'-E'"/>
			</xsl:call-template>
		</xsl:if>
		
		<xsl:if test="$baseTypeDetails/xsd:union">
			<!-- xsl:message> - union for base details <xsl:value-of select="$type"/> [<xsl:value-of select="$baseTypeDetails[1]/xsd:union/@memberTypes"/>]</xsl:message -->
			<xsl:value-of select="w2e:message(concat(' - element union ',name(../../parent::node()),' type ',$type),4)"/>
			<xsl:variable name="matched" select="$baseTypeDetails[1]/xsd:union"/>
			<xsl:apply-templates select="$matched[1]">
				<xsl:with-param name="ancestors" select="$ancestors"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:apply-templates>
		</xsl:if>
		
		<xsl:if test="$baseTypeDetails/xsd:list">
			<xsl:message> - list for base details <xsl:value-of select="$type"/> [<xsl:value-of select="$baseTypeDetails/xsd:list/@itemType"/>]</xsl:message>
			<xsl:value-of select="w2e:message(concat(' - element index ',name(../../parent::node()),' type ',$type),4)"/>
			<xsl:variable name="matched" select="$baseTypeDetails[1]/xsd:index"/>
			<xsl:apply-templates select="$matched[1]">
				<xsl:with-param name="ancestors" select="$ancestors"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:apply-templates>
		</xsl:if>
		
		<xsl:choose>
			<xsl:when
				test="count(//xsd:complexType[@name=$type]) &gt; 0 and(not(count(//xsd:simpleType[@name=$type]/xsd:restriction) &gt;0))">
				<xsl:variable name="matched" select="//xsd:complexType[@name=$type]"/>
				<xsl:apply-templates select="$matched[1]"> </xsl:apply-templates>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="baseType">
					<xsl:choose>
						<xsl:when test="count(//xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration) &gt; 0">
							<xsl:value-of select="w2e:message(concat('[xsd:simpleContent/xsd:extension/xsd:enumeration]=&gt;',@base,' Parent: ',name(../../parent::node()),' ',../../parent::node()/@name),4)"/>

							<xsl:variable name="matched" select="//xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration"/>
							
							<!-- this is needed for when people redefine base types; hence duplicates can occur -->
							<xsl:value-of select="concat('&quot;',$matched[1]/@value,'&quot;')"/>
							
						</xsl:when>
						<xsl:when test="count(//xsd:simpleType[@name=$type]/xsd:restriction/@base) &gt; 0">
							<xsl:variable name="matched" select="//xsd:simpleType[@name=$type]/xsd:restriction"/>
							<xsl:value-of select="$matched[1]/@base"/>
							<xsl:value-of select="w2e:message(concat('xsd:restriction: ',$type),4)"/>
							<xsl:choose>
								<xsl:when test="count($matched[1]/xsd:enumeration) = 0"/>
								<xsl:otherwise>
									<xsl:call-template name="outputRestrictions">
										<xsl:with-param name="typeName" select="$type"/>
										<xsl:with-param name="ns" select="@targetNamespace"/>
										<xsl:with-param name="enumValues" select="$matched[1]"/>
									</xsl:call-template>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:when>
						<xsl:otherwise><xsl:value-of select="@base"/></xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:value-of select="w2e:message(concat('baseType ',$baseType,' ',if(contains($baseType,'&quot;')) then $type else ''),4)"/>
				<xsl:attribute name="xsd:resolvedType" select="if(contains($baseType,'&quot;')) then $type else $baseType"/>
				<xsl:value-of select="w2e:type-to-example($baseType,'base')"/>
			</xsl:otherwise>
			<!-- DO NOT COMMENT out these! -->
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xsd:extension" mode="complex">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<xsl:value-of select="w2e:message(concat('extension (complex): ',@base),3)"/>
		<xsl:variable name="type"
			select="if (string-length(substring-after(@base,':')) = 0) then @base else substring-after(@base,':')"/>
		<xsl:variable name="ns">
			<xsl:call-template name="namespace-uri-of-qname">
				<xsl:with-param name="qname" select="@base"/>
			</xsl:call-template>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('ns=',$ns),4)"/ -->
		<xsl:variable name="namespace-for-name">
			<xsl:value-of select="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace"/>
		</xsl:variable>
		<xsl:variable name="ns-to-use">
			<xsl:value-of select="if (string-length($ns) = 0) then $namespace-for-name else $ns"/>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('namespace to use=',$ns-to-use,':',$type),4)"/ -->
		<xsl:variable name="useAttr">
			<xsl:call-template name="chkExcludeAttr">
				<xsl:with-param name="names"
					select="if (xsd:attribute/@ref) then xsd:attribute/@ref else xsd:attribute/@name"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:apply-templates select="xsd:attribute">
			<xsl:with-param name="useList" select="$useAttr"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xsd:attributeGroup">
			<xsl:with-param name="parent" select="$parent"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xsd:anyAttribute"/>
		<xsl:call-template name="inlineAttributeRules">
			<xsl:with-param name="use" select="$useAttr"/>
		</xsl:call-template>
		<xsl:variable name="matched" select="//xsd:complexType[@name=$type]"/>
		<xsl:apply-templates select="$matched[1]">
			<xsl:with-param name="ancestors" select="$ancestors"/>
			<xsl:with-param name="ns" select="$ns-to-use"/>
			<xsl:with-param name="parent" select="$parent"/>
		</xsl:apply-templates>
		<xsl:choose>
			<xsl:when test="xsd:sequence">
				<xsl:value-of
					select="w2e:message(concat('extension with sequence: ',xsd:sequence/xsd:element[1]/@ref),3)"/>
				<xsl:apply-templates select="xsd:sequence">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
				<xsl:apply-templates select="xsd:any"/>
			</xsl:when>
			<xsl:when test="xsd:choice">
				<xsl:apply-templates select="xsd:choice">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="xsd:all">
				<xsl:apply-templates select="xsd:all">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="xsd:group">
				<xsl:apply-templates select="xsd:group">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xsd:enumeration">
		<xsl:value-of select="w2e:message(concat('xsd:enumeration: ',@value),4)"/>
	</xsl:template>

	<xsl:template match="xsd:attributeGroup[@name]">
		<xsl:param name="parent"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:attributeGroup: ',@name),4)"/ -->
		<xsl:variable name="useAttr">
			<xsl:call-template name="chkExcludeAttr">
				<xsl:with-param name="names"
					select="if (xsd:attribute/@ref) then xsd:attribute/@ref else xsd:attribute/@name"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:apply-templates select="xsd:attribute">
			<xsl:with-param name="useList" select="$useAttr"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xsd:attributeGroup">
			<xsl:with-param name="parent" select="$parent"/>
		</xsl:apply-templates>
		<xsl:call-template name="inlineAttributeRules">
			<xsl:with-param name="use" select="$useAttr"/>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="xsd:attributeGroup[@ref]">
		<xsl:param name="parent"/>
		<xsl:variable name="useAttr">
			<xsl:call-template name="chkExcludeAttr">
				<xsl:with-param name="names"
					select="if (xsd:attribute/@ref) then xsd:attribute/@ref else xsd:attribute/@name"/>
				<xsl:with-param name="parent" select="$parent"/>
			</xsl:call-template>
		</xsl:variable>

		<xsl:variable name="checkName" select="./ancestor::*/@name"/>

		<xsl:if test="not(contains($baseDataTypes,concat($checkName[1],'|')))">
			<xsl:attribute name="xsd:attr-annotation"><xsl:value-of select="$checkName[1]"/> |
					<xsl:value-of select="@ref | @name"/></xsl:attribute>
		</xsl:if>
		<!-- xsl:value-of select="w2e:message(concat('xsd:attributeGroup(ref): ',@ref),4)"/ -->
		<xsl:choose>
			<xsl:when test="count(xsd:attribute|xsd:attributeGroup) &gt; 0">
				<xsl:apply-templates select="xsd:attribute">
					<xsl:with-param name="useList" select="$useAttr"/>
				</xsl:apply-templates>
				<xsl:apply-templates select="xsd:attributeGroup">
					<xsl:with-param name="parent" select="$parent"/>
				</xsl:apply-templates>
				<xsl:call-template name="inlineAttributeRules">
					<xsl:with-param name="use" select="$useAttr"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="ref"
					select="if (string-length(substring-after(@ref,':')) = 0) then @ref else substring-after(@ref,':')"/>
				<xsl:choose>
					<xsl:when test="not(@ref=$ref)">
						<xsl:variable name="matched" select="//xsd:attributeGroup[@name=$ref]"/>
						<xsl:apply-templates select="$matched[1]"/>
					</xsl:when>
					<xsl:when test="@ref=$ref">
						<xsl:variable name="matched" select="//xsd:attributeGroup[@name=$ref]"/>
						<xsl:choose>
							<xsl:when test="count($matched[1]/xsd:attribute) &gt; 0">
								<xsl:for-each select="$matched[1]/xsd:attribute">
									<xsl:variable name="attrType"
										select="if(string-length(@type) &gt; 0) then @type else 'string'"/>
									<xsl:call-template name="outputAttribute">
										<xsl:with-param name="name" select="@name"/>
										<xsl:with-param name="ns" select="''"/>
										<xsl:with-param name="value"
											select="w2e:type-to-example($attrType, 'attr')"/>
										<xsl:with-param name="use" select="@use"/>
									</xsl:call-template>
								</xsl:for-each>
								<xsl:if test="count($matched[1]/xsd:anyAttribute) &gt; 0">
									<xsl:call-template name="outputAttribute">
										<xsl:with-param name="name" select="'anyAttribute'"/>
										<xsl:with-param name="ns" select="''"/>
										<xsl:with-param name="value"
											select="w2e:type-to-example('##any', 'attr')"/>
										<xsl:with-param name="use" select="@use"/>
									</xsl:call-template>
								</xsl:if>
							</xsl:when>
							<xsl:otherwise>
								<xsl:call-template name="outputAttribute">
									<xsl:with-param name="name" select="@ref"/>
									<xsl:with-param name="ns" select="''"/>
									<xsl:with-param name="value"
										select="w2e:type-to-example(@ref, 'attr')"/>
									<xsl:with-param name="use" select="@use"/>
								</xsl:call-template>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="outputAttribute">
							<xsl:with-param name="name" select="@ref"/>
							<xsl:with-param name="ns" select="''"/>
							<xsl:with-param name="value" select="w2e:type-to-example(@ref, 'attr')"/>
							<xsl:with-param name="use" select="@use"/>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xsd:anyAttribute">
		<xsl:value-of
			select="w2e:message(concat('xsd:anyAttribute: ',@namespace,'+',@processContents),4)"/>
		<!-- xsl:comment> <xsl:value-of select="concat('xsd:anyAttribute: ',@namespace,'+',@processContents)"/> </xsl:comment-->
		<xsl:attribute name="{'xsd:anyAttribute'}">
			<xsl:value-of
				select="concat('xsd:anyAttribute - ns[',@namespace,']+[',@processContents,']')"/>
		</xsl:attribute>
	</xsl:template>

	<xsl:template match="xsd:any">
		<xsl:value-of
			select="w2e:message(concat('xsd:any: ns[',@namespace,']+[',@processContents,']'),4)"/>
		<!-- xsl:comment> <xsl:value-of select="concat('xsd:any: ',@namespace,'+',@processContents)"/> </xsl:comment-->
		<xsl:element name="{'xsd:any'}"><xsl:value-of
				select="concat('xsd:any - ns[',@namespace,']+[',@processContents,']+minOccurs[',@minOccurs,']')"
			/></xsl:element>%any%</xsl:template>

	<!-- xsl:template match="xsd:attribute[xsd:simpleType/xsd:union and @name]">  ???
		<xsl:variable name="unionNames" select=""/>
		<xsl:variable name="processUnion">
			<xsd:simpleType name="SystemEnvironmentCodeContentType"/>
		</xsl:variable>
		<xsl:apply-templates select="$processUnion"/>
	</xsl:template -->

	<xsl:template match="xsd:attribute[@name and(not(xsd:simpleType/xsd:restriction))]">
		<xsl:param name="useList"/>
		<xsl:variable name="chk" select="@name"/>
		<!-- xsl:message>1 - <xsl:value-of select="$chk"/>[<xsl:copy-of select="$useList"/>]</xsl:message -->
		<xsl:if test="not($useList//attribute[@e=$chk])">
			<xsl:variable name="baseType">
				<xsl:call-template name="chkType">
					<xsl:with-param name="type" select="@type"/>
				</xsl:call-template>
			</xsl:variable>
			<!-- Now also returning union of types first value example -->
			<!-- xsl:value-of select="w2e:message(concat('xsd:attribute with name:',@name,' + ',$baseType),4)"/ -->
			<xsl:call-template name="outputAttribute">
				<xsl:with-param name="name" select="@name"/>
				<xsl:with-param name="ns" select="''"/>
				<xsl:with-param name="value"
					select="if (@fixed) then @fixed else w2e:type-to-example($baseType, 'attr')"/>
				<xsl:with-param name="use" select="@use"/>
			</xsl:call-template>
			<xsl:variable name="name" select="@name"/>
			<xsl:variable name="chkUse" select="@*/name()"/>
			<!-- xsl:message>use 14 - <xsl:value-of select="$name"/> - <xsl:value-of select="$chkUse"/> <xsl:if test="not($chkUse[. = 'use'])"> YES</xsl:if></xsl:message -->
			<xsl:if test="not($chkUse[. = 'use'])">
				<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-use')"/>
				<xsl:attribute name="{$attrName}">optional</xsl:attribute>
			</xsl:if>
			<xsl:for-each select="@*">
				<xsl:if test="not(name()='name') and(not(starts-with(name(),$ns-prefix)))">
					<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-',name())"/>
					<xsl:attribute name="{$attrName}" select="."/>
				</xsl:if>
			</xsl:for-each>
			<xsl:variable name="typeChk"
				select="if(contains(@type,':')) then substring-after(@type,':') else @type"/>
			<!-- xsl:value-of select="w2e:message(concat('chk xsd:enumerations for simpleType:',@name,'+',$typeChk),4)"/-->
			<xsl:choose>
				<xsl:when
					test="count(//xsd:simpleType[@name=$typeChk]/xsd:restriction/xsd:enumeration) &gt; 0">
					<!-- xsl:value-of select="w2e:message(concat('xsd:enumerations for simpleType:',@name,'+',$typeChk),4)"/-->
					<xsl:variable name="matched" select="//xsd:simpleType[@name=$typeChk]"/>
					<!-- xsl:value-of select="w2e:message(concat('outputEnum (2): ',$name),4)"/ -->
					<xsl:call-template name="outputEnumeration">
						<xsl:with-param name="typeName" select="$name"/>
						<xsl:with-param name="ns" select="@targetNamespace"/>
						<xsl:with-param name="enumValues" select="$matched[1]/xsd:restriction/xsd:enumeration"/>
						<xsl:with-param name="mode"/>
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="count(//xsd:simpleType[@name=$typeChk]/xsd:restriction) &gt; 0">
					<!-- xsl:value-of select="w2e:message(concat('properties for attribute:',@name,'+',$typeChk),4)"/ -->
					<xsl:variable name="matched" select="//xsd:simpleType[@name=$typeChk]"/>
					<xsl:variable name="resolvedType" select="$matched/xsd:restriction/@base"/>
					<xsl:attribute name="{concat('xsd:attr-base-',local-name())}" select="$resolvedType[1]"/>
					<xsl:for-each select="$matched/xsd:restriction/*">
						<xsl:if test="contains($xsdTypeAttributes,name())">
							<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-',local-name())"/>
							<xsl:attribute name="{$attrName}" select="@value"/>
						</xsl:if>
					</xsl:for-each>
				</xsl:when>
			</xsl:choose>

		</xsl:if>
		<!-- xsl:attribute name="{@name}"><xsl:value-of select="w2e:type-to-example(@type, 'attr')"/>	</xsl:attribute -->
	</xsl:template>

	<xsl:template match="xsd:attribute[xsd:simpleType/xsd:restriction and @name]">
		<xsl:param name="useList"/>
		<xsl:variable name="chk" select="@name"/>
		<!-- xsl:message>2 -<xsl:value-of select="$chk"/>[<xsl:copy-of select="$useList"/>]</xsl:message -->
		<xsl:if test="not($useList//attribute[@e=$chk])">
			<!-- xsl:value-of select="w2e:message(concat('xsd:attribute with simpleType:',@name),4)"/ -->
			<xsl:call-template name="outputAttribute">
				<xsl:with-param name="name" select="@name"/>
				<xsl:with-param name="ns" select="''"/>
				<xsl:with-param name="value" select="if (@fixed) then @fixed else w2e:type-to-example(xsd:simpleType/xsd:restriction/@base, 'attr')"/>
				<xsl:with-param name="use" select="@use"/>
			</xsl:call-template>

			<xsl:variable name="name" select="@name"/>
			<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-type')"/>
			<xsl:attribute name="{$attrName}" select="xsd:simpleType/xsd:restriction/@base"/>

			<xsl:if test="count(xsd:simpleType/xsd:restriction/xsd:enumeration) &gt; 0">
				<!-- xsl:value-of select="w2e:message(concat('xsd:enumerations for base attribute:',$name),4)"/ -->
				<!-- xsl:value-of select="w2e:message(concat('outputEnum (3): ',$name),4)"/ -->
				<xsl:call-template name="outputEnumeration">
					<xsl:with-param name="typeName" select="$name"/>
					<xsl:with-param name="ns" select="@targetNamespace"/>
					<xsl:with-param name="enumValues" select="xsd:simpleType/xsd:restriction/xsd:enumeration"/>
					<xsl:with-param name="mode"/>
				</xsl:call-template>
			</xsl:if>
			<xsl:if test="count(xsd:simpleType/xsd:restriction) &gt; 0">
				<xsl:for-each select="xsd:simpleType/xsd:restriction/*">
					<xsl:if test="contains($xsdTypeAttributes,name())">
						<xsl:variable name="attrName"
							select="concat('xsd:attr-',$name,'-',local-name())"/>
						<xsl:attribute name="{$attrName}" select="@value"/>
					</xsl:if>
				</xsl:for-each>
			</xsl:if>
		</xsl:if>
		<!-- xsl:attribute name="{@name}"><xsl:value-of select="w2e:type-to-example(xsd:simpleType/xsd:restriction/@base, 'attr')"/>	</xsl:attribute -->
	</xsl:template>

	<xsl:template match="xsd:attribute[@ref]">
		<xsl:param name="useList"/>
		<xsl:variable name="chk" select="@ref"/>
		<!-- xsl:message>3 -<xsl:value-of select="$chk"/>[<xsl:copy-of select="$useList"/>]</xsl:message -->
		<xsl:if test="not($useList//attribute[@e=$chk])">
			<!-- xsl:value-of select="w2e:message(concat('xsd:attribute with ref:',@ref),4)"/ -->
			<xsl:variable name="ref"
				select="if (string-length(substring-after(@ref,':')) = 0) then @ref else substring-after(@ref,':')"/>
			<xsl:variable name="ns">
				<xsl:call-template name="namespace-uri-of-qname">
					<xsl:with-param name="qname" select="@ref"/>
				</xsl:call-template>
			</xsl:variable>

			<xsl:variable name="attrType">
				<xsl:choose>
					<xsl:when test="count(//xsd:attribute[@name=@ref]/@type) &gt; 0">
						<xsl:variable name="matched" select="//xsd:attribute[@name=@ref]/@type"/>
						<!-- this is needed for when people use include with same attribute definitions; hence duplicates can occur -->
						<xsl:value-of select="$matched[1]"/>
					</xsl:when>
					<xsl:when test="count(//xsd:attribute[@name=$ref]/@type) &gt; 0">
						<xsl:variable name="matched" select="//xsd:attribute[@name=$ref]/@type"/>
						<!-- this is needed for when people use include with same attribute definitions; hence duplicates can occur -->
						<xsl:value-of select="$matched[1]"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$ref"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:variable name="baseType">
				<xsl:call-template name="chkType">
					<xsl:with-param name="type" select="$attrType"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:call-template name="outputAttribute">
				<xsl:with-param name="name" select="$ref"/>
				<xsl:with-param name="ns" select="$ns"/>
				<xsl:with-param name="value"
					select="if (@fixed) then @fixed else w2e:type-to-example($baseType, 'attr')"/>
				<xsl:with-param name="use" select="@use"/>
			</xsl:call-template>
		</xsl:if>
		<!-- xsl:attribute name="{$ref}" namespace="{$ns}"><xsl:value-of select="w2e:type-to-example($attrType, 'attr')"/>	</xsl:attribute-->
	</xsl:template>

	<xsl:template name="chkType">
		<xsl:param name="type"/>
		<!-- check if there is a simpletype defined -->
		<xsl:variable name="simpleType" select="//xsd:simpleType[@name=$type]"/>
		<xsl:choose>
			<xsl:when test="count($simpleType) &gt; 0">
				<xsl:choose>
					<xsl:when test="count($simpleType[1]/xsd:restriction/xsd:enumeration) &gt; 0">
						<xsl:value-of
							select="$simpleType[1]/xsd:restriction/xsd:enumeration[1]/@value"/>
					</xsl:when>
					<xsl:when test="count($simpleType[1]/xsd:union) &gt; 0">
						<xsl:value-of select="$simpleType[1]/xsd:union/@memberTypes"/>
					</xsl:when>
					<xsl:when test="count($simpleType/xsd:restriction[contains(@base,'xsd:')])">
						<xsl:value-of
							select="$simpleType/xsd:restriction[contains(@base,'xsd:')][1]/@base"/>
					</xsl:when>
					<xsl:when test="count($simpleType/xsd:restriction[contains(@base,'xs:')])">
						<xsl:value-of
							select="$simpleType/xsd:restriction[contains(@base,'xs:')][1]/@base"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$simpleType/xsd:restriction/@base"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$type"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="outputAttribute">
		<xsl:param name="name"/>
		<xsl:param name="ns"/>
		<xsl:param name="value"/>
		<xsl:param name="use"/>
		<xsl:choose>
			<xsl:when test="string-length($ns) &gt; 0">
				<xsl:attribute name="{$name}" namespace="{$ns}">
					<xsl:value-of select="$value"/>
				</xsl:attribute>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="string-length($name) &gt; 0">
					<xsl:attribute name="{$name}">
						<xsl:value-of select="$value"/>
					</xsl:attribute>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:if test="$use = 'optional'">
			<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-use')"/>
			<xsl:attribute name="{$attrName}">optional</xsl:attribute>
		</xsl:if>
	</xsl:template>

	<xsl:template name="outputEnumeration">
		<xsl:param name="typeName"/>
		<xsl:param name="ns"/>
		<xsl:param name="enumValues"/>
		<xsl:param name="mode"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:enumerations: ',$typeName),4)"/ -->
		<xsl:variable name="values">
			<xsl:for-each select="$enumValues">
				<xsl:variable name="tempStr" select="./@value"/>
				<xsl:if test="position() &gt; 1"> |</xsl:if>'<xsl:value-of select="replace($tempStr,'''','\\''')"/>'</xsl:for-each>
		</xsl:variable>
		<xsl:value-of select="w2e:message(concat('enum values: ',$typeName,' - ',$values),3)"/>
		<xsl:if test="string-length($values) &gt; 1">
			<xsl:attribute name="{concat('xsd:attr-',$typeName,'-enumValues',$mode)}">
				<xsl:call-template name="chkUTF8">
					<xsl:with-param name="thisText" select="$values"/>
					<xsl:with-param name="context" select="$typeName"/>
				</xsl:call-template>
				<!-- xsl:value-of select="$values"/-->
			</xsl:attribute>
			<xsl:if test="contains($CAMmode,'anno')">
				<xsl:if test="count($enumValues/xsd:annotation/xsd:documentation) &gt; 0">
					<xsl:for-each select="$enumValues/xsd:annotation/xsd:documentation">
						<xsl:variable name="pos" select="position()"/>
						<xsl:attribute
							name="{concat('xsd:attr-',$typeName,'-',position(),'-enumAnno',$mode)}">
							<xsl:call-template name="chkUTF8">
								<xsl:with-param name="thisText"
									select="concat($enumValues[$pos]/@value,' - ',text())"/>
								<xsl:with-param name="context" select="$typeName"/>
							</xsl:call-template>
						</xsl:attribute>
					</xsl:for-each>
				</xsl:if>
			</xsl:if>
		</xsl:if>
	</xsl:template>

	<xsl:template name="outputRestrictions">
		<xsl:param name="typeName"/>
		<xsl:param name="ns"/>
		<xsl:param name="enumValues"/>
		<xsl:value-of select="w2e:message(concat('xsd:restrictions: ',$typeName),4)"/>
		<xsl:variable name="values">
			<xsl:for-each select="$enumValues/xsd:enumeration"><xsl:if test="./preceding-sibling::*"> |</xsl:if>'<xsl:value-of select="./@value"/>'</xsl:for-each>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('values: ',$values),4)"/ -->
		<xsl:attribute name="{concat('xsd:attr-',$typeName,'-enumValues')}">
			<xsl:call-template name="chkUTF8">
				<xsl:with-param name="thisText" select="$values"/>
				<xsl:with-param name="context" select="$typeName"/>
			</xsl:call-template>
			<!-- xsl:value-of select="$values"/ -->
		</xsl:attribute>
	</xsl:template>

	<!-- output templates and functions-->
	<xsl:template name="outputElementWithType">
		<xsl:param name="ancestors"/>
		<xsl:param name="ref-name"/>
		<xsl:param name="ref-attributes"/>
		<xsl:param name="parent"/>
		<xsl:param name="parentType"/>
		<!-- xsl:message> - element <xsl:value-of select="@type"/></xsl:message -->
		<xsl:variable name="chkType" select="@type"/>
		<xsl:variable name="type">
			<xsl:choose>
				<xsl:when test="count(//xsd:simpleType[@name=$chkType]/xsd:restriction/@base) &gt; 0">
					<xsl:variable name="matched" select="//xsd:simpleType[@name=$chkType]/xsd:restriction/@base"/>
					<xsl:value-of select="$matched[1]"/>
				</xsl:when>
				<xsl:when test="count(//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:extension/@base) &gt; 0">
					<xsl:variable name="matched" select="//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:extension/@base"/>
					<xsl:variable name="chkMatch"
						select="if (string-length(substring-after($matched[1],':')) = 0) then $matched[1] else substring-after($matched[1],':')"/>
					<xsl:value-of select="w2e:message(concat('MatchedType: ',@name,' - ',$chkMatch),4)"/>
					<xsl:choose>
						<xsl:when test="count(//xsd:simpleType[@name=$chkMatch]/xsd:restriction/@base) &gt; 0">
							<xsl:variable name="getType" select="//xsd:simpleType[@name=$chkMatch]/xsd:restriction/@base"/>
							<xsl:value-of select="if (string-length(substring-after($getType[1],':')) = 0) then $getType[1] else substring-after($getType[1],':')"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$chkMatch"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="if (string-length(substring-after(@type,':')) = 0) then @type else substring-after(@type,':')"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="namespace-for-name">
			<xsl:choose>
				<!-- xsl:when test="string-length($targetNsPrefix) &gt; 0"><xsl:value-of select="$targetNsPrefix"/></xsl:when -->
				<xsl:when test="count(ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace) &gt; 0">
					<xsl:value-of select="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace"/>
				</xsl:when>
				<xsl:otherwise>asm1</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:variable name="ns">
			<xsl:call-template name="namespace-uri-of-qname">
				<xsl:with-param name="qname" select="@type"/>
				<xsl:with-param name="context" select="@targetNamespace"/>
			</xsl:call-template>
		</xsl:variable>
		
		<!-- xsl:value-of select="w2e:message(concat('$namespace-for-name=',$namespace-for-name,' $ns- ',$ns),3)"/ -->
		
		<xsl:variable name="ns-to-use">
			<xsl:value-of select="if (string-length($ns) = 0) then $namespace-for-name else $ns"/>
		</xsl:variable>
		<xsl:variable name="prefix1">
			<xsl:call-template name="prefix-for-namespace">
				<xsl:with-param name="namespace-uri" select="$namespace-for-name"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="prefix">
			<!-- xsl:value-of select="if (string-length($prefix1) = 1) then concat(generate-id(),':') else $prefix1"/ -->
			<xsl:value-of select="if (string-length($prefix1) = 1) then ' ' else $prefix1"/>
		</xsl:variable>
		<xsl:variable name="fullname" select="if (string-length($prefix) = 1) then @name else concat($prefix,@name)"/>
		<xsl:value-of select="w2e:message(concat('element fullname=',$fullname),3)"/>
		
		<xsl:choose>
			<xsl:when test="string-length($chkType) &gt; 0 and( count(//xsd:complexType[@name = $chkType]/xsd:choice) &gt; 0)">
				<xsl:value-of select="w2e:message(concat(' - element ',@name,' of type: ',$chkType,' is a choice!'),2)"/>
				<xsl:variable name="choiceComponents" select="//xsd:complexType[@name = $chkType]/xsd:choice"/>
				<xsl:element name="{@name}" namespace="{$namespace-for-name}">
					<!-- reference element rules output -->
					<xsl:if test="string-length($ref-name) &gt; 0">
						<xsl:for-each select="$ref-attributes">
							<!-- xsl:value-of select="w2e:message(concat($ref-name,' reference attr=',name(),'-',.),3)"/ -->
							<xsl:if test="not(contains(name(),':'))">
								<xsl:variable name="attrName" select="concat('xsd:',name())"/>
								<xsl:attribute name="{$attrName}" select="."/>
							</xsl:if>
						</xsl:for-each>
					</xsl:if>
					<xsl:if test="count(//xsd:complexType[@name=$chkType]/xsd:attribute|xsd:attributeGroup) &gt; 0">
						<!-- xsl:message> - attributes - <xsl:value-of select="$chkType"/> @<xsl:value-of select="//xsd:complexType[@name=$chkType]/xsd:attribute/@name|xsd:attributeGroup/@ref"/></xsl:message -->
						<xsl:variable name="theseAttributes" select="//xsd:complexType[@name=$chkType]/xsd:attribute"/>
						<xsl:variable name="attribNames">
							<xsl:value-of select="//xsd:complexType[@name=$chkType]/xsd:attribute/@name"/>
						</xsl:variable>
						<xsl:variable name="useAttrChk">
							<xsl:call-template name="chkExcludeAttr">
								<xsl:with-param name="names" select="$attribNames"/>
								<xsl:with-param name="parent" select="$parent"/>
							</xsl:call-template>
						</xsl:variable>
						<!-- xsl:message> - picked <xsl:value-of select="concat('choice-attrs: ',$chkType,' + ',count($theseAttributes),' attrs ',$attribNames)"/></xsl:message -->
						<xsl:value-of select="w2e:message(concat('choice-attrs: ',$chkType,' + ',count($theseAttributes),' attrs ',$attribNames),4)"/>
						<xsl:apply-templates select="$theseAttributes">
							<xsl:with-param name="useList" select="$useAttrChk"/>
						</xsl:apply-templates>
						<xsl:apply-templates select="//xsd:complexType[@name=$chkType]/xsd:attributeGroup">
							<xsl:with-param name="parent" select="$parent"/>
						</xsl:apply-templates>
					</xsl:if>
					<xsl:apply-templates select="$choiceComponents[1]">
						<xsl:with-param name="ancestors" select="$ancestors"/>
						<xsl:with-param name="ref-name" select="$ref-name"/>
						<xsl:with-param name="ref-attributes" select="$ref-attributes"/>
						<xsl:with-param name="parent" select="$parent"/>
						<xsl:with-param name="parentType" select="$parentType"/>
					</xsl:apply-templates>
				</xsl:element>
			</xsl:when>
		<xsl:otherwise>
			
		<xsl:value-of select="w2e:message(concat('outputElementwithType: ',$fullname,' (',@name,') - ',$type,' / ',@type,' ns ',$namespace-for-name),3)"/>
		<!-- xsl:message> - namespace for <xsl:value-of select="$fullname"/> - <xsl:value-of select="$prefix"/> - <xsl:value-of select="$prefix1"/></xsl:message -->
		<xsl:element name="{$fullname}" namespace="{$namespace-for-name}">
			<xsl:if test="$prefix1 != $prefix and(string-length($prefix) &gt; 0)">
				<xsl:namespace name="{if(contains($prefix,':')) then substring-before($prefix,':') else $prefix}" select="$namespace-for-name"/>
			</xsl:if>

			<!-- reference element rules output -->
			<xsl:if test="string-length($ref-name) &gt; 0">
				<xsl:for-each select="$ref-attributes">
					<!-- xsl:value-of select="w2e:message(concat($ref-name,' reference attr=',name(),'-',.),3)"/ -->
					<xsl:if test="not(contains(name(),':'))">
						<xsl:variable name="attrName" select="concat('xsd:',name())"/>
						<xsl:attribute name="{$attrName}" select="."/>
					</xsl:if>
				</xsl:for-each>
			</xsl:if>

			<!-- element cardinality rules being output -->
			<xsl:for-each select="@*">
				<xsl:if test="not(contains(name(),':'))">
					<xsl:variable name="attrName" select="concat('xsd:',name())"/>
					<xsl:attribute name="{$attrName}" select="."/>
				</xsl:if>
			</xsl:for-each>

			<!-- see if there is a base type being used -->
			<xsl:if test="count(//xsd:complexType[@name=$type]/xsd:simpleContent/xsd:extension/@base) &gt; 0">
				<xsl:variable name="baseTypes" select="//xsd:complexType[@name=$type]/xsd:simpleContent/xsd:extension/@base"/>
				<xsl:variable name="baseType" select="if(contains($baseTypes[1],':')) then substring-after($baseTypes[1],':') else $baseTypes[1]"/>
				<xsl:variable name="resolvedType">
					<xsl:choose>
						<xsl:when test="count(//xsd:complexType[@name=$baseType]/xsd:simpleContent/xsd:extension/@base) &gt; 0">
							<xsl:variable name="typeFound" select="//xsd:complexType[@name=$baseType]/xsd:simpleContent/xsd:extension/@base"/>
							<xsl:value-of select="$typeFound[1]"/>
						</xsl:when>
						<xsl:when test="count(//xsd:complexType[@name=$baseTypes[1]]/xsd:simpleContent/xsd:extension/@base) &gt; 0">
							<xsl:variable name="typeFound" select="//xsd:complexType[@name=$baseTypes[1]]/xsd:simpleContent/xsd:extension/@base"/>
							<xsl:value-of select="$typeFound[1]"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$baseTypes[1]"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				<xsl:attribute name="{'xsd:base'}" select="$resolvedType"/>
				
				<!-- Now see if there is an enumeration too -->
				<xsl:variable name="chkResolvedType" select="if (contains($resolvedType[1],':')) then substring-after($resolvedType[1],':') else $resolvedType[1]"/>
				<xsl:if test="count(//xsd:simpleType[@name=$chkResolvedType]/xsd:restriction/xsd:enumeration) &gt; 0">
					<!-- xsl:value-of select="w2e:message(concat('outputElementEnumeration: ',@name,' - ',$resolvedType),4)"/ -->
					<xsl:variable name="matched" select="//xsd:simpleType[@name=$chkResolvedType]"/>
					<!-- xsl:value-of select="w2e:message(concat('outputEnum (4): ',@name),4)"/ -->
					<xsl:call-template name="outputEnumeration">
						<xsl:with-param name="typeName" select="@name"/>
						<xsl:with-param name="ns" select="@targetNamespace"/>
						<xsl:with-param name="enumValues" select="$matched[1]/xsd:restriction/xsd:enumeration"/>
						<xsl:with-param name="mode" select="'-E'"/>
					</xsl:call-template>
				</xsl:if>
				<xsl:if test="count(//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:extension/xsd:attribute) &gt; 0">
					<xsl:variable name="theseAttributes" select="//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:extension/xsd:attribute"/>
					<xsl:variable name="useAttrChk">
						<xsl:call-template name="chkExcludeAttr">
							<xsl:with-param name="names" select="if ($theseAttributes//@ref) then $theseAttributes/xsd:attribute/@ref else $theseAttributes//@name"/>
							<xsl:with-param name="parent" select="$parent"/>
						</xsl:call-template>
					</xsl:variable>
					<xsl:value-of select="w2e:message(concat('extension-attrs: ',$chkType,' + ',count($theseAttributes),' attrs'),4)"/>
					<xsl:apply-templates select="$theseAttributes">
						<xsl:with-param name="useList" select="$useAttrChk"/>
					</xsl:apply-templates>
				</xsl:if>
			</xsl:if>

			<!-- xsl:value-of select="w2e:message(concat('@type name: ',$type),4)"/ -->
			<!-- xsl:value-of select="w2e:message(concat('match ',concat('(',@name,'+',@type,')'),'ancestors:',$ancestors),4)"/ -->
			<xsl:choose>
				<xsl:when test="not(contains($ancestors,concat('(',@name,'+',@type,')')))">
					<xsl:variable name="name1" select="@name"/>
					<xsl:variable name="type1" select="@type"/>
					<!-- xsl:value-of select="w2e:message(concat('not enclosed $ns-to-use=',$ns-to-use,';$type=',$type),4)"/ -->
					<xsl:variable name="matched"
						select="if (count(//xsd:complexType[@name=$type and ancestor::xsd:schema[1]/@targetNamespace=$ns-to-use])) then //xsd:complexType[@name=$type and ancestor::xsd:schema[1]/@targetNamespace=$ns-to-use] else //xsd:complexType[@name=$chkType and ancestor::xsd:schema[1]/@targetNamespace=$ns-to-use]"/>

					<!--<xsl:value-of select="w2e:message(concat('matched ',$matched),3)"/>-->
					<!-- find the matched item that matches the namespace of the type reference -->
					<xsl:variable name="chkNS" select="$matched[1]/ancestor::xsd:schema[1]/@targetNamespace[1]"/>
					<xsl:value-of select="w2e:message(concat('type chkNS ',$chkNS),3)"/>
					<xsl:choose>
						<xsl:when test="$chkNS=$ns-to-use">
							<xsl:value-of select="w2e:message(concat('first match NS ',$chkNS),4)"/>
							<xsl:apply-templates select="$matched[1]">
								<xsl:with-param name="ns" select="$ns-to-use"/>
								<xsl:with-param name="ancestors" select="concat($ancestors,'(',$name1,'+',$type1,')')"/>
								<xsl:with-param name="parent" select="concat($parent,'\',$name1)"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:otherwise>
							<xsl:variable name="prefixNS">
								<!-- check namespace prefix of base item -->
								<xsl:choose>
									<xsl:when
										test="count(//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:extension/@base) &gt; 0">
										<xsl:variable name="matched"
											select="//xsd:complexType[@name=$chkType and ancestor::xsd:schema[1]/@targetNamespace=$ns-to-use]/xsd:simpleContent/xsd:extension/@base"/>
										<xsl:variable name="this"
											select="$matched[1]/parent::xsd:schema/parent::xsd:import/@namespace[1]"/>
										<xsl:value-of select="$this[1]"/>
									</xsl:when>
									<xsl:when
										test="count(//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:restriction/@base) &gt; 0">
										<xsl:variable name="matched"
											select="//xsd:complexType[@name=$chkType and ancestor::xsd:schema[1]/@targetNamespace=$ns-to-use]/xsd:simpleContent/xsd:restriction/@base"/>
										<xsl:variable name="this"
											select="$matched[1]/parent::xsd:schema/parent::xsd:import/@namespace[1]"/>
										<xsl:value-of select="$this[1]"/>
									</xsl:when>
									<xsl:otherwise/>
								</xsl:choose>
							</xsl:variable>
							<xsl:value-of select="w2e:message(concat('type prefixNS ',$prefixNS),4)"/>
							<xsl:variable name="matchedRef">
								<xsl:for-each select="$matched">
									<xsl:variable name="chkNS1"
										select="parent::xsd:schema/parent::xsd:schema[1]/@targetNamespace[1]"/>
									<!-- xsl:value-of select="w2e:message(concat('compareNS ',$chkNS1),4)"/ -->
									<xsl:choose>
										<xsl:when test="string-length($chkNS1)=0">0</xsl:when>
										<xsl:when test="$chkNS1=$prefixNS and(string-length($prefixNS) &gt; 0)">1</xsl:when>
										<xsl:when test="$chkNS1=$ns-to-use">1</xsl:when>
										<xsl:otherwise>0</xsl:otherwise>
									</xsl:choose>
								</xsl:for-each>
							</xsl:variable>
							<!-- xsl:message>+ type: [<xsl:value-of select="$type"/>][<xsl:value-of select="$chkType"/>]</xsl:message -->
							<xsl:choose>
								<xsl:when test="contains($matchedRef,'1')">
									<xsl:variable name="offset" select="string-length(concat(substring-before($matchedRef,'1'),'1'))"/>
									<!-- xsl:value-of select="w2e:message(concat('matchedRef NS ',$offset),4)"/ -->
									<xsl:apply-templates select="$matched[$offset]">
										<xsl:with-param name="ns" select="$ns-to-use"/>
										<xsl:with-param name="ancestors" select="concat($ancestors,'(',$name1,'+',$type1,')')"/>
										<xsl:with-param name="parent" select="concat($parent,'\',$name1)"/>
									</xsl:apply-templates>
								</xsl:when>
								<xsl:when test="//xsd:complexType[@name=$type]/xsd:sequence and(count($matched[1])=0)">
									<xsl:apply-templates select="//xsd:complexType[@name=$type]">
										<xsl:with-param name="ns" select="substring-before($elementNameValue,':')"/>
										<xsl:with-param name="ancestors" select="concat($ancestors,'(',$name1,'+',$type1,')')"/>
										<xsl:with-param name="parent" select="concat($parent,'\',$name1)"/>
									</xsl:apply-templates>
								</xsl:when>
								<xsl:when test="//xsd:complexType[@name=$type]/xsd:all and(count($matched[1])=0)">
									<xsl:value-of select="w2e:message(concat('matched xsd:all ',@name,'+',$type),4)"/>
									<xsl:apply-templates select="//xsd:complexType[@name=$type]/xsd:all">
										<xsl:with-param name="ns" select="substring-before($elementNameValue,':')"/>
										<xsl:with-param name="ancestors" select="concat($ancestors,'(',$name1,'+',$type1,')')"/>
										<xsl:with-param name="parent" select="concat($parent,'\',$name1)"/>
									</xsl:apply-templates>
								</xsl:when>
								<xsl:when test="//xsd:simpleType[@name=$type]/xsd:union">
									<xsl:value-of select="w2e:message(concat('matched simpleType with union ',@name,'+',$type),4)"/>
									<xsl:variable name="matchedType" select="//xsd:simpleType[@name=$type]/xsd:union"/>
									<xsl:choose>
										<xsl:when test="//xsd:simpleType[@name=$type]/xsd:union">
											<xsl:apply-templates select="$matchedType[1]">
												<xsl:with-param name="typeName"><xsl:value-of select="$type"/></xsl:with-param>
											</xsl:apply-templates>
										</xsl:when>
										<xsl:otherwise>
											<xsl:apply-templates select="$matchedType[1]" mode="simple"/>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:when>
								<xsl:when test="//xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration">
									<xsl:value-of select="w2e:message(concat('matched simpleType with enumeration ',@name,'+',$type),4)"/>
									<xsl:call-template name="outputEnumeration">
										<xsl:with-param name="typeName" select="$type"/>
										<xsl:with-param name="ns" select="@targetNamespace"/>
										<xsl:with-param name="enumValues" select="//xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration"/>
										<xsl:with-param name="mode" select="'-E'"/>
									</xsl:call-template>
								</xsl:when>
								<xsl:when test="//xsd:simpleType[@name=$chkType] and(count($matched[1])=0)">
									<xsl:value-of select="w2e:message(concat('matched simpleType base ',@name,'+',$chkType),4)"/>
									<xsl:variable name="matchedType" select="//xsd:simpleType[@name=$chkType]"/>
									<xsl:choose>
										<xsl:when test="//xsd:simpleType[@name=$chkType]/xsd:union">
											<xsl:apply-templates select="$matchedType[1]">
												<xsl:with-param name="typeName"><xsl:value-of select="$chkType"/></xsl:with-param>
											</xsl:apply-templates>
										</xsl:when>
										<xsl:otherwise>
											<xsl:apply-templates select="$matchedType[1]" mode="simple"/>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:when>
								<xsl:when test="//xsd:simpleType[@name=$type] and(count($matched[1])=0)">
									<xsl:value-of select="w2e:message(concat('matched simpleType type ',@name,'+',$type),4)"/>
									<xsl:variable name="matchedType" select="//xsd:simpleType[@name=$type]"/>
									<xsl:variable name="matchedBase" select="//xsd:simpleType[@name=$type]/xsd:restriction/@base"/>
									<xsl:variable name="chkBaseType" select="substring-after($matchedBase[1],':')"/>
									<xsl:variable name="matchedBaseType" select="$entireXSDcollection//xsd:simpleType[@name=$chkBaseType]"/>
									<xsl:value-of select="w2e:message(concat('chk base type ',@name,'+',$chkBaseType),4)"/>
									<!-- Note: Ignore restrictions from the parent type, use only the ones from the base type -->
									<xsl:choose>
										<xsl:when test="//xsd:simpleType[@name=$type]/xsd:union">
											<xsl:apply-templates select="$matchedType[1]">
												<xsl:with-param name="typeName"><xsl:value-of select="$type"/></xsl:with-param>
											</xsl:apply-templates>
										</xsl:when>
										<xsl:when test="count($matchedBaseType/.) &gt; 0">
											<xsl:value-of select="w2e:message(concat('apply simpleType type ',@name,'+',$chkBaseType),4)"/>
											<!-- xsl:variable name="thisType" select="$matchedBaseType[1]/xsd:restriction/@base"/ -->
											<xsl:apply-templates select="$matchedBaseType[1]" mode="simple"/>
										</xsl:when>
										<xsl:otherwise>
											<xsl:apply-templates select="$matchedType[1]" mode="simple"/>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:when>
								<xsl:otherwise>
									<!-- Look to see if it references a type anywhere -->
									<xsl:value-of
										select="w2e:message(concat('unmatched base ',@name,'+',$type),4)"/>
									<xsl:variable name="unmatch"
										select="if(count($entireXSDcollection//xsd:complexType[@name=$chkType])) then $entireXSDcollection//xsd:complexType[@name=$chkType] else $entireXSDcollection//xsd:complexType[@name=$type] "/>
									<!-- xsl:message><xsl:value-of select="count($matched[1])"></xsl:value-of>[<xsl:copy-of select="$unmatch[1]"></xsl:copy-of>]</xsl:message -->
									<xsl:choose>
										<xsl:when test="count(unmatch[1]) &gt; 0">
											<!-- complexType found -->
											<xsl:apply-templates select="$unmatch[1]">
												<xsl:with-param name="ns" select="$ns-to-use"/>
												<xsl:with-param name="ancestors"
												select="concat($ancestors,'(',$name1,'+',$type1,')')"/>
												<xsl:with-param name="parent"
												select="concat($parent,'\',$name1)"/>
											</xsl:apply-templates>
										</xsl:when>
										<xsl:otherwise>
											<!-- must be a simpleType or base XSD type -->
											<xsl:value-of select="w2e:message(concat('matched simpleType ',@name,'+',$type),4)"/>
											<xsl:variable name="simpleType"
												select="if(count($entireXSDcollection//xsd:simpleType[@name=$chkType])) then $entireXSDcollection//xsd:simpleType[@name=$chkType] else $entireXSDcollection//xsd:simpleType[@name=$type]"/>
											<!-- xsl:message> simpleType <xsl:value-of select="$chkType"/> - <xsl:value-of select="$type"/> [<xsl:copy-of select="$simpleType[1]"/>]</xsl:message -->
											<xsl:choose>
												<xsl:when test="count($simpleType[1]) = 0"><!-- type is base XSD type - e.g. date, integer, string, etc. -->
													<xsl:value-of select="w2e:message(concat(' - base XSD type for ',@name,'+',$type),4)"/>
												</xsl:when>
												<xsl:otherwise>
												  <xsl:apply-templates select="$simpleType[1]">
													<xsl:with-param name="ns" select="$ns-to-use"/>
													<xsl:with-param name="ancestors" select="concat($ancestors,'(',$name1,'+',$type1,')')"/>
													<xsl:with-param name="parent" select="concat($parent,'\',$name1)"/>
												  </xsl:apply-templates>
												</xsl:otherwise>
											</xsl:choose>
										</xsl:otherwise>
									</xsl:choose>

								</xsl:otherwise>
							</xsl:choose>

						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<!-- xsl:value-of select="w2e:message(concat('enclosed recursive:',$type),2)"/ -->
					<xsl:message> - enclosed recursive: <xsl:value-of select="$type"/></xsl:message>
					<xsl:comment>recursive definition - type = <xsl:value-of select="$type"/></xsl:comment>
					<!--<xsl:apply-templates select="//xsd:complexType[@name=$type]" mode="enclosed">
						<xsl:with-param name="ns" select="$ns-to-use"/>
					</xsl:apply-templates>-->
				</xsl:otherwise>
			</xsl:choose>
			<xsl:choose>
				<xsl:when test="@fixed">
					<xsl:value-of select="@fixed"/>
				</xsl:when>
				<xsl:when test="//xsd:simpleType[@name=$type]/xsd:list"><xsl:value-of select="w2e:type-to-example(concat('&quot;',//xsd:simpleType[@name=$type]/xsd:list[1]/@itemType,' list','&quot;'),'value')"/></xsl:when>
				<xsl:when test="//xsd:simpleType[@name=$type]">
					<!-- xsl:value-of select="w2e:message(concat('outputEnum (5): ',$type),4)"/ -->
					<!-- xsl:call-template name="outputEnumeration">
						<xsl:with-param name="typeName" select="$type"/>
						<xsl:with-param name="ns" select="@targetNamespace"/>
						<xsl:with-param name="enumValues" select="//xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration"/>
						<xsl:with-param name="mode" select="'-E'"/>
					</xsl:call-template -->
					<xsl:variable name="typeBase"
						select="if (count(//xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration) &gt; 0) then //xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration[1]/@value else //xsd:simpleType[@name=$type]/xsd:restriction[1]/@base"/>
					<xsl:value-of
						select="if (count(//xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration) &gt; 0) then w2e:type-to-example(concat('&quot;',$typeBase[1],'&quot;'),'value') else if (string-length($typeBase[1]) &gt; 0) then w2e:type-to-example($typeBase[1],'type') else  w2e:type-to-example($type,'type')"/>
					<!-- xsl:value-of select="w2e:type-to-example(concat('&quot;',//xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration[1]/@value,'&quot;'),'value')"/ -->
				</xsl:when>
				<xsl:when test="//xsd:simpleType[@name=$chkType]">
					<!-- xsl:value-of select="w2e:message(concat('outputEnum (6): ',$chkType),4)"/ -->
					<xsl:call-template name="outputEnumeration">
						<xsl:with-param name="typeName" select="$chkType"/>
						<xsl:with-param name="ns" select="@targetNamespace"/>
						<xsl:with-param name="enumValues" select="//xsd:simpleType[@name=$chkType]/xsd:restriction/xsd:enumeration"/>
						<xsl:with-param name="mode" select="'-E'"/>
					</xsl:call-template>
					<xsl:variable name="typeBase"
						select="if (count(//xsd:simpleType[@name=$chkType]/xsd:restriction/xsd:enumeration) &gt; 0) then //xsd:simpleType[@name=$chkType]/xsd:restriction/xsd:enumeration[1]/@value else //xsd:simpleType[@name=$chkType]/xsd:restriction[1]/@base"/>
					<xsl:value-of
						select="if (count(//xsd:simpleType[@name=$chkType]/xsd:restriction/xsd:enumeration) &gt; 0) then w2e:type-to-example(concat('&quot;',$typeBase[1],'&quot;'),'value') else if (string-length($typeBase[1]) &gt; 0) then w2e:type-to-example($typeBase[1],'type') else  w2e:type-to-example($type,'type')"/>
					<!-- xsl:value-of select="w2e:type-to-example(concat('&quot;',//xsd:simpleType[@name=$chkType]/xsd:restriction/xsd:enumeration[1]/@value,'&quot;'),'value')"/ -->
				</xsl:when>
				<xsl:when test="not(//xsd:complexType[@name=$type]) and(not(//xsd:complexType[@name=$chkType]))"><xsl:value-of select="w2e:type-to-example($type,'type')"/></xsl:when>
				<xsl:when test="contains($type,'xsd:') or(contains($type,'xs:'))"><xsl:value-of select="w2e:type-to-example($type,'type')"/></xsl:when>
				<xsl:otherwise/>
			</xsl:choose>
			<!-- xsl:value-of select="w2e:message(concat('Augmentation check=',$ref-name,' - ',$parentType),3)"/ -->
			<xsl:variable name="augmentationChk"
				select="distinct-values($entireXSDcollection//xsd:element[@name=$ref-name][@type=$parentType]/@type)"/>
			<xsl:for-each select="$augmentationChk">
				<xsl:variable name="thisType" select="."/>
				<!-- xsl:value-of select="w2e:message(concat('base type found=',$thisType),3)"/ -->
				<xsl:variable name="typeNameChk"
					select="if(contains($thisType,':')) then substring-after($thisType,':') else $thisType"/>
				<xsl:variable name="augmentationDetail"
					select="$entireXSDcollection//xsd:complexType[@name=$typeNameChk]/xsd:complexContent/xsd:extension[@base=$thisType]/xsd:sequence/xsd:element"/>
				<xsl:for-each select="$augmentationDetail">
					<xsl:value-of select="w2e:message(concat('Augmentation item=',@ref),3)"/>
					<xsl:apply-templates select=".">
						<xsl:with-param name="ns" select="substring-before(@ref,':')"/>
						<xsl:with-param name="ancestors" select="$ancestors"/>
						<xsl:with-param name="parent" select="$parent"/>
					</xsl:apply-templates>
				</xsl:for-each>
			</xsl:for-each>
			<xsl:value-of select="w2e:message(concat('closing fullname=',$fullname),3)"/>
		</xsl:element>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="outputElementWithNoType">
		<xsl:param name="ancestors"/>
		<xsl:param name="ref-attributes"/>
		<xsl:param name="parent"/>
		<xsl:value-of select="w2e:message(concat('outputElementNoType: ',@name),4)"/>
		<xsl:variable name="type"
			select="if (string-length(substring-after(@type,':')) = 0) then @type else substring-after(@type,':') "/>
		<xsl:variable name="namespace-for-name">
			<xsl:choose>
				<xsl:when
					test="count(ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace) &gt; 0">
					<xsl:value-of
						select="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace"/>
				</xsl:when>
				<xsl:otherwise>asm1</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('$namespace-for-name=',$namespace-for-name),3)"/ -->
		<xsl:variable name="ns">
			<xsl:call-template name="namespace-uri-of-qname">
				<xsl:with-param name="qname" select="@type"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ns-to-use">
			<xsl:value-of select="if (string-length($ns) = 0) then $namespace-for-name else $ns"/>
		</xsl:variable>
		<xsl:variable name="prefix1">
			<xsl:call-template name="prefix-for-namespace">
				<xsl:with-param name="namespace-uri" select="$namespace-for-name"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="prefix">
			<!-- xsl:value-of select="if (string-length($prefix1) = 1) then concat(generate-id(),':') else $prefix1"/ -->
			<xsl:value-of select="if (string-length($prefix1) = 1) then '' else $prefix1"/>
		</xsl:variable>
		<xsl:variable name="fullname"
			select="if (string-length($prefix) = 0) then @name else concat($prefix,@name)"/>
		<!-- xsl:value-of select="w2e:message(concat('fullname=',$fullname),3)"/ -->
		<xsl:element name="{$fullname}" namespace="{$namespace-for-name}">

			<xsl:if test="$prefix1 != $prefix and(string-length($prefix) &gt; 0)">
				<xsl:namespace name="{substring-before($prefix,':')}" select="$namespace-for-name"/>
			</xsl:if>
			<xsl:for-each select="@*">
				<xsl:if test="not(contains(name(),':'))">
					<xsl:variable name="attrName" select="concat('xsd:',name())"/>
					<xsl:attribute name="{$attrName}" select="."/>
				</xsl:if>
			</xsl:for-each>

			<!-- xsl:value-of select="w2e:message(concat($ref-name,' reference attr=',name(),'-',.),3)"/ -->
			<xsl:if test="count($ref-attributes) &gt; 1">
				<xsl:for-each select="$ref-attributes">
					<xsl:variable name="attrDetail" select="."/>
					<xsl:value-of select="w2e:message(concat(' ref attr=',name($attrDetail)),3)"/>
					<xsl:choose>
						<xsl:when test="contains(name($attrDetail),':')">
							<xsl:attribute name="{name($attrDetail)}" select="."/>
						</xsl:when>
						<xsl:when test="name($attrDetail)='ref'"/>
						<xsl:otherwise>
							<xsl:attribute name="xsd:{name($attrDetail)}" select="."/>
						</xsl:otherwise>
					</xsl:choose>

				</xsl:for-each>
			</xsl:if>

			<xsl:choose>
				<xsl:when test="xsd:complexType">
					<xsl:value-of select="w2e:message(concat('element with complexType found ',$fullname),3)"/>
					<xsl:apply-templates select="xsd:complexType">
						<xsl:with-param name="ancestors" select="concat($ancestors,'(',@name,')')"/>
						<xsl:with-param name="parent" select="concat($parent,'\',@name)"/>
					</xsl:apply-templates>
				</xsl:when>
				<xsl:when test="@fixed"><xsl:value-of select="@fixed"/></xsl:when>
				<xsl:when test="xsd:simpleType/xsd:restriction/xsd:enumeration">%"<xsl:value-of select="xsd:simpleType/xsd:restriction/xsd:enumeration[1]/@value"/>"%</xsl:when>
				<xsl:when test="xsd:simpleContent/xsd:extension/@base">
						<xsl:variable name="baseTypeName" select="xsd:simpleContent/xsd:extension/@base"/>
						<xsl:value-of select="w2e:message(concat('outputElementEnumeration from extension: ',@name,' - ',$baseTypeName[1]),1)"/>
						<xsl:variable name="matched" select="//xsd:simpleType[@name=$baseTypeName[1]]"/>
						<xsl:call-template name="outputEnumeration">
							<xsl:with-param name="typeName" select="@name"/>
							<xsl:with-param name="ns" select="@targetNamespace"/>
							<xsl:with-param name="enumValues" select="$matched[1]/xsd:restriction/xsd:enumeration"/>
							<xsl:with-param name="mode" select="'-E'"/>
						</xsl:call-template>
				</xsl:when>
				<xsl:otherwise><xsl:text>%anyContent%</xsl:text></xsl:otherwise>
			</xsl:choose>
			<!-- xsl:value-of select="w2e:message(concat('closing fullname=',$fullname),3)"/ -->
		</xsl:element>
		
	</xsl:template>

	<xsl:template name="outputElementWithRef">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<xsl:value-of select="w2e:message(concat('outputElementWithRef: ',@ref),3)"/>
		<xsl:variable name="ref-name" select="@ref"/>
		<xsl:variable name="ref">
			<xsl:choose>
				<xsl:when test="count(//xsd:element[@name=$ref-name]) &gt; 0">
					<xsl:value-of select="$ref-name"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="if (string-length(substring-after(@ref,':')) = 0) then @ref else substring-after(@ref,':')"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="ref-ns">
			<xsl:choose>
				<xsl:when test="contains(@ref,':')"><xsl:value-of select="substring-before(@ref,':')"/></xsl:when>
				<xsl:when test="not($targetNsPrefix = '')"><xsl:value-of select="$targetNsPrefix"/></xsl:when>
				<xsl:otherwise/>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select="w2e:message(concat('ref=',$ref,' - ns=',$ref-ns,' refname=',$ref-name),2)"/>
		<xsl:variable name="ref-attributes" select="@*"/>

		<xsl:variable name="matchedType">
			<xsl:for-each select="//xsd:element[@name=$ref]">
				<xsl:variable name="chkTypeNS" select="substring-before(@type,':')"/>
				<xsl:variable name="thisNamespaceURI" select="namespace-uri-for-prefix($ref-ns,.)"/>
				<xsl:variable name="thisNamespaces" select="ancestor::*/./@targetNamespace"/>
				<xsl:choose>
					<xsl:when test="$chkTypeNS=$ref-ns"><xsl:value-of select="position()"/>|</xsl:when>
					<xsl:when test="$thisNamespaceURI = $thisNamespaces[1] and(count($thisNamespaces) = 1)"><xsl:value-of select="position()"/>|</xsl:when>
					<xsl:otherwise/>
				</xsl:choose>
				<!-- xsl:message> namespace (<xsl:value-of select="namespace-uri-for-prefix($ref-ns,.)"/>) <xsl:value-of select="ancestor::*/./@targetNamespace"/></xsl:message -->
			</xsl:for-each>
			<xsl:value-of select="0"/>
		</xsl:variable>
		<!-- xsl:message> matched: <xsl:value-of select="$ref"/> - <xsl:value-of select="$matchedType"/></xsl:message -->
		<xsl:variable name="matchedPosition" select="xsd:integer(tokenize($matchedType,'\|')[1])"/>
		
		<!-- xsl:value-of select="w2e:message(concat('matching position=',$matchedPosition),3)"/ -->
		<!-- Namespace aware matching on reference type for augmentation types [starts-with(@type,$ref-ns)] -->
		<xsl:for-each select="//xsd:element[@name=$ref]">
			<!-- xsl:value-of select="w2e:message(concat('matching ref=',@name,' - type=',@type),3)"/ -->
			<xsl:variable name="chkTypeNS" select="substring-before(@type,':')"/>
			<!-- xsl:if test="$chkTypeNS=$ref-ns"><xsl:value-of select="w2e:message(concat('matching ns=',$chkTypeNS,' - type=',@type),3)"/></xsl:if -->
			<xsl:choose>
				<xsl:when
					test="position()=1 and($matchedPosition &lt; 1) or(position()=$matchedPosition)">
					<!-- get first exact match only (included schemas may have duplicated definitions) -->
					<xsl:apply-templates select=".">
						<xsl:with-param name="ancestors" select="concat($ancestors,'(',$ref,')')"/>
						<xsl:with-param name="ref-name" select="$ref-name"/><!-- changed from $ref 4/26/2012 -->
						<xsl:with-param name="ref-attributes" select="$ref-attributes"/>
						<xsl:with-param name="parent" select="concat($parent,'\',@ref)"/>
					</xsl:apply-templates>
				</xsl:when>
				<xsl:otherwise>
					<xsl:comment>duplicate definition ignored: <xsl:value-of select="$ref"/>(<xsl:value-of select="position()"/>)</xsl:comment>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="outputEnclosedType">
		<xsl:param name="typeName"/>
		<xsl:param name="typeNS"/>
		<xsl:value-of select="w2e:message(concat('outputElementEnclosed: ',@name),4)"/>
		<xsl:variable name="type"
			select="if (string-length(substring-after($typeName[1],':')) = 0) then $typeName[1] else substring-after($typeName[1],':') "/>
		<xsl:variable name="namespace-for-name">
			<xsl:value-of select="$typeNS"/>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('$namespace-for-name=',$namespace-for-name),3)"/ -->
		<xsl:variable name="ns">
			<xsl:call-template name="namespace-uri-of-qname">
				<xsl:with-param name="qname" select="$typeName"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ns-to-use">
			<xsl:value-of select="if (string-length($ns) = 0) then $namespace-for-name else $ns"/>
		</xsl:variable>
		<xsl:variable name="prefix">
			<xsl:call-template name="prefix-for-namespace">
				<xsl:with-param name="namespace-uri" select="$namespace-for-name"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="fullname"
			select="if (string-length($prefix) = 0) then $typeName else concat($prefix,$typeName)"/>
		<xsl:value-of select="w2e:message(concat('enclosed fullname=',$fullname),3)"/>

		<xsl:element name="{$fullname}" namespace="{$namespace-for-name}">
			<xsl:for-each select="@*">
				<xsl:if test="not(contains(name(),':'))">
					<xsl:variable name="attrName" select="concat('xsd:',name())"/>
					<xsl:attribute name="{$attrName}" select="."/>
				</xsl:if>
			</xsl:for-each>
			<xsl:apply-templates
				select="//xsd:complexType[@name=$typeName]/xsd:complexContent/xsd:extension/xsd:attribute|xsd:attributeGroup"/>
			<!-- xsl:value-of select="w2e:message(concat('closing fullname=',$fullname),3)"/ -->
		</xsl:element>
	</xsl:template>

	<xsl:template name="inlineAttributeRules">
		<xsl:param name="use"/>
		<xsl:if test="count(./attribute::*) &gt; 0">
			<xsl:attribute name="xsd:attr-values" select="'true'"/>
		</xsl:if>
		<xsl:for-each select="xsd:attribute">
			<xsl:choose>
				<xsl:when test="@ref">
					<!-- xsl:value-of select="w2e:message(concat('attribute ref rule=',@ref),3)"/ -->
					<xsl:variable name="ref-name"
						select="if (string-length(substring-after(@ref,':')) = 0) then @ref else substring-after(@ref,':')"/>
					<xsl:variable name="ref-var" select="//xsd:attribute[@name=$ref-name]"/>
					<xsl:variable name="name" select="$ref-var[1]/@name"/>
					<xsl:variable name="chkUse" select="$ref-var[1]/@*/name()"/>
					<xsl:if test="not($chkUse[. = 'use'])">
						<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-use')"/>
						<xsl:attribute name="{$attrName}">optional</xsl:attribute>
					</xsl:if>
					<!-- xsl:message>use 3 - <xsl:value-of select="$chkUse"/></xsl:message -->
					<xsl:for-each select="$ref-var[1]/@*">
						<xsl:if test="not(name()='name') and(not(starts-with(name(),$ns-prefix)))">
							<xsl:variable name="attrName"
								select="concat('xsd:attr-',$ref-name,'-',name())"/>
							<xsl:attribute name="{$attrName}" select="."/>
						</xsl:if>
					</xsl:for-each>
				</xsl:when>
				<xsl:otherwise>
					<xsl:variable name="name" select="@name"/>
					<xsl:variable name="chkUse" select="@*/name()"/>
					<!-- xsl:message>use 1 - <xsl:value-of select="$name"/> - <xsl:value-of select="$chkUse"/> <xsl:if test="not($chkUse[. = 'use'])"> YES</xsl:if></xsl:message -->
					<xsl:if test="not($chkUse[. = 'use'])">
						<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-use')"/>
						<xsl:attribute name="{$attrName}">optional</xsl:attribute>
					</xsl:if>
					<xsl:for-each select="@*">
						<xsl:if test="not(name()='name') and(not(starts-with(name(),$ns-prefix)))">
							<xsl:variable name="attrName"
								select="concat('xsd:attr-',$name,'-',name())"/>
							<xsl:attribute name="{$attrName}" select="."/>
						</xsl:if>
					</xsl:for-each>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
		<xsl:for-each select="xsd:attributeGroup">
			<xsl:variable name="grpName" select="@name"/>
			<xsl:for-each select="//xsd:attributeGroup[@name=$grpName]">
				<xsl:variable name="name" select="@name"/>
				<xsl:variable name="chkUse" select="@*/name()"/>
				<xsl:if test="not($chkUse[. = 'use'])">
					<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-use')"/>
					<xsl:attribute name="{$attrName}">optional</xsl:attribute>
				</xsl:if>
				<!-- xsl:message>use 2 - <xsl:value-of select="$chkUse"/></xsl:message -->
				<xsl:for-each select="@*">
					<xsl:if test="not(name()='name') and(not(starts-with(name(),$ns-prefix)))">
						<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-',name())"/>
						<xsl:attribute name="{$attrName}" select="."/>
					</xsl:if>
				</xsl:for-each>
			</xsl:for-each>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="inlineAttributeRulesType">
		<xsl:param name="attrs"/>
		<xsl:param name="attrGrp"/>
		<xsl:if test="count($attrs) &gt; 0">
			<xsl:attribute name="xsd:attr-values" select="'true'"/>
		</xsl:if>
		<xsl:for-each select="$attrs">
			<xsl:value-of select="w2e:message(concat('attribute =',@ref|@name),4)"/>
			<xsl:choose>
				<xsl:when test="@ref">
					<!-- xsl:value-of select="w2e:message(concat('attribute ref rule=',@ref),3)"/ -->
					<xsl:variable name="ref-name"
						select="if (string-length(substring-after(@ref,':')) = 0) then @ref else substring-after(@ref,':')"/>
					<xsl:variable name="ref-var" select="//xsd:attribute[@name=$ref-name]"/>
					<xsl:variable name="name" select="$ref-var[1]/@name"/>
					<xsl:variable name="chkUse" select="$ref-var[1]/@*/name()"/>
					<!-- xsl:message>use 12 - <xsl:value-of select="$name"/> - <xsl:value-of select="$chkUse"/> <xsl:if test="not($chkUse[. = 'use'])"> YES</xsl:if></xsl:message -->
					<xsl:if test="not($chkUse[. = 'use'])">
						<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-use')"/>
						<xsl:attribute name="{$attrName}">optional</xsl:attribute>
					</xsl:if>
					<xsl:for-each select="$ref-var[1]/@*">
						<xsl:if test="not(name()='name') and(not(starts-with(name(),$ns-prefix)))">
							<xsl:variable name="attrName"
								select="concat('xsd:attr-',$ref-name,'-',name())"/>
							<xsl:attribute name="{$attrName}" select="."/>
						</xsl:if>
					</xsl:for-each>
				</xsl:when>
				<xsl:otherwise>
					<xsl:variable name="name" select="@name"/>
					<xsl:variable name="chkUse" select="@*/name()"/>
					<!-- xsl:message>use 11 - <xsl:value-of select="$name"/> - <xsl:value-of select="$chkUse"/> <xsl:if test="not($chkUse[. = 'use'])"> YES</xsl:if></xsl:message -->
					<xsl:if test="not($chkUse[. = 'use'])">
						<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-use')"/>
						<xsl:attribute name="{$attrName}">optional</xsl:attribute>
					</xsl:if>
					<xsl:for-each select="@*">
						<xsl:if test="not(name()='name') and(not(starts-with(name(),$ns-prefix)))">
							<xsl:variable name="attrName"
								select="concat('xsd:attr-',$name,'-',name())"/>
							<xsl:attribute name="{$attrName}" select="."/>
						</xsl:if>
					</xsl:for-each>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
		<xsl:for-each select="$attrGrp">
			<xsl:variable name="grpName" select="@name"/>
			<xsl:for-each select="//xsd:attributeGroup[@name=$grpName][1]">
				<xsl:value-of select="w2e:message(concat('attribGrp =',@ref|@name),4)"/>
				<xsl:variable name="name" select="@name|@ref"/>
				<xsl:variable name="chkUse" select="@*/name()"/>
				<!-- xsl:message>use 13 - <xsl:value-of select="$name"/> - <xsl:value-of select="$chkUse"/> <xsl:if test="not($chkUse[. = 'use'])"> YES</xsl:if></xsl:message -->
				<xsl:if test="not($chkUse[. = 'use'])">
					<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-use')"/>
					<xsl:attribute name="{$attrName}">optional</xsl:attribute>
				</xsl:if>
				<xsl:for-each select="@*">
					<xsl:if test="not(name()='name') and(not(starts-with(name(),$ns-prefix)))">
						<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-',name())"/>
						<xsl:attribute name="{$attrName}" select="."/>
					</xsl:if>
				</xsl:for-each>
			</xsl:for-each>
		</xsl:for-each>

	</xsl:template>

	<xsl:template match="text()"/>

	<xsl:template name="dedupRule">
		<xsl:param name="baseRules"/>
		<as:BusinessUseContext>
			<as:Rules>
				<as:default>
					<as:context>
						<xsl:for-each
							select="$baseRules/as:BusinessUseContext/as:Rules/as:default/as:context/as:constraint">
							<xsl:variable name="this"
								select="if(@action) then @action else '%NONE%'"/>
							<xsl:variable name="count"
								select="count($baseRules/as:BusinessUseContext/as:Rules/as:default/as:context/as:constraint[@action=$this][position() &lt; 3])"/>
							<xsl:choose>
								<xsl:when test="$count &gt; 1">
									<xsl:variable name="chk"
										select="./following::*[@action=$this][1]"/>
									<!-- optimization of duplicate removal -->
									<!-- xsl:message> <xsl:value-of select="$this"/> - <xsl:value-of select="position()"/> | <xsl:value-of select="count($chk[@action=$this])"/></xsl:message -->
									<xsl:choose>
										<xsl:when test="count($chk[@action=$this][1]) &gt; 0"
											><!-- xsl:comment>duplicate: <xsl:value-of select="$this"/></xsl:comment --></xsl:when>
										<xsl:otherwise>
											<!-- if lookup option and restrictValues - then refactor -->
											<xsl:choose>
												<xsl:when
												test="$useLookups='yes' and(contains(@action,'restrictValues'))">
												<xsl:call-template name="outputLookupRule">
												<xsl:with-param name="rule" select="."/>
												</xsl:call-template>
												</xsl:when>
												<xsl:otherwise>
												<xsl:copy-of select="."/>
												</xsl:otherwise>
											</xsl:choose>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:when>
								<xsl:otherwise>
									<xsl:choose>
										<xsl:when
											test="$useLookups='yes' and(contains(@action,'restrictValues'))">
											<xsl:call-template name="outputLookupRule">
												<xsl:with-param name="rule" select="."/>
											</xsl:call-template>
										</xsl:when>
										<xsl:otherwise>
											<xsl:copy-of select="."/>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:for-each>
					</as:context>
				</as:default>
			</as:Rules>
		</as:BusinessUseContext>
	</xsl:template>

	<xsl:template match="*" mode="anno">
		<xsl:param name="schemaContent"/>
		<xsl:param name="rules"/>
		<xsl:param name="annoAttrs"/>
		<xsl:variable name="findNS" select="name(.)"/>
		<xsl:variable name="find" select="local-name(.)"/>
		<xsl:variable name="choiceNodes">
			<xsl:choose>
				<xsl:when test="parent::*/local-name() = 'choice'">
					<xsl:variable name="findParent">
						<!-- When part of choice - then look back to find the details of the parent node above the choice element -->
						<xsl:choose>
							<xsl:when
								test="$schemaContent//xsd:element/xsd:complexType/xsd:sequence/xsd:choice/xsd:element[@name = $findNS]/@name">
								<xsl:variable name="elementName"
									select="$schemaContent//xsd:element/xsd:complexType/xsd:sequence/xsd:choice/xsd:element[@name = $findNS]/../../../../@name"/>
								<xsl:value-of select="$elementName"/>
							</xsl:when>
							<xsl:when
								test="$schemaContent//xsd:element/xsd:complexType/xsd:sequence/xsd:choice/xsd:element[@name = $find]/@name">
								<xsl:variable name="elementName"
									select="$schemaContent//xsd:element/xsd:complexType/xsd:sequence/xsd:choice/xsd:element[@name = $find]/../../../../@name"/>
								<xsl:value-of select="$elementName"/>
							</xsl:when>
							<xsl:when
								test="$schemaContent//xsd:complexType/xsd:sequence/xsd:choice/xsd:sequence/xsd:element[@name = $findNS]/@name">
								<xsl:variable name="typeName"
									select="$schemaContent//xsd:complexType/xsd:sequence/xsd:choice/xsd:sequence/xsd:element[@name = $findNS]/../../../../@name"/>
								<xsl:variable name="elementName"
									select="if($schemaContent//xsd:element[@type=$typeName]/@name) then $schemaContent//xsd:element[@type=$typeName]/@name  else $typeName"/>
								<xsl:value-of select="$elementName"/>
							</xsl:when>
							<xsl:when
								test="$schemaContent//xsd:complexType/xsd:choice/xsd:sequence/xsd:element[@name = $findNS]/@name">
								<xsl:variable name="typeName"
									select="$schemaContent//xsd:complexType/xsd:choice/xsd:sequence/xsd:element[@name = $findNS]/../../../@name"/>
								<xsl:variable name="elementName"
									select="if($schemaContent//xsd:element[@type=$typeName]/@name) then $schemaContent//xsd:element[@type=$typeName]/@name else $typeName"/>
								<xsl:value-of select="$elementName"/>
							</xsl:when>
							<xsl:when
								test="$schemaContent//xsd:complexType/xsd:choice/xsd:sequence/xsd:element[@name = $find]/@name">
								<xsl:variable name="typeName"
									select="$schemaContent//xsd:complexType/xsd:choice/xsd:sequence/xsd:element[@name = $find]/../../../@name"/>
								<xsl:variable name="elementName"
									select="if($schemaContent//xsd:element[@type=$typeName]/@name) then $schemaContent//xsd:element[@type=$typeName]/@name else $typeName"/>
								<xsl:value-of select="$elementName"/>
							</xsl:when>
							<xsl:when
								test="$schemaContent//xsd:complexType/xsd:choice/xsd:element[@name = $findNS]/@name">
								<xsl:variable name="typeName"
									select="$schemaContent//xsd:complexType/xsd:choice/xsd:element[@name = $findNS]/../../@name"/>
								<xsl:variable name="elementName"
									select="if($schemaContent//xsd:element[@type=$typeName]/@name) then $schemaContent//xsd:element[@type=$typeName]/@name else $typeName"/>
								<xsl:value-of select="$elementName"/>
							</xsl:when>
							<xsl:when
								test="$schemaContent//xsd:complexType/xsd:choice/xsd:element[@name = $find]/@name">
								<xsl:variable name="typeName"
									select="$schemaContent//xsd:complexType/xsd:choice/xsd:element[@name = $find]/../../@name"/>
								<xsl:variable name="elementName"
									select="if($schemaContent//xsd:element[@type=$typeName]/@name) then $schemaContent//xsd:element[@type=$typeName]/@name else $typeName"/>
								<xsl:value-of select="$elementName"/>
							</xsl:when>
							<xsl:when
								test="$schemaContent//xsd:complexType/xsd:choice/xsd:element[@ref = $find]/@ref">
								<xsl:variable name="typeName"
									select="$schemaContent//xsd:complexType/xsd:choice/xsd:element[@ref = $find]/../../@name"/>
								<xsl:variable name="elementName"
									select="if($schemaContent//xsd:element[@type=$typeName]/@name) then $schemaContent//xsd:element[@type=$typeName]/@name else $typeName"/>
								<xsl:value-of select="$elementName"/>
							</xsl:when>
							<xsl:when
								test="$schemaContent//xsd:complexType/xsd:choice/xsd:element[@ref = $findNS]/@ref">
								<xsl:variable name="typeName"
									select="$schemaContent//xsd:complexType/xsd:choice/xsd:element[@ref = $findNS]/../../@name"/>
								<xsl:variable name="elementName"
									select="if($schemaContent//xsd:element[@type=$typeName]/@name) then $schemaContent//xsd:element[@type=$typeName]/@name else $typeName"/>
								<xsl:value-of select="$elementName"/>
							</xsl:when>
							<xsl:when
								test="$schemaContent//xsd:complexType/xsd:sequence/xsd:choice/xsd:element[@ref = $find]/@ref">
								<xsl:variable name="typeName"
									select="$schemaContent//xsd:complexType/xsd:sequence/xsd:choice/xsd:element[@ref = $find]/../../../@name"/>
								<xsl:variable name="elementName"
									select="if($schemaContent//xsd:element[@type=$typeName]/@name) then  $schemaContent//xsd:element[@type=$typeName]/@name else $typeName"/>
								<xsl:value-of select="$elementName"/>
							</xsl:when>
							<xsl:when
								test="$schemaContent//xsd:complexType/xsd:sequence/xsd:choice/xsd:element[@name = $find]/@name">
								<xsl:variable name="typeName"
									select="$schemaContent//xsd:complexType/xsd:sequence/xsd:choice/xsd:element[@name = $find]/../../../@name"/>
								<xsl:variable name="elementName"
									select="if($schemaContent//xsd:element[@type=$typeName]/@name) then $schemaContent//xsd:element[@type=$typeName]/@name else $typeName"/>
								<xsl:value-of select="$elementName"/>
							</xsl:when>
						</xsl:choose>
					</xsl:variable>
					<xsl:value-of select="$findParent"/>
				</xsl:when>
				<xsl:otherwise/>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="parentNode">
			<xsl:choose>
				<xsl:when test="parent::*/local-name() = 'choice'">
					<xsl:value-of select="tokenize($choiceNodes,' ')[1]"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="parent::*/name()"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="thisNode" select="name()"/>
		<xsl:variable name="altType" select="$schemaContent//xsd:element[@name=$find]/@type"/>
		<xsl:variable name="findType"
			select="if (string-length(@type) &gt; 0) then @type else substring-after($altType[1],':')"/>
		<xsl:variable name="baseXPath" select="concat('/',$parentNode,'/',name(.))"/>
		<xsl:variable name="xpathChk"
			select="if(starts-with($baseXPath,'//')) then $baseXPath else concat('/',$baseXPath)"/>
		<xsl:variable name="xpath" select="replace($xpathChk,'xsd:choice/','')"/>
		<!-- xsl:variable name="chkExclude" select="concat('|',substring-before(substring-after($xpath,'//'),':'),':')"></xsl:variable -->
		<xsl:variable name="excludeNS">
			<xsl:choose>
				<xsl:when test="contains($xpath,':')">
					<xsl:for-each select="$annoExclTokens">
						<xsl:if test="string-length(.) &gt; 0 and(contains($xpath,concat('/',.)))"
							>true</xsl:if>
					</xsl:for-each>
				</xsl:when>
				<xsl:otherwise>false</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="fullXpath">
			<xsl:choose>
				<xsl:when test="contains(replace($xpath,'//',''),'/')">
					<xsl:value-of select="$xpath"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:variable name="searchXpath" select="replace($xpath,'//','/')"/>
					<xsl:choose>
						<xsl:when test="$rules//as:constraint[contains(@action,$searchXpath)]">
							<xsl:variable name="pathRules"
								select="$rules//as:constraint[contains(@action,$searchXpath)]/@action"/>
							<xsl:variable name="matchPathRule" select="$pathRules[1]"/>
							<xsl:value-of
								select="concat('//',substring-before(substring-after($matchPathRule,'//'),'/'),$searchXpath)"
							/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$xpath"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('anno =',$find,' | ',$findType),4)"/ -->
		<!-- xsl:message>[[<xsl:copy-of select="."></xsl:copy-of>]]</xsl:message -->
		<xsl:variable name="chkSchemaElement"
			select="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]"/>
		<xsl:variable name="chkElementMatch">
			<xsl:for-each select="$chkSchemaElement/@*">
				<xsl:choose>
					<xsl:when test="name()='substitutionGroup'">
						<xsl:element name="{name()}">
							<xsl:value-of select="."/>
						</xsl:element>
					</xsl:when>
					<xsl:when test="name()='base'">
						<xsl:element name="{name()}">
							<xsl:value-of select="."/>
						</xsl:element>
					</xsl:when>
					<xsl:when
						test="name()='type' and(not(starts-with(.,'xsd:') or(starts-with(.,'xs:')))) and(not(.=$findNS))">
						<xsl:element name="{name()}">
							<xsl:value-of select="."/>
						</xsl:element>
					</xsl:when>
					<xsl:when test="name()='ref'">
						<xsl:element name="{name()}">
							<xsl:value-of select="."/>
						</xsl:element>
					</xsl:when>
					<xsl:otherwise/>
				</xsl:choose>
			</xsl:for-each>
		</xsl:variable>
		<!-- xsl:message> chk - <xsl:value-of select="$findNS"/> [<xsl:copy-of select="$chkElementMatch"/>] </xsl:message -->

		<xsl:variable name="refAnno">
			<xsl:if test="not($parentNode = 'AllRootLevelTypeDefinitions') and(not($parentNode = $elementNameValue))">
				<xsl:variable name="chkRefMatch" select="$schemaContent//xsd:complexType[@name=$parentNode]/xsd:sequence/xsd:element"/>
				<xsl:variable name="chkBaseMatch" select="$schemaContent//xsd:complexType[@name=$parentNode]/xsd:complexContent/xsd:extension/@base"/>

				<xsl:variable name="groupName">
					<xsl:variable name="tempGroup">
						<xsl:value-of select="if(ends-with($thisNode,'Type')) then $thisNode else concat($thisNode,'Type')"/>
					</xsl:variable>
					<xsl:value-of select="if(contains($tempGroup,':')) then substring-after($tempGroup,':') else $tempGroup"/>
				</xsl:variable>

				<xsl:variable name="chkRefGroup" select="$schemaContent//xsd:complexType[@name=$groupName]/xsd:sequence/xsd:group"/>
				<xsl:variable name="childItems">|<xsl:for-each select="$chkRefMatch/@name"><xsl:value-of select="concat(.,'|')"/></xsl:for-each></xsl:variable>
				<xsl:variable name="groupItems">|<xsl:for-each select="$chkRefGroup/@ref | @name"><xsl:value-of select="concat(.,'|')"/></xsl:for-each></xsl:variable>
				<!-- xsl:message> - group chk <xsl:value-of select="$groupName"/> - <xsl:value-of select="$findNS"/> - <xsl:value-of select="$groupItems"/></xsl:message -->

				<!-- Inverse logic check - if no match exists, then it must be a ref -->
				<xsl:if test="not($childItems='|') and(not(contains($childItems,concat('|',$findNS,'|'))))">
					<xsl:value-of select="w2e:message(concat('ref check = ',$parentNode,'/',$findNS,' - ',$childItems),4)"/>
					<camed:documentation type="Ref">
						<xsl:value-of select="concat($parentNode,'/',$findNS)"/>
					</camed:documentation>
				</xsl:if>
				<!-- If no names exists, then see if its a base name ref item -->
				<xsl:if test="$childItems='|' or(not($chkBaseMatch=''))">
					<!-- This is reverse logic - pick only the ref= names for this check -->
					<xsl:variable name="baseRefItems"
						select="$schemaContent//xsd:complexType[@name=$chkBaseMatch]/xsd:sequence/xsd:element"/>
					<xsl:variable name="baseItems">|<xsl:for-each select="$baseRefItems/@ref"><xsl:value-of select="concat(.,'|')"/></xsl:for-each></xsl:variable>
					<xsl:if test="not($baseItems='|') and(contains($baseItems,concat('|',$findNS,'|')))">
						<xsl:value-of select="w2e:message(concat('ref base = [',$chkBaseMatch,'] ',$parentNode,'/',$findNS,' - ',$baseItems),4)"/>
						<camed:documentation type="BaseRef">
							<xsl:value-of select="concat($chkBaseMatch,' ',$parentNode,'/',$findNS)"/>
						</camed:documentation>
					</xsl:if>
				</xsl:if>
				<xsl:if test="not($groupItems='|') and(not(contains($groupItems,concat('|',$findNS,'|'))))">
					<xsl:value-of select="w2e:message(concat('group check = ',$parentNode,'/',$findNS,' - ',$groupItems),4)"/>
					<camed:documentation type="Group">
						<!-- xsl:value-of select="if(ends-with($thisNode,'Type')) then $thisNode else concat($thisNode,'Type')"/ -->
						<xsl:for-each select="tokenize($groupItems,'\|')">
							<xsl:variable name="this" select="."/>
							<xsl:if test="$this &gt; ' '">
								<xsl:value-of select="concat($this,$newline)"/>
								<xsl:variable name="thisGroup" select="if(contains($this,':')) then substring-after($this,':') else $this"/>
								<xsl:variable name="groupDetail" select="$schemaContent//xsd:group[@name = $thisGroup]"/>
								<xsl:for-each select="$groupDetail[1]/xsd:annotation/xsd:documentation">[<xsl:value-of select="concat(.,']',$newline)"/></xsl:for-each>
								<xsl:for-each select="$groupDetail[1]/xsd:sequence">
									<xsl:apply-templates mode="groupDetails" select=".">
										<xsl:with-param name="offset">- </xsl:with-param>
										<xsl:with-param name="schemaDetails" select="$schemaContent"/>
									</xsl:apply-templates>
								</xsl:for-each>
							</xsl:if>
						</xsl:for-each>
					</camed:documentation>
				</xsl:if>
			</xsl:if>
		</xsl:variable>

		<xsl:choose>
			<xsl:when test="contains($excludeNS,'true')"/>
			<!-- Could add a 'recursive' annotation here, but for now done elsewhere -->
			<!-- Filter annotations for dictionary build modes of structure items -->
			<xsl:when
				test="string-length($chkElementMatch[1]) &gt; 0 or(count($schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:documentation) &gt; 0)">
				<xsl:variable name="docText" select="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:documentation"/>
				<xsl:variable name="attributeGroup" select="$annoAttrs//camed:annotation[@item=$fullXpath][1]/camed:documentation[1]"/>
				<xsl:variable name="annolines">
					<xsl:for-each select="$chkElementMatch/*">
						<!-- one of @substitutionGroup | @base | @type | @ref -->
						<xsl:variable name="thisName" select="name(.)"/>
						<!-- for duplicate checking -->
						<xsl:choose>
							<xsl:when test="name(.)='type' and(.=$findNS)"/>
							<!-- ignore when type name = same as element name -->
							<xsl:otherwise>
								<xsl:variable name="FullMatchPath">
									<xsl:call-template name="resolveTypePath">
										<xsl:with-param name="MatchName" select="."/>
										<xsl:with-param name="MatchType" select="name()"/>
										<xsl:with-param name="schemaContent" select="$schemaContent"/>
									</xsl:call-template>
								</xsl:variable>
								<xsl:variable name="moreItems"
									select="count(following::*[name(.)=$thisName])"/>
								<xsl:choose>
									<xsl:when test="contains($baseTypes,concat($FullMatchPath,'|'))"/>
									<!-- ignore base types -->
									<xsl:when test="$moreItems &gt; 0"/>
									<!-- only output the last item of duplicates -->
									<xsl:when test="name(.)='substitutionGroup'">
										<camed:documentation type="{name(.)}">	<xsl:value-of select="$FullMatchPath"/></camed:documentation>
									</xsl:when>
									<xsl:otherwise>
										<camed:documentation
											type="{concat(upper-case(substring(name(.),1,1)),substring(name(.),2))}">
											<xsl:value-of select="$FullMatchPath"/>
										</camed:documentation>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:for-each>
					<xsl:if test="count($docText[1]/.) &gt; 0">
						<camed:documentation type="Definition">
							<xsl:call-template name="outputAnno">
								<xsl:with-param name="annoDoc" select="$docText[1]/."/>
								<xsl:with-param name="context" select="$findNS"/>
							</xsl:call-template>
						</camed:documentation>
					</xsl:if>
					<xsl:copy-of select="$refAnno"/>
					<!-- Add ref annotation -->

					<!-- Look to see if there is an extension being used and no substitutionGroup -->
					<xsl:if
						test="not($chkElementMatch/substitutionGroup) and count($schemaContent//xsd:complexType[@name=$find]/xs:complexContent/xsd:extension/@base) &gt; 0">
						<camed:documentation type="Extension">
							<xsl:value-of select="$schemaContent//xsd:complexType[@name=$find]/xs:complexContent/xsd:extension/@base[1]"/>
						</camed:documentation>
					</xsl:if>
					
					<!-- Check for Key / KeyRef details -->
					<xsl:if test="count($schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:key | xsd:keyref) &gt; 0">
						<xsl:if test="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:key">
							<camed:documentation type="Key">
								<xsl:for-each select="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:key">
									<xsl:if test="position() &gt; 1"><xsl:value-of select="$newline"/></xsl:if>
									<xsl:text>Target: </xsl:text><xsl:value-of select="xs:selector/@xpath"/><xsl:for-each select="xs:field/@xpath"> | <xsl:value-of select="."/></xsl:for-each> - Name: <xsl:value-of select="./@name"/>
								</xsl:for-each>
							</camed:documentation>
						</xsl:if>
						<xsl:if test="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:keyref">
							<camed:documentation type="KeyRef">
								<xsl:for-each select="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:keyref">
									<xsl:if test="position() &gt; 1"><xsl:value-of select="$newline"/></xsl:if>
									<xsl:text>Target: </xsl:text><xsl:value-of select="xs:selector/@xpath"/><xsl:for-each select="xs:field/@xpath"> | <xsl:value-of select="."/></xsl:for-each> - Refer: <xsl:value-of select="./@refer"/>
								</xsl:for-each>
							</camed:documentation>
						</xsl:if>
					</xsl:if>
					
					<!-- Output any appinfo annotations -->
					<xsl:if test="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:appinfo">
						<xsl:variable name="appInfoItems" select="count($schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:appinfo)"/>
							<xsl:for-each select="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:appinfo">
								<xsl:choose>
									<xsl:when test="contains(@source,':')">
										<xsl:variable name="sourceName" select="tokenize(@source,':')"/>
										<xsl:variable name="typeName" select="$sourceName[last()]"/>
										<camed:documentation type="{if(string-length($typeName) &gt; 0) then $typeName else @source}">
											<xsl:value-of select="."/>
										</camed:documentation>
									</xsl:when>
									<xsl:otherwise>
										<camed:documentation type="{@source}"><xsl:value-of select="."/></camed:documentation>
									</xsl:otherwise>
								</xsl:choose>
								<xsl:if test="position() = $appInfoItems">
									<xsl:choose>
										<xsl:when test="contains(@source,':')">
											<xsl:variable name="sourceName" select="tokenize(@source,':')"/>
											<xsl:variable name="typeName" select="$sourceName[last()]"/>
											<camed:documentation type="URN"><xsl:value-of select="substring-before(@source,concat(':',$typeName))"/></camed:documentation>
										</xsl:when>
										<xsl:otherwise><camed:documentation type="URN">Appinfo</camed:documentation></xsl:otherwise>
									</xsl:choose>
								</xsl:if>
							</xsl:for-each>
					</xsl:if>
					
					<xsl:copy-of select="$attributeGroup"/>
					<!-- Add in any attributeGroup details from annotations -->
				</xsl:variable>
				<!-- Output the collected annotation lines -->
				<xsl:if test="$annolines/camed:documentation">
					<camed:annotation item="{$fullXpath}">
						<xsl:copy-of select="$annolines"/>
					</camed:annotation>
					<!-- Now see if there is multiple references for a choice -->
					<xsl:choose>
						<xsl:when test="parent::*/local-name() = 'choice' and(count(tokenize($choiceNodes,' ')) &gt; 1)">
							<!-- Now output annotations for more path matches -->
							<xsl:variable name="firstChoice" select="tokenize($choiceNodes,' ')[1]"/>
							<!-- xsl:message><xsl:value-of select="$choiceNodes"/> - <xsl:value-of select="$fullXpath"/></xsl:message -->
							<xsl:for-each select="tokenize($choiceNodes,' ')">
								<xsl:if test="position() &gt; 1">
									<xsl:variable name="thisParent" select="."/>
									<camed:annotation
										item="{replace($fullXpath,$firstChoice,$thisParent)}">
										<xsl:copy-of select="$annolines"/>
									</camed:annotation>
								</xsl:if>
							</xsl:for-each>
						</xsl:when>
						<xsl:otherwise/>
					</xsl:choose>
				</xsl:if>
			</xsl:when>
			<xsl:when test="count($schemaContent//xsd:simpleType[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:documentation) &gt; 0">
				<xsl:variable name="docText"
					select="$schemaContent//xsd:simpleType[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:documentation"/>
				<camed:annotation item="{$xpath}">
					<camed:documentation type="Definition">
						<xsl:call-template name="outputAnno">
							<xsl:with-param name="annoDoc" select="$docText[1]/."/>
							<xsl:with-param name="context" select="$findNS"/>
						</xsl:call-template>
					</camed:documentation>
					<xsl:copy-of select="$refAnno"/>
					<!-- Add ref annotation -->
				</camed:annotation>
			</xsl:when>
			<xsl:when test="count($schemaContent//xsd:simpleType[@name=$findType]/xsd:annotation/xsd:documentation) &gt; 0">
				<xsl:variable name="docText" select="$schemaContent//xsd:simpleType[@name=$findType]/xsd:annotation/xsd:documentation"/>
				<camed:annotation item="{$xpath}">
					<camed:documentation type="Definition">
						<xsl:call-template name="outputAnno">
							<xsl:with-param name="annoDoc" select="$docText[1]/."/>
							<xsl:with-param name="context" select="$findNS"/>
						</xsl:call-template>
					</camed:documentation>
					<xsl:copy-of select="$refAnno"/>
					<!-- Add ref annotation -->
				</camed:annotation>
			</xsl:when>
			<xsl:when
				test="count($schemaContent//xsd:complexType[@name=$find]/xsd:annotation/xsd:documentation) &gt; 0">
				<xsl:variable name="docText"
					select="$schemaContent//xsd:complexType[@name=$find]/xsd:annotation/xsd:documentation"/>
				<camed:annotation item="{$xpath}">
					<xsl:if
						test="count($schemaContent//xsd:complexType[@name=$find]/xs:complexContent/xsd:extension/@base) &gt; 0">
						<camed:documentation type="Extension">
							<xsl:value-of select="$schemaContent//xsd:complexType[@name=$find]/xs:complexContent/xsd:extension/@base[1]"/>
						</camed:documentation>
					</xsl:if>
					<camed:documentation type="Definition">
						<xsl:call-template name="outputAnno">
							<xsl:with-param name="annoDoc" select="$docText[1]/."/>
							<xsl:with-param name="context" select="$findNS"/>
						</xsl:call-template>
					</camed:documentation>
					<xsl:copy-of select="$refAnno"/>
					<!-- Add ref annotation -->
				</camed:annotation>
			</xsl:when>
			<xsl:when
				test="@type and(count($schemaContent//xsd:complexType[@name=$findType]/xsd:annotation/xsd:documentation) &gt; 0)">
				<xsl:variable name="docText"
					select="$schemaContent//xsd:complexType[@name=$findType]/xsd:annotation/xsd:documentation"/>
				<camed:annotation item="{$xpath}">
					<xsl:if
						test="count($schemaContent//xsd:complexType[@name=$findType]/xs:complexContent/xsd:extension/@base) &gt; 0">
						<camed:documentation type="Extension">
							<xsl:value-of select="$schemaContent//xsd:complexType[@name=$findType]/xs:complexContent/xsd:extension/@base[1]"/>
						</camed:documentation>
					</xsl:if>
					<camed:documentation type="Definition">
						<xsl:call-template name="outputAnno">
							<xsl:with-param name="annoDoc" select="$docText[1]/."/>
							<xsl:with-param name="context" select="$findNS"/>
						</xsl:call-template>
					</camed:documentation>
					<xsl:copy-of select="$refAnno"/>
					<!-- Add ref annotation -->
				</camed:annotation>
			</xsl:when>
			<xsl:when
				test="count($schemaContent//xsd:complexType[@name=$find]/xsd:complexContent/xsd:extension/@base) &gt; 0">
				<camed:annotation item="{$xpath}">
					<camed:documentation type="Extension">
						<xsl:value-of select="$schemaContent//xsd:complexType[@name=$find]/xs:complexContent/xsd:extension/@base[1]"/>
					</camed:documentation>
				</camed:annotation>
			</xsl:when>
			<xsl:when
				test="count($schemaContent//xsd:complexType[@name=$findType]/xsd:complexContent/xsd:extension/@base) &gt; 0">
				<camed:annotation item="{$xpath}">
					<camed:documentation type="Extension">
						<xsl:value-of select="$schemaContent//xsd:complexType[@name=$findType]/xs:complexContent/xsd:extension/@base[1]"/>
					</camed:documentation>
					<xsl:copy-of select="$refAnno"/>
					<!-- Add ref annotation -->
				</camed:annotation>
			</xsl:when>
			<xsl:otherwise/>
			<!-- xsl:message>skipping [<xsl:value-of select="concat($find,'-',$findNS,'-',$findType)"></xsl:value-of>]</xsl:message -->
		</xsl:choose>

		<xsl:if test="count($schemaContent//xsd:group[@name=$find]) &gt; 0 or (count($schemaContent//xsd:group[@name=concat($find,'Group')]) &gt; 0)">
			<camed:annotation item="{$fullXpath}">
				<camed:documentation type="Group">Item is a XSD group construct: <xsl:value-of select="if(count($schemaContent//xsd:group[@name=concat($find,'Group')]) &gt; 0) then concat($find,'Group') else $find"/></camed:documentation>
			</camed:annotation>
		</xsl:if>
		
		<!-- Look for appinfo annotations -->
		<xsl:if
			test="count($schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:appinfo) &gt; 0 and(count($schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:documentation) = 0)">
			<camed:annotation item="{$fullXpath}">
				<xsl:if test="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:appinfo">
					<camed:documentation type="Appinfo">
						<xsl:for-each select="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:appinfo">
	[source] - <xsl:value-of select="@source"/>
	[text] - <xsl:value-of select="."/>
						</xsl:for-each>
					</camed:documentation>
				</xsl:if>
			</camed:annotation>
		</xsl:if>

		<!-- Now document Key / KeyRef information if present for this node element definition but no default documentation details -->
		<xsl:if
			test="count($schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:key | xsd:keyref) &gt; 0 and(count($schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:documentation) = 0)">
			<camed:annotation item="{$fullXpath}">
				<xsl:variable name="docText"
					select="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:documentation"/>
				<xsl:if test="count($docText[1]/.) &gt; 0">
					<camed:documentation type="Definition">
						<xsl:call-template name="outputAnno">
							<xsl:with-param name="annoDoc" select="$docText[1]/."/>
							<xsl:with-param name="context" select="$findNS"/>
						</xsl:call-template>
					</camed:documentation>
				</xsl:if>
				<xsl:if test="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:key">
					<camed:documentation type="Key">
						<xsl:for-each select="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:key">
							<xsl:if test="position() &gt; 1"><xsl:value-of select="$newline"/></xsl:if>
							<xsl:text>Target: </xsl:text><xsl:value-of select="xs:selector/@xpath"/><xsl:for-each select="xs:field/@xpath"> | <xsl:value-of select="."/></xsl:for-each> - Name: <xsl:value-of select="./@name"/>
						</xsl:for-each>
					</camed:documentation>
				</xsl:if>
				<xsl:if test="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:keyref">
					<camed:documentation type="KeyRef">
						<xsl:for-each select="$schemaContent//xsd:element[@name=$find or(@name=$findNS)]/xsd:keyref">
							<xsl:if test="position() &gt; 1"><xsl:value-of select="$newline"/></xsl:if>
							<xsl:text>Target: </xsl:text><xsl:value-of select="xs:selector/@xpath"/><xsl:for-each select="xs:field/@xpath"> | <xsl:value-of select="."/></xsl:for-each> - Refer: <xsl:value-of select="./@refer"/>
						</xsl:for-each>
					</camed:documentation>
				</xsl:if>
				<xsl:copy-of select="$refAnno"/>
				<!-- Add ref annotation -->
			</camed:annotation>
		</xsl:if>

		<xsl:if
			test="position() = 1 and ($parentNode = $elementNameValue or($parentNode='AllRootLevelTypeDefinitions'))">
			<camed:annotation item="{concat('//',$parentNode)}">
				<xsl:variable name="docText"
					select="$entireXSDcollection//xsd:element[@name=$elementNameValue]/xsd:annotation/xsd:documentation"/>
				<xsl:if test="count($docText[1]/.) &gt; 0">
					<camed:documentation type="Definition">
						<xsl:call-template name="outputAnno">
							<xsl:with-param name="annoDoc" select="$docText[1]/."/>
							<xsl:with-param name="context" select="$findNS"/>
						</xsl:call-template>
					</camed:documentation>
				</xsl:if>
				<camed:documentation type="Generated">Built by XSD 2 CAM - <xsl:value-of select="$version"/> - <xsl:value-of select="current-dateTime()"/></camed:documentation>
				<xsl:if
					test="count($entireXSDcollection//xsd:import) &gt;0 or(count($entireXSDcollection//xsd:include))">
					<!-- Output root element with annotations for imports and includes  -->
					<camed:documentation type="Imports">Definitions were imported from:<xsl:value-of select="$newline"/>
						<xsl:for-each
							select="$entireXSDcollection//xsd:import | $entireXSDcollection//xsd:include">
							<xsl:variable name="nsPrefixURI" select="xsd:schema/@targetNamespace"/>
							<xsl:value-of select="concat(name(),' + ',@schemaLocation,' nsURI: ',$nsPrefixURI,' ',$newline)"/>
						</xsl:for-each>
					</camed:documentation>
				</xsl:if>
				<xsl:copy-of select="$refAnno"/>
				<!-- Add ref annotation -->
			</camed:annotation>
		</xsl:if>

		<xsl:apply-templates select="*" mode="anno">
			<xsl:with-param name="schemaContent" select="$schemaContent"/>
			<xsl:with-param name="rules" select="$rules"/>
			<xsl:with-param name="annoAttrs" select="$annoAttrs"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="@*" mode="anno">
			<xsl:with-param name="schemaContent" select="$schemaContent"/>
			<xsl:with-param name="rules" select="$rules"/>
			<xsl:with-param name="annoAttrs" select="$annoAttrs"/>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="@*" mode="anno">
		<xsl:param name="schemaContent"/>
		<xsl:param name="rules"/>
		<xsl:param name="annoAttrs"/>
		<xsl:if test="not(substring-before(name(.),':')='xsd')">
			<xsl:variable name="findNS" select="name(.)"/>
			<xsl:variable name="find" select="local-name(.)"/>
			<xsl:variable name="baseXPath" select="concat('/',name(../.),'/@',name(.))"/>
			<xsl:variable name="xpathChk"
				select="if(starts-with($baseXPath,'//')) then $baseXPath else concat('/',$baseXPath)"/>
			<xsl:variable name="xpath" select="replace($xpathChk,'xsd:choice/','')"/>
			<!-- xsl:variable name="chkExclude" select="concat('|',substring-before(substring-after($xpath,'//'),':'),':')"></xsl:variable -->
			<xsl:variable name="excludeNS">
				<xsl:choose>
					<xsl:when test="contains($xpath,':')">
						<xsl:for-each select="$annoExclTokens">
							<xsl:choose>
								<xsl:when test="string-length(.)=0">false</xsl:when>
								<xsl:when test="contains($xpath,concat('/@',.))">true</xsl:when>
								<xsl:when test="contains($xpath,concat('/',.))">true</xsl:when>
							</xsl:choose>
						</xsl:for-each>
					</xsl:when>
					<xsl:otherwise>false</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			<xsl:choose>
				<xsl:when test="contains($excludeNS,'true')"/>
				<xsl:when
					test="count($schemaContent//xsd:attribute[@name=$find or(@name=$findNS)]/xsd:simpleType/xsd:annotation/xsd:documentation) &gt; 0">
					<xsl:variable name="docText"
						select="$schemaContent//xsd:attribute[@name=$find or(@name=$findNS)]/xsd:simpleType/xsd:annotation/xsd:documentation"/>
					<camed:annotation item="{$xpath}">
						<camed:documentation type="Definition">
							<xsl:call-template name="outputAnno">
								<xsl:with-param name="annoDoc" select="$docText[1]/."/>
								<xsl:with-param name="context" select="$findNS"/>
							</xsl:call-template>
						</camed:documentation>
					</camed:annotation>
				</xsl:when>
				<xsl:when
					test="count($schemaContent//xsd:attribute[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:documentation) &gt; 0">
					<xsl:variable name="docText"
						select="$schemaContent//xsd:attribute[@name=$find or(@name=$findNS)]/xsd:annotation/xsd:documentation"/>
					<camed:annotation item="{$xpath}">
						<camed:documentation type="Definition">
							<xsl:call-template name="outputAnno">
								<xsl:with-param name="annoDoc" select="$docText[1]/."/>
								<xsl:with-param name="context" select="$findNS"/>
							</xsl:call-template>
						</camed:documentation>
					</camed:annotation>
				</xsl:when>
			</xsl:choose>
		</xsl:if>
	</xsl:template>

	<xsl:template name="outputAnno">
		<xsl:param name="annoDoc"/>
		<xsl:param name="context"/>
		<xsl:choose>
			<xsl:when test="contains($annoContent,'all') and(count($annoDoc/node()) &gt; 1)">
				<xsl:copy-of select="$annoDoc/node()"/>
			</xsl:when>
			<xsl:when test="count($annoDoc/node()) &gt; 1">
				<xsl:apply-templates mode="annoTags" select="$annoDoc/node()"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="$annoDoc/text()">
					<xsl:call-template name="chkUTF8">
						<xsl:with-param name="thisText" select="$annoDoc/."/>
						<xsl:with-param name="context" select="concat($context,' (annotation)')"/>
					</xsl:call-template>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="*" mode="annoTags"><xsl:value-of select="name(.)"/><xsl:variable name="content"><xsl:value-of select="./self::node()/text()"/></xsl:variable>[<xsl:if test="string-length(normalize-space($content)) &gt; 1">"<xsl:value-of select="$content"/>"</xsl:if>] <xsl:apply-templates select="child::*" mode="annoTags"/></xsl:template>

	<xsl:template name="chkUTF8">
		<xsl:param name="thisText"/>
		<xsl:param name="context"/>
		<xsl:variable name="zeroPass">
			<xsl:choose>
				<xsl:when test="contains($thisText,$utf8-r0) or(contains($thisText,$utf8-r1)) or(contains($thisText,$utf8-r2))  or(contains($thisText,$utf8-r3))  or(contains($thisText,$utf8-r4))  or(contains($thisText,$utf8-r5))  or(contains($thisText,$utf8-r6))  or(contains($thisText,$utf8-r7))  or(contains($thisText,$utf8-r8)) or(contains($thisText,$utf8-r9)) or(contains($thisText,$utf8-r10)) or(contains($thisText,$utf8-r11)) or(contains($thisText,$utf8-r12))">
					<!-- xsl:value-of select="w2e:message(concat(' - ',$context,'; non-UTF8 char replaced: [',$firstPass,']'),1)"/ -->
					<xsl:message> - <xsl:value-of select="$context"/> non-UTF8 char replaced [0]: [<xsl:value-of select="$thisText"/>]</xsl:message>
					<xsl:variable name="temp1a"><xsl:value-of select="replace(replace(replace(replace(replace(replace(replace(replace(replace(replace(replace(replace($thisText,$utf8-r1,$quote),$utf8-r2,$dash),$utf8-r3,$quote),$utf8-r4,$quotes),$utf8-r5,$quotes),$utf8-r6,$period),$utf8-r7,$dash),$utf8-r8,$period),$utf8-r9,$twiddle),$utf8-r10,$space),$utf8-r11,$dot),$utf8-r12,$quote)"/></xsl:variable>
					<xsl:value-of select="$temp1a"/></xsl:when>
				<xsl:otherwise><xsl:value-of select="$thisText"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="firstPass">
			<xsl:choose>
				<xsl:when test="contains($zeroPass,$utf8-1) or(contains($zeroPass,$utf8-2)) or(contains($zeroPass,$utf8-3)) or(contains($zeroPass,$utf8-4)) or(contains($zeroPass,$utf8-5)) or(contains($zeroPass,$utf8-6)) or(contains($zeroPass,$utf8-7)) or(contains($zeroPass,$utf8-8)) or(contains($zeroPass,$utf8-9)) or(contains($zeroPass,$utf8-10)) or(contains($zeroPass,$utf8-32)) or(contains($zeroPass,$utf8-33)) or(contains($zeroPass,$utf8-34)) or(contains($zeroPass,$utf8-35)) or(contains($zeroPass,$utf8-36)) or(contains($zeroPass,$utf8-37)) or(contains($zeroPass,$utf8-30))">
					<!-- xsl:value-of select="w2e:message(concat(' - ',$context,'; non-UTF8 char replaced: [',$zeroPass,']'),1)"/ -->
					<xsl:message> - <xsl:value-of select="$context"/> non-UTF8 char replaced [1]: [<xsl:value-of select="$zeroPass"/>]</xsl:message>
					<xsl:variable name="temp1"><xsl:value-of select="replace(replace(replace(replace(replace(replace(replace(replace(replace(replace(replace($zeroPass,$utf8-1,'!'),$utf8-34,$quotes),$utf8-36,'c'),$utf8-37,'c'),$utf8-35,'e'),$utf8-30,'e'),$utf8-2,$quotes),$utf8-3,$quotes),$utf8-4,$quotes),$utf8-5,$quotes),$utf8-6,$quotes)"/></xsl:variable>
					<xsl:value-of select="replace(replace(replace(replace(replace(replace($temp1,$utf8-7,'!'),$utf8-8,$quotes),$utf8-9,$quotes),$utf8-10,''),$utf8-32,'!'),$utf8-33,'!')"/>
				</xsl:when>
				<xsl:otherwise><xsl:value-of select="$zeroPass"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:choose>
			<xsl:when test="contains($firstPass,$utf8-11) or(contains($firstPass,$utf8-12)) or(contains($firstPass,$utf8-13))  or(contains($firstPass,$utf8-38))  or(contains($firstPass,$utf8-14)) or(contains($firstPass,$utf8-15)) or(contains($firstPass,$utf8-16)) or(contains($firstPass,$utf8-17)) or(contains($firstPass,$utf8-18)) or(contains($firstPass,$utf8-19)) or(contains($firstPass,$utf8-31))">
				<!-- xsl:value-of select="w2e:message(concat(' - ',$context,'; non-UTF8 char replaced: [',$firstPass,']'),1)"/ -->
				<xsl:message> - <xsl:value-of select="$context"/> non-UTF8 char replaced [2]: [<xsl:value-of select="$firstPass"/>]</xsl:message>
				<xsl:variable name="temp1a"><xsl:value-of select="replace(replace(replace(replace(replace(replace($firstPass,$utf8-11,$quotes),$utf8-35,'e'),$utf8-31,'A'),$utf8-12,$quotes),$utf8-38,'??'),$utf8-13,$quotes)"/></xsl:variable>
				<xsl:variable name="temp2a"><xsl:value-of select="replace(replace(replace(replace(replace(replace($temp1a,$utf8-14,$quotes),$utf8-15,$quotes),$utf8-16,$quotes),$utf8-17,''),$utf8-18,'o'),$utf8-19,'-')"/></xsl:variable>
				<xsl:value-of select="$temp2a"/></xsl:when>
			<xsl:otherwise><xsl:value-of select="$firstPass"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="chkExcludeAttr">
		<xsl:param name="names"/>
		<xsl:param name="parent"/>
		<xsl:variable name="temp" select="tokenize($parent,'\\')"/>
		<xsl:variable name="last" select="$temp[count($temp)]"/>
		<xsl:variable name="previous" select="$temp[count($temp) - 1]"/>
		<xsl:for-each select="$names">
			<xsl:variable name="chk" select="concat($previous,'\',$last,'\@',.)"/>
			<xsl:choose>
				<xsl:when test="$excludeAttrPaths/Attribute[@n=$chk]" exclude-result-prefixes="#all">
					<attribute e="{.}"/>
				</xsl:when>
				<xsl:otherwise exclude-result-prefixes="#all">
					<attribute n="{.}"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="chkExcludeElement">
		<xsl:param name="name"/>
		<xsl:param name="parent"/>
		<xsl:variable name="temp" select="tokenize($parent,'\\')"/>
		<xsl:variable name="last" select="$temp[count($temp)]"/>
		<xsl:variable name="previous"
			select="if($temp[count($temp) - 1]='') then $temp[count($temp) - 2] else $temp[count($temp) - 1]"/>
		<xsl:variable name="chk" select="concat($last,'\',$name[1])"/>
		<xsl:variable name="chkElem" select="concat($previous,'\',$last,'\',$name[1])"/>
		<!-- xsl:value-of select="w2e:message(concat('chkExclude Grp - ',$chk,' and Elem ',$chkElem),4)"/ -->
		<xsl:choose>
			<xsl:when test="$excludeGrpPaths/Group[@n=$chk]">true</xsl:when>
			<xsl:when test="$excludeElemPaths/Element[@n=$chkElem]">true</xsl:when>
			<xsl:otherwise>false</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- xsl:template match="xsd:simpleType">
		<xsl:param name="position"/>
		<xsl:variable name="typeName" select="@name"/>
		<xsl:variable name="mode"/>
		<xsl:value-of select="w2e:message(concat('simpleType (base) ',$position,': ',$typeName),4)"/>
	</xsl:template -->

	<xsl:template match="xsd:simpleType" mode="simple">
		<xsl:param name="position"/>
		<xsl:variable name="typeName" select="@name"/>
		<xsl:variable name="mode"/>
		<xsl:value-of select="w2e:message(concat('simpleType (simple) ',$position,': ',$typeName),4)"/>
		<!-- xsl:message>simpleType: [<xsl:value-of select="$typeName"/>]</xsl:message -->
		<xsl:choose>
			<!-- xsl:when test="./xsd:union | ./xsd:list">
				<xsl:message> - union / list <xsl:value-of select="./xsd:union/@memberTypes | ./xsd:list/@itemType"/></xsl:message>
			</xsl:when -->
			<xsl:when test="count(xsd:restriction/xsd:enumeration) &gt; 0">
				<!-- xsl:value-of select="w2e:message('OutputElementSimpleType',4)"/ -->
				<xsl:variable name="value" select="xsd:restriction/xsd:enumeration[1]/@value"/>

				<!-- xsl:element name="{@name}"-->
				<xsl:for-each select="@*">
					<xsl:variable name="attrName" select="concat('xsd:',name())"/>
					<xsl:attribute name="{$attrName}" select="."/>
				</xsl:for-each>
				<xsl:for-each select="xsd:restriction/@*">
					<xsl:variable name="attrName" select="concat('xsd:',name())"/>
					<xsl:attribute name="{$attrName}" select="if(contains(.,'xs:')) then replace(.,'xs:','xsd:') else ."/>
				</xsl:for-each>
				<xsl:for-each select="xsd:restriction/*">
					<xsl:if test="contains('xsd:minLength xsd:maxLength xs:minLength xs:maxLength xs:totalDigits xsd:totalDigits xs:fractionDigits xsd:fractionDigits',name())">
						<xsl:variable name="attrName" select="concat('xsd:',local-name())"/>
						<xsl:attribute name="{$attrName}" select="@value"/>
					</xsl:if>
				</xsl:for-each>
				<!-- xsl:value-of select="w2e:message(concat('outputEnum (type): ',$typeName),4)"/ -->
				<xsl:call-template name="outputEnumeration">
					<xsl:with-param name="typeName" select="@name"/>
					<xsl:with-param name="ns" select="@targetNamespace"/>
					<xsl:with-param name="enumValues" select="xsd:restriction/xsd:enumeration"/>
					<xsl:with-param name="mode" select="'-E'"/>
				</xsl:call-template>
				<!-- xsl:value-of select="w2e:type-to-example(concat('&quot;',$value[1],'&quot;'),'type')"/ -->
				<!-- xsl:value-of select="w2e:message(concat('closing fullname=',@name),3)"/ -->
				<!-- /xsl:element -->
			</xsl:when>
			<xsl:when test="count(xsd:restriction) &gt; 0">
				<!-- xsl:element name="{@name}" -->
				<xsl:for-each select="xsd:restriction/@*">
					<xsl:variable name="attrName" select="concat('xsd:',name())"/>
					<xsl:choose>
						<xsl:when test="name() = 'base' and($position &gt; '1')">
							<xsl:attribute name="{concat($attrName,'1')}" select="."/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="{$attrName}" select="."/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:for-each>
				<xsl:variable name="patternMasks"><xsl:for-each select="xsd:restriction/*"><xsl:if test="local-name() = 'pattern'"><xsl:value-of select="@value"/>|||</xsl:if></xsl:for-each></xsl:variable>
				<xsl:for-each select="xsd:restriction/*">
					<xsl:if test="contains($xsdTypeAttributes,name())">
						<xsl:variable name="attrName" select="concat('xsd:',local-name())"/>
						<xsl:attribute name="{$attrName}" select="if(local-name()='pattern') then $patternMasks else @value"/>
					</xsl:if>
				</xsl:for-each>
				<xsl:if test="xsd:restriction/@base"><!-- xsl:value-of select="w2e:type-to-example(xsd:restriction/@base,'type')"/ --></xsl:if>
				<!-- /xsl:element -->
			</xsl:when>
			<xsl:when test="count(xsd:union) &gt; 0">
				<xsl:value-of select="w2e:message(concat('union (type): ',xsd:union/@memberTypes),4)"/>
				<xsl:apply-templates select="xsd:union"/>
			</xsl:when>
			<xsl:otherwise/>
		</xsl:choose>

	</xsl:template>

	<xsl:template match="xsd:union">
		<xsl:param name="typeName"/>
		<xsl:value-of select="w2e:message(concat('found union simpleTypes: ',@memberTypes),4)"/>
		<xsl:variable name="typesList" select="tokenize(@memberTypes,' ')"/>
		<xsl:for-each select="$typesList">
			<xsl:value-of select="w2e:message(concat(' - union type: ',.),4)"/>
			<xsl:variable name="thisOne" select="position()"/>
			<xsl:variable name="suffix" select="if(position()=1) then '' else position() - 1"/>
			<xsl:if test="string-length(.) &gt; 0">
				<xsl:variable name="thisFullTypeName" select="."/>
				<xsl:variable name="thisTypeName" select="if(contains(.,':')) then substring-after(.,':') else ."/>
				<xsl:variable name="simpleFullType" select="$entireXSDcollection//xsd:simpleType[@name=$thisFullTypeName]"/>
				<xsl:variable name="simpleType" select="if(string-length($thisTypeName) &gt; 0) then $entireXSDcollection//xsd:simpleType[@name=$thisTypeName] else ''"/>
				<xsl:choose>
					<xsl:when test="count($simpleFullType) &gt; 0">
						<xsl:value-of select="w2e:message(concat(' - - full type: ',.),4)"/>
						<xsl:apply-templates select="$simpleFullType" mode="simple">
							<xsl:with-param name="position" select="xsd:string(position())"/>
						</xsl:apply-templates>
					</xsl:when>
					<xsl:when test="count($simpleType) &gt; 0">
						<!-- xsl:message>SimpleType : <xsl:value-of select="$thisTypeName"/></xsl:message -->
						<xsl:value-of select="w2e:message(concat(' - - simple type: ',.),4)"/>
						<xsl:choose>
							<xsl:when test="$simpleType[1]//xsd:union and($typeName='')">
								<xsl:message> - warning: recursive simpleType union skipped: <xsl:value-of select="$thisTypeName"/></xsl:message>
							</xsl:when>
							<xsl:otherwise>
								<!-- xsl:if test="$simpleType[1]//xsd:union"><xsl:message><xsl:value-of select="$typeName"/> - <xsl:copy-of select="$simpleType[1]"/></xsl:message></xsl:if -->
								<xsl:value-of select="w2e:message(concat(' - - apply: ',$thisTypeName),4)"/>
								
								<xsl:variable name="unionTypes">
									<xsl:element name="xsd:simpleType">
										<xsl:attribute name="name" select="$simpleType[1]/@name"/>
										<xsl:copy-of select="$simpleType[1]/xsd:annotation"/>
										<xsl:element name="xsd:restriction">
											<xsl:attribute name="base" select="$simpleType[1]//xsd:restriction[1]/@base"/>
											<xsl:variable name="enumerations">
												<items>
												<xsl:for-each select="$simpleType/*/xsd:enumeration/@value"><item value="{.}"/></xsl:for-each>
												</items>
											</xsl:variable>
											<xsl:for-each select="$simpleType/*/xsd:enumeration">
												<xsl:variable name="this" select="@value"/>
												<xsl:variable name="index" select="count($enumerations//item[@value = $this])"/>
												<xsl:variable name="offsets">
													<xsl:choose>
														<xsl:when test="$index=1"/>
														<xsl:otherwise>
															<xsl:for-each select="$enumerations//item">
																<xsl:if test="@value = $this"><offset location="{position()}"/></xsl:if>
															</xsl:for-each>
														</xsl:otherwise>
													</xsl:choose>
												</xsl:variable>
												<xsl:if test="$index &lt; 2 or(position() = $offsets/offset[1]/@location)">
													<!-- xsl:message> - <xsl:value-of select="position()"/> - <xsl:value-of select="$offsets/offset/@location"/></xsl:message -->
													<xsl:element name="xsd:enumeration">
														<xsl:attribute name="value" select="@value"/>
														<xsl:copy-of select="xsd:annotation"/>
													</xsl:element>
												</xsl:if>
											</xsl:for-each>
										</xsl:element>
									</xsl:element>
								</xsl:variable>
								<!-- xsl:message> - union [<xsl:copy-of select="$unionTypes"/>]</xsl:message -->
								<xsl:apply-templates select="$unionTypes" mode="simple">
									<xsl:with-param name="position" select="xsd:string(position())"/>
								</xsl:apply-templates>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:when test="starts-with($thisFullTypeName,'xsd:') or(starts-with($thisFullTypeName,'xs:'))">
						<!-- xsd schema types union -->
						<xsl:value-of select="w2e:message(concat('xsd schema type: ',$thisFullTypeName),4)"/>
						<xsl:attribute name="{concat('xsd:base',$suffix)}" select="$typesList[$thisOne]"/>
					</xsl:when>
					<xsl:otherwise/>
				</xsl:choose>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template match="xsd:group">
		<xsl:param name="ancestors"/>
		<xsl:param name="parent"/>
		<xsl:variable name="refItem" select="@ref|@name"/>
		<xsl:value-of select="w2e:message(concat('xsd:group: ',$refItem),4)"/>
		<xsl:variable name="findRefItem"
			select="if(contains($refItem,':')) then substring-after($refItem,':') else $refItem"/>
		<xsl:choose>
			<xsl:when test="$entireXSDcollection//xsd:group[@name=$findRefItem]">
				<xsl:value-of select="w2e:message(concat('xsd:group matched: ',$findRefItem),4)"/>
				<xsl:variable name="matchedGroup"
					select="$entireXSDcollection//xsd:group[@name=$findRefItem]"/>

				<xsl:comment> - xsd:group <xsl:value-of select="replace($findRefItem,'Group','')"/></xsl:comment>

				<!-- xsl:message> - xsd:group <xsl:value-of select="replace($findRefItem,'Group','')"/></xsl:message -->
				<!-- xsl:message> - <xsl:value-of select="@ref|@name"/> - <xsl:value-of 	select="$matchedGroup[1]/./*/name()"/></xsl:message -->
				<!-- xsl:if test="xsd:sequence">
					<xsl:message> - <xsl:value-of select="@ref|@name"/> - <xsl:value-of
							select="xsd:sequence/*/name()"/></xsl:message>
				</xsl:if -->
				<xsl:variable name="groupDetail">
					<!-- xsl:element name="{replace($findRefItem,'Group','')}" -->
					<xsl:choose>
						<xsl:when test="$matchedGroup[1]/./xsd:sequence/xsd:choice/xsd:group">
							<!-- If there is groups within a choice - then flatten by removing the outer choice -->
							<xsl:message> - flatten choice within group of groups - <xsl:value-of select="$refItem"/></xsl:message>
							<xsl:for-each select="$matchedGroup[1]/./xsd:sequence/*">
								<!-- xsl:message> - <xsl:value-of select="@ref|@name"/> <xsl:value-of select="name()"/></xsl:message -->
								<xsl:choose>
									<xsl:when test="name() = 'xsd:choice' or(name() = 'xs:choice')">
										<!-- xsl:message> - remove choice <xsl:value-of select="name()"/></xsl:message -->
										<xsl:apply-templates select="./*">
											<xsl:with-param name="ancestors" select="$ancestors"/>
											<xsl:with-param name="parent" select="$parent"/>
										</xsl:apply-templates>
									</xsl:when>
									<xsl:otherwise>
										<!-- xsl:message> - process <xsl:value-of select="name()"/></xsl:message -->
										<xsl:apply-templates select=".">
											<xsl:with-param name="ancestors" select="$ancestors"/>
											<xsl:with-param name="parent" select="$parent"/>
										</xsl:apply-templates>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:for-each>
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates select="$matchedGroup[1]/*">
								<xsl:with-param name="ancestors" select="$ancestors"/>
								<xsl:with-param name="parent" select="$parent"/>
							</xsl:apply-templates>
						</xsl:otherwise>
					</xsl:choose>
					<!-- /xsl:element -->
				</xsl:variable>

				<xsl:variable name="revisedGroupDetails">
					<xsl:call-template name="removeExtraElements">
						<xsl:with-param name="groupStructure" select="$groupDetail"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:copy-of select="$revisedGroupDetails"/>
				<!-- xsl:message> - end group <xsl:value-of select="replace($findRefItem,'Group','')"/></xsl:message -->
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="w2e:message(concat('xsd:group definition missing: ',$findRefItem),1)"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="resolveTypePath">
		<xsl:param name="MatchName"/>
		<xsl:param name="MatchType"/>
		<xsl:param name="schemaContent"/>
		<xsl:choose>
			<xsl:when test="$MatchType='substitutionGroup'">
				<xsl:choose>
					<xsl:when
						test="$schemaContent//xsd:element[@name=$MatchName]/@substitutionGroup">
						<xsl:value-of select="concat($MatchName,'/')"/>
						<xsl:call-template name="resolveTypePath">
							<xsl:with-param name="MatchName"
								select="$schemaContent//xsd:element[@name=$MatchName]/@substitutionGroup"/>
							<xsl:with-param name="MatchType" select="$MatchType"/>
							<xsl:with-param name="schemaContent" select="$schemaContent"/>
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$MatchName"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="$MatchType='base'">
				<xsl:value-of select="$MatchName"/>
			</xsl:when>
			<xsl:when test="$MatchType='type'">
				<xsl:value-of select="$MatchName"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$MatchName"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="*" mode="annoAttrLines">
		<xsl:apply-templates select="*" mode="annoAttrLines"/>
		<xsl:apply-templates select="@*" mode="annoAttrLines"/>
	</xsl:template>
	<xsl:template match="@xsd:attr-annotation" mode="annoAttrLines">
		<xsl:variable name="baseXPath"
			select="if(name(../../.)='' and($elementNameValue='*')) then concat('//AllRootLevelTypeDefinitions/',name(../.)) else concat('/',name(../../.),'/',name(../.))"/>
		<xsl:variable name="xpathChk"
			select="if(starts-with($baseXPath,'//')) then $baseXPath else concat('/',$baseXPath)"/>
		<xsl:variable name="xpath" select="replace($xpathChk,'xsd:choice/','')"/>
		<!-- xsl:variable name="chkExclude" select="concat('|',substring-before(substring-after($xpath,'//'),':'),':')"></xsl:variable -->
		<camed:annotation item="{$xpath}">
			<xsl:choose>
				<xsl:when test=".='recursive'"><camed:documentation type="Recursive">true</camed:documentation></xsl:when>
				<xsl:otherwise><camed:documentation type="attributeGroup"><xsl:value-of select="."/></camed:documentation></xsl:otherwise>
			</xsl:choose>
		</camed:annotation>
	</xsl:template>
	<xsl:template match="@*" mode="annoAttrLines"/>

	<xsl:template name="outputLookupRule">
		<xsl:param name="rule"/>
		<xsl:variable name="chkEnumeration" select="tokenize(@action,'\|')"/>
		<xsl:choose>
			<xsl:when test="count($chkEnumeration) &gt; 3 or $useLookups='all'">
				<!-- check for more than 3 enumerations or all mode -->
				<xsl:variable name="actionStub"
					select="replace(replace(replace(substring-before($rule/@action,','),'restrictValues','lookup'),'xsd:choice/',''),'xs:choice/','')"/>
				<xsl:variable name="actionXpath" select="substring-after($actionStub,'(')"/>
				<xsl:variable name="actionTarget" select="tokenize($actionXpath,'/')"/>
				<xsl:variable name="actionTargets" select="count($actionTarget)"/>
				<xsl:element name="as:constraint">
					<xsl:if test="$rule/@condition">
						<xsl:attribute name="condition" select="$rule/@condition"/>
					</xsl:if>
					<xsl:attribute name="action"><xsl:value-of select="$actionStub"/>,<xsl:value-of select="replace(replace(concat($actionTarget[$actionTargets],'-',$actionTarget[$actionTargets - 1]),'@','attr-'),':','-')"/>)</xsl:attribute>
					<xsl:copy-of select="$rule/as:annotation"/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy-of select="$rule"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- write unique restrictionValues out to lookup files -->
	<xsl:template name="writeLookupsIncludes">
		<xsl:param name="baseRules"/>
		<xsl:variable name="outputPath"/>
		<xsl:variable name="lookups">
			<xsl:for-each select="$baseRules//as:constraint">
				<xsl:if test="starts-with(@action,'restrictValues')">
					<xsl:variable name="actionStub"
						select="replace(replace(replace(substring-before(@action,','),'restrictValues','lookup'),'xsd:choice/',''),'xs:choice/','')"/>
					<xsl:variable name="actionXpath" select="substring-after($actionStub,'(')"/>
					<xsl:variable name="actionTarget" select="tokenize($actionXpath,'/')"/>
					<xsl:variable name="actionTargets" select="count($actionTarget)"/>
					<lookup
						name="{replace(replace(concat($actionTarget[position()=$actionTargets],'-',$actionTarget[position()=$actionTargets - 1]),'@','attr-'),':','-')}"
						values="{substring-after(@action,',')}"/>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>
		<xsl:variable name="uniqueLookups" select="distinct-values($lookups/lookup/@name)"/>

		<!-- Now write the list of lookups -->
		<xsl:variable name="uniqueLookups" select="distinct-values($lookups/lookup/@name)"/>
		<xsl:message>writing codelist includes...</xsl:message>
		<!-- This is needed because bug in editor prevents writing directly into the template - allocateExtension not supporting .LookupLists type -->
		<!-- xsl:result-document exclude-result-prefixes="xsd" format="lookupContent" indent="yes" href="lookups-include.xml" -->
		<as:Extension xmlns:as="http://www.oasis-open.org/committees/cam"
			name="uk.org.jcam.camed.extensions.LookupLists">
			<xsl:for-each select="$uniqueLookups">
				<xsl:variable name="this" select="."/>
				<xsl:for-each select="$lookups/lookup[@name=$this][1]">
					<xsl:variable name="lookupFile"><xsl:if test="string-length($outputPath) &gt; 0"><xsl:if test="not(starts-with($outputPath,'file:/'))">file:/</xsl:if><xsl:value-of select="$outputPath"/><xsl:if test="not(ends-with($outputPath,'/'))">/</xsl:if></xsl:if>lookup-<xsl:value-of select="@name"/>.ccl</xsl:variable>
					<xsl:variable name="ruleEnumerations" select="tokenize(./@values,'\|')"/>
					<xsl:variable name="totalCodes" select="count($ruleEnumerations)"/>
					<xsl:if test="$useLookups='all' or($totalCodes &gt; 3)">
						<!-- xsl:message> - <xsl:value-of select="$lookupFile"/> (<xsl:value-of select="$totalCodes"/>)</xsl:message -->
						<as:include><xsl:value-of select="concat($lookupsFilePath,$lookupFile)"/></as:include>
					</xsl:if>
				</xsl:for-each>
			</xsl:for-each>
		</as:Extension>
		<!-- /xsl:result-document -->
	</xsl:template>

	<!-- write unique restrictionValues out to lookup files -->
	<xsl:template name="writeLookups">
		<xsl:param name="baseRules"/>
		<xsl:variable name="outputPath"/>
		<xsl:variable name="lookups">
			<xsl:for-each select="$baseRules//as:constraint">
				<xsl:if test="starts-with(@action,'restrictValues')">
					<xsl:variable name="actionStub"
						select="replace(replace(replace(substring-before(@action,','),'restrictValues','lookup'),'xsd:choice/',''),'xs:choice/','')"/>
					<xsl:variable name="actionXpath" select="substring-after($actionStub,'(')"/>
					<xsl:variable name="actionTarget" select="tokenize($actionXpath,'/')"/>
					<xsl:variable name="actionTargets" select="count($actionTarget)"/>
					<lookup
						name="{replace(replace(concat($actionTarget[position()=$actionTargets],'-',$actionTarget[position()=$actionTargets - 1]),'@','attr-'),':','-')}"
						values="{substring-after(@action,',')}"/>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>
		<xsl:variable name="uniqueLookups" select="distinct-values($lookups/lookup/@name)"/>
		<!-- xsl:message> writing max of <xsl:value-of select="count($uniqueLookups)"/> codelists:</xsl:message -->
		<xsl:for-each select="$uniqueLookups">
			<xsl:variable name="this" select="."/>
			<xsl:for-each select="$lookups/lookup[@name=$this][1]">
				<xsl:variable name="lookupFile"><xsl:if test="string-length($outputPath) &gt; 0"><xsl:if test="not(starts-with($outputPath,'file:/'))">file:/</xsl:if><xsl:value-of select="$outputPath"/><xsl:if test="not(ends-with($outputPath,'/'))">/</xsl:if></xsl:if>lookup-<xsl:value-of select="@name"/>.ccl</xsl:variable>
				<xsl:variable name="ruleEnumerations" select="tokenize(./@values,'\|')"/>
				<xsl:variable name="totalCodes" select="count($ruleEnumerations)"/>
				<xsl:if test="$useLookups='all' or($totalCodes &gt; 3)">
					<xsl:message> - <xsl:value-of select="concat($lookupsFilePath,$lookupFile)"/>(<xsl:value-of select="$totalCodes"/>)</xsl:message>
					<xsl:result-document exclude-result-prefixes="xsd" format="lookupContent"
						indent="yes" href="{concat($lookupsFilePath,$lookupFile)}">
						<look:LookupList xmlns:look="http://jcam.org.uk/LookupLists">
							<xsl:variable name="lookupName"><xsl:value-of select="$this"/></xsl:variable>
							<xsl:attribute name="name" select="$lookupName"/>
							<xsl:for-each select="$ruleEnumerations">
								<xsl:variable name="codeValueTmp" select="replace(.,'''','')"/>
								<xsl:variable name="codeValue"
									select="if(ends-with($codeValueTmp,' ') or(ends-with($codeValueTmp,')'))) then substring($codeValueTmp,1,string-length($codeValueTmp) - 1) else $codeValueTmp"/>
								<look:item>
									<xsl:attribute name="name" select="$codeValue"/>
									<xsl:value-of select="$codeValue"/>
								</look:item>
							</xsl:for-each>
						</look:LookupList>
					</xsl:result-document>
				</xsl:if>
			</xsl:for-each>
		</xsl:for-each>

		<!-- Now write file that contains the list of lookups -->
		<xsl:variable name="uniqueLookups" select="distinct-values($lookups/lookup/@name)"/>
		<xsl:message>output codelist includes:</xsl:message>
		<!-- This is needed because bug in editor prevents writing directly into the template - allocateExtension not supporting .LookupLists type -->
		<xsl:variable name="lookupFilesList"><xsl:if test="string-length($outputPath) &gt; 0"><xsl:if test="not(starts-with($outputPath,'file:/'))">file:/</xsl:if><xsl:value-of select="$outputPath"/><xsl:if test="not(ends-with($outputPath,'/'))">/</xsl:if></xsl:if>lookups-include.cclr</xsl:variable>
		<xsl:result-document exclude-result-prefixes="xsd" format="lookupContent" indent="yes"
			href="{concat($lookupsFilePath,$lookupFilesList)}">
			<as:Extension xmlns:as="http://www.oasis-open.org/committees/cam"
				name="uk.org.jcam.camed.extensions.LookupLists">
				<xsl:for-each select="$uniqueLookups">
					<xsl:variable name="this" select="."/>
					<xsl:for-each select="$lookups/lookup[@name=$this][1]">
						<xsl:variable name="lookupFile"><xsl:if test="string-length($outputPath) &gt; 0"><xsl:if test="not(starts-with($outputPath,'file:/'))">file:/</xsl:if><xsl:value-of select="$outputPath"/><xsl:if test="not(ends-with($outputPath,'/'))">/</xsl:if></xsl:if>lookup-<xsl:value-of select="@name"/>.ccl</xsl:variable>
						<xsl:variable name="ruleEnumerations" select="tokenize(./@values,'\|')"/>
						<xsl:variable name="totalCodes" select="count($ruleEnumerations)"/>
						<xsl:if test="$useLookups='all' or($totalCodes &gt; 3)">
							<xsl:message> - <xsl:value-of select="$lookupFile"/> (<xsl:value-of select="$totalCodes"/>)</xsl:message>
							<as:include><xsl:value-of select="$lookupFile"/></as:include>
						</xsl:if>
					</xsl:for-each>
				</xsl:for-each>
			</as:Extension>
		</xsl:result-document>
	</xsl:template>

	<xsl:template name="removeExtraElements">
		<!-- used to remove extra repeating elements from inside xsd:group collections -->
		<xsl:param name="groupStructure"/>
		<xsl:variable name="groupStructureNoChoice">
			<xsl:apply-templates select="$groupStructure/*" mode="removeXSDchoice">
				<xsl:with-param name="makeOptional"/>
			</xsl:apply-templates>
		</xsl:variable>
		<xsl:variable name="elements" select="$groupStructureNoChoice/*/name()"/>
		<!-- xsl:message> - group elements [<xsl:value-of select="$elements"/>]</xsl:message -->
		<xsl:variable name="duplicates"><xsl:for-each select="distinct-values($elements)">	<xsl:variable name="this" select="."/><xsl:if test="count($elements[. = $this]) &gt; 1"><xsl:value-of select="$this"/>|</xsl:if></xsl:for-each></xsl:variable>
		<xsl:choose>
			<xsl:when test="contains($duplicates,'|')">
				<!-- xsl:message> - duplicates: <xsl:value-of select="$duplicates"/></xsl:message -->
				<xsl:apply-templates select="$groupStructureNoChoice" mode="removeDups">
					<xsl:with-param name="dups" select="concat('|',$duplicates)"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy-of select="$groupStructureNoChoice"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="*" mode="removeXSDchoice">
		<xsl:param name="makeOptional"/>
		<xsl:choose>
			<xsl:when test="name(.) = 'xsd:choice'">
				<!-- xsl:message> [<xsl:value-of select="./*/name()"/>]</xsl:message -->
				<xsd:choice>
					<xsl:apply-templates select="./*" mode="removeXSDchoice">
						<xsl:with-param name="makeOptional">true</xsl:with-param>
					</xsl:apply-templates>
				</xsd:choice>
			</xsl:when>
			<xsl:when test="name(.) = 'xs:choice'">
				<!-- xsl:message> [<xsl:value-of select="./*/name()"/>]</xsl:message -->
				<xsd:choice>
					<xsl:apply-templates select="./*" mode="removeXSDchoice">
						<xsl:with-param name="makeOptional">true</xsl:with-param>
					</xsl:apply-templates>
				</xsd:choice>
			</xsl:when>
			<xsl:when test="$makeOptional='true'">
				<xsl:variable name="element-to-process" select="name(.)"/>
				<xsl:choose>
					<xsl:when test="contains($element-to-process,':')">
						<xsl:variable name="ns-prefix" select="substring-before($element-to-process,':')"/>
						<xsl:variable name="ns-uri"><xsl:for-each select="$namespaces"><xsl:if test="name(.) = $ns-prefix"><xsl:value-of select="namespace-uri-for-prefix(name(.),/xs:schema)"/></xsl:if></xsl:for-each></xsl:variable>
						<xsl:element name="{$element-to-process}" namespace="{$ns-uri}">
							<xsl:attribute name="xsd:minOccurs">0</xsl:attribute>
							<!-- Make each choice element optional -->
							<xsl:copy-of select="@*"/>
							<xsl:copy-of select="text()"/>
							<xsl:copy-of select="./*"/>
						</xsl:element>
					</xsl:when>
					<xsl:otherwise>
						<xsl:element name="{$element-to-process}">
							<xsl:attribute name="xsd:minOccurs">0</xsl:attribute>
							<!-- Make each choice element optional -->
							<xsl:copy-of select="@*"/>
							<xsl:copy-of select="text()"/>
							<xsl:copy-of select="./*"/>
						</xsl:element>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy-of select="."/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="*" mode="removeDups">
		<xsl:param name="dups"/>
		<xsl:variable name="elementList" select="reverse(//name())"/>
		<xsl:variable name="this" select="name(.)"/>
		<xsl:variable name="thisPos" select="position()"/>
		<xsl:choose>
			<xsl:when test="contains($dups,concat('|',$this,'|'))">
				<!-- xsl:message> - - dup - <xsl:value-of select="$this"/> - <xsl:value-of select="$elementList[. = $this][1]"/></xsl:message -->
				<xsl:variable name="findOffset"><xsl:for-each select="$elementList"><xsl:value-of select="if(. = $this) then '0|' else '1|'"/></xsl:for-each></xsl:variable>
				<xsl:variable name="checkOffset"
					select="(count($elementList) - 1) div 2 - count(tokenize(substring-before($findOffset,'0'),'\|')) div 2"/>
				<!-- xsl:message> - - <xsl:value-of select="$thisPos"/> = <xsl:value-of select="count(tokenize(substring-before($findOffset,'0'),'\|')) div 2"/> - <xsl:value-of select="(count($elementList) - 1) div 2"/> - <xsl:value-of select="$elementList"/></xsl:message -->
				<xsl:if test="$thisPos &gt; $checkOffset"><xsl:copy-of select="."/></xsl:if>
			</xsl:when>
			<xsl:otherwise><xsl:copy-of select="."/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="*" mode="groupDetails">
		<xsl:param name="offset"/>
		<xsl:param name="schemaDetails"/>

		<xsl:for-each select="xsd:group"><xsl:value-of select="$offset"/><xsl:value-of select="./@ref | @name"/><xsl:value-of select="$newline"/><xsl:variable name="this" select="./@ref | @name"/>
			<xsl:variable name="findThis" select="if(contains($this,':')) then substring-after($this,':') else $this"/>
			<xsd:group><xsl:value-of select="$schemaDetails//xsd:group[@name = $findThis]/xsd:sequence/xsd:element/@name"/><xsl:value-of select="$newline"/></xsd:group>
		</xsl:for-each>

		<xsl:if test="xsd:choice">
			<xsl:value-of select="concat($offset,'choice',$newline)"/>
			<xsl:choose>
				<xsl:when test="xsd:choice/xsd:group">
					<!-- if choice contains group then remove choice -->
					<xsl:apply-templates select="xsd:choice/." mode="groupDetails">
						<xsl:with-param name="offset" select="concat($offset,'- ')"/>
						<xsl:with-param name="schemaDetails" select="$schemaDetails"/>
					</xsl:apply-templates>
				</xsl:when>
				<xsl:otherwise>
					<!-- retain choice if does not contain group components -->
					<xsd:choice>
						<xsl:apply-templates select="xsd:choice/." mode="groupDetails">
							<xsl:with-param name="offset" select="concat($offset,'- ')"/>
							<xsl:with-param name="schemaDetails" select="$schemaDetails"/>
						</xsl:apply-templates>
					</xsd:choice>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>

</xsl:stylesheet>
